/*
* ==============================================================================
*  Name        : MemoryManager.h
*  Part of     : WEBKIT / MemMan
*  Interface   : 
*  Description : 
*  Version     : %Version%
*
* ==============================================================================
*/
#ifndef __MEMORYMANAGER_H__
#define __MEMORYMANAGER_H__

//  INCLUDES
#include "MemTypes.h"
// CONSTANTS

// MACROS
#define OOM_PRE_CHECK(s, c)  { if( MemoryManager::PreCheck( s, c ) ) { 
#define OOM_POST_CHECK_FAILED(__r)    } } { __r;}

// DATA TYPES
enum ECMMPolicyType
{
	CMM_POLICY_NO_DISPLAY_IMAGE = 0x0001,
	CMM_POLICY_NO_JAVASCRIPT   = 0x0002,
	CMM_POLICY_NO_MEM_CACHE     = 0x0004
};

enum ECMMCheckResultType
{
	CMM_CHECKRESULT_NO_CMM                      = 0x0000,   // no oom
	CMM_CHECKRESULT_CHECK_CMM                   = 0x0001,   // last memory check failed
	CMM_CHECKRESULT_USER_ALLOC_CMM              = 0x0002,   // system heap allocation failed
	CMM_CHECKRESULT_RESCUE_CMM                  = 0x0004,   // rescue buffer allocation failed
	CMM_CHECKRESULT_COLLECTING                  = 0x0010,   // memory manager is collecting memory
	CMM_CHECKRESULT_STOPPING                    = 0x0020,   // memory manager is stopping operations 
	CMM_CHECKRESULT_COLLECTING_RESOURCES        = 0x0040    // memory manager is collecting dead and if required live resources
};

enum ECMMPriorityType
{
	CMM_PRIORITY_LOW    = 0,
    CMM_PRIORITY_MIDDLE,
	CMM_PRIORITY_HIGH,
	CMM_PRIORITY_CRITICAL
};

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS
class CAllocator;
class FunctionLogger;

// CLASS DECLARATION

/**
*
*  @lib memman.lib
*  @since 1.0
*/
class IMemoryCollector
{
public:
	/**
	* Collect unreferenced resources, this function is called when allocation from 
	* System heap goes beyond a threshold
	* @since 1.0
	* @return amount of memory collected
	*/
    virtual guint32 CollectDeadResources() = 0;

	/**
	* Collect images from current page, this function is called when CollectDeadResources 
	* is not able to collect the required memory
	* @since 1.0
	* @param amount of memory needs to be collected
	* @return amount of memory collected
	*/
    virtual guint32 CollectLiveImageResources(guint32 aRequiredSize) = 0;

    /**
	* Check if the images from current page are of aRequiredSize, this function is called 
    * before calling CollectLiveImageResources
	* @since 1.0
	* @param amount of memory needs to be checked against live images
	* @return true if the aRequiredSize of memory is occupied by images from current page
	*/
    virtual bool ShouldCollectLiveImages(const guint32 aRequiredSize) = 0;
	/**
	* restore the entity controlled memory collector when there is enough memory
	* System heap fails
	* @since 1.0
	* @param
	* @return
	*/
	virtual void Restore() = 0;
	
	/**
	* Priority of this collector, 0 - lowest, 10 - highest;
	* the lower the priority, the earlier this collector is executed.
	* @since 1.0
	* @param
	* @return
	*/
	virtual ECMMPriorityType Priority() = 0;

	virtual void SetCapacity(unsigned minDeadBytes, unsigned maxDeadBytes, unsigned totalBytes) = 0;
};

/**
*
*  @lib memman.lib
*  @since 1.0
*/
class ICMMNotifier
{
public:
	/**
	* client provided notifying utility function, called by the allocator 
	* to inform the user
	* @since 1.0
	* @param
	* @return
	*/
	virtual gint32 Notify() = 0;
};

/**
*  A class handles all memory operations and CMM issues
*  @lib memman.lib
*  @since 1.0
*/
class MemoryManager
{
public:
	/**
	* allocate a memory buffer
	* @since 1.0
	* @param aSize bytes of memory to be allocated
	* @return pointer to the allocated memory
	*/
	static void* Alloc(guint32 aSize);

	/**
	* allocate a memory buffer and fill it with 0
	* @since 1.0
	* @param aElements number of elements to be allocated
	* @param aSize size of each element
	* @return pointer to the allocated memory
	*/
	static void* Calloc(guint32 aElements, guint32 aSize);

	/**
	* re-allocate a memory buffer and copy the content of original buffer
	* to the new buffer
	* @since 1.0
	* @param aPtr pointer to the original buffer
	* @param aSize size of the new buffer
	* @return pointer to the new buffer
	*/
	static void* ReAlloc(void* aPtr, guint32 aSize);

	/**
	* free the memory
	* @since 1.0
	* @param aPtr pointer to the memory to be freed
	* @param 
	* @return 
	*/
	static void Free(void* aPtr);

   	/**
	* Returns the available memory in bytes
	* @since 1.0
	* @param 
	* @return available memory in bytes
	*/
#ifndef NDEBUG
	static unsigned AvailableMemory();
#endif // NDEBUG

    /**
	* register a memory collector, memory manager doesn't own this collector
	* @since 1.0
	* @param aCollector the collector to be registered
	* @param 
	* @return 
	*/
	static void AddCollector(IMemoryCollector* aCollector);

	/**
	* set the CMM notifier, memory manager doesn't own this notifier
	* @since 1.0
	* @param aNotifier the notifier
	* @param 
	* @return 
	*/
	static void SetNotifier(ICMMNotifier* aNotifier);

	/**
	* check if the memory manager is able to reserve enough memory for the coming operation.
	* @since 1.0
	* @param iRequiredSize Size to be allocated
    * @param bContiguous True if the required memory needs to be contiguous
	* @return result of prechecking, ETrue = successful
	*/
	static gbool PreCheck(guint iRequiredSize, gbool bContiguous);

	/**
	* check if the memory manager is able to reserve enough memory without going OOM.
	* @since 1.0
	* @param iRequiredSize Size to be allocated
	* @return result of prechecking, ETrue = successful
	*/
    static gbool PreCheckWithThreshold(guint iRequiredSize);

	/**
	* Get the status of memory manager, specific state could be retrieve by AND (&) operator
	* with ECMMCheckResultType enum
	* @since 1.0
	* @param 
	* @param
	* @return the status of memory during this operation.
	*/
	static guint32 Status();

	/**
	* Trigger an CMM event, this function is only for debugging purpose
	* @since 1.0
	* @param aType defines where memory allocation fails
	* @param
	* @return
	*/
	static void SetStatus(ECMMCheckResultType aType = CMM_CHECKRESULT_NO_CMM);

	/**
	* Returns true if OOM is sent
	* @since 1.0
	* @param
	* @param
	* @return
	*/
    static gbool IsOOMSent();

    static void SetInOOMState();
        
    static void ResetOOMState();
};

#endif// __MEMORYMANAGER_H__
