#ifndef _SYS_ALLOCATOR_H_
#define _SYS_ALLOCATOR_H_

#include "MemTypes.h"
#include "SysAssert.h"

//////////////////////// Target specific block starts /////////////////////////
#define PRIMARY_CHUNK_SIZE			(2 * 1024 * 1024U)
#define SECONDARY_CHUNK_SIZE		(2 * 1024 * 1024U)

#define	MAX_BLOCKS					(6U)

const unsigned kChunkSizes[] = { PRIMARY_CHUNK_SIZE,
								SECONDARY_CHUNK_SIZE,
								SECONDARY_CHUNK_SIZE,
								SECONDARY_CHUNK_SIZE,
								SECONDARY_CHUNK_SIZE,
								SECONDARY_CHUNK_SIZE };

//////////////////////// Target specific block ends ///////////////////////////

//////////////////////// Pre-processor defines ////////////////////////////////
#define MM_NEVER_GET_HERE()			SysAssert(0);
#define MM_DEFAULT_GRANULARITY		(512 * 1024U) //Should be same as dlmalloc

// Collect only when we are not able to allocate two segments
#define MM_COLLECT_THRESHOLD        (3 * MM_DEFAULT_GRANULARITY)

// Throw OOM if we are not able to allocate a single segment
#define MM_OOM_THRESHOLD            (2 * MM_DEFAULT_GRANULARITY)

#define MM_BAD_ALLOC				(-1)

// Returns N high bits e.g. N = 3 will return binary 111 i.e. 7
#define MM_LSB_N_ONES(N)			((0x1U << (N)) - 1)

// Returns an unsigned int whose N'th bit from LHS is set
#define MM_LSB_NTH_ONE(N)			(0x1U << (N))

// Returns an unsigned int with X bits set at Y location from LHS 
#define MM_ALLOC_MAP(X, Y)			((MM_LSB_N_ONES(X)) << (Y))

// Returns true if Y can be allocated in the X allocation map
#define MM_CAN_ALLOCATE(X, Y)		(((~X) & (Y)) == (Y))


class ICMMNotifier;
class IMemoryCollector;
enum ECMMCheckResultType;

///////////////////////////////////////////////////////////////////////////////
//  Class MemChunk
///////////////////////////////////////////////////////////////////////////////
class MemChunk
{
public:
    // Constructor
    MemChunk() : m_chunk(0), m_maxSegments(0), m_allocMap(0), m_isContiguous(0) {}

    // Public methods
	/**
	* @brief Allocates memory to the chunk with the given size
	* @version 1.0
	* @param[in] aSize - Amount of memory to be allocated
	* @return void
	* @warning Asserts when called if the chunk is already created
	**/
    void		create(const unsigned aSize);


	/**
	* @brief Deallocates the memory chunk
	* @version 1.0
	* @return void
	* @warning Asserts if the chunk is not allocated
	**/
    void		destroy();


	/**
	* @brief Returns the memory pointer if aSize bytes can be allocated
	* @version 1.0
	* @param[in] aSize - Number of bytes to be allocated from the chunk
	* @return void* - A valid pointer if aSize bytes are available in the chunk
	*					NULL otherwise
	* @warning Asserts if aSize is not a multiple of MM_DEFAULT_GRANULARITY
	**/
    void*		allocate(const unsigned aSize);


	/**
	* @brief Marks the given memory as unused
	* @version 1.0
	* @param[in] aPtr - Pointer whose memory needs to be deallocated
    * @param[in] aSize - Number of bytes to be deallocated from the chunk
	* @return void
	* @warning Asserts if aPtr is not allocated from the memory chunk
	**/
    void        deallocate(const void* aPtr, const unsigned aSize);


	/**
	* @brief Returns the number of bytes available in the memory chunk
	* @version 1.0
	* @return unsigned - Number of bytes available in the memory chunk
	**/
	unsigned	availableMemory(); 


	/**
	* @brief Returns true if memory is allocated for the memory chunk
	* @version 1.0
	* @return bool - True if memory is allocated to the chunk, false otherwise
	**/
	bool		isInitialized() { return !!m_chunk; }


	/**
	* @brief Returns true if the memory chunk is full
	* @version 1.0
	* @return bool - True if the memory chunk is initialized and is full,
						false otherwise
	**/
	bool		isFull() 
	{ 
		return !m_chunk || (m_allocMap == MM_LSB_N_ONES(m_maxSegments));
	}


	/**
	* @brief Returns true if the memory chunk has no allocations
	* @version 1.0
	* @return bool - True if the memory chunk has no allocations or 
						the chunk is not initialized, false otherwise
	**/
	bool		isEmpty() 
	{
		return !m_chunk || (0 == m_allocMap); 
	}


	/**
	* @brief Returns if the pointer is allocated from the memory chunk
	* @version 1.0
	* @param[in] aPtr - Pointer whose existence in the memory chunk needs to be checked
	* @return bool - True if aPtr was allocated from the memory chunk, false otherwise
	**/
	bool		isValidPtr(const void* aPtr) 
	{
		return isInitialized() && (m_chunk <= aPtr && 
			aPtr <= ((char*)m_chunk + ((m_maxSegments - 1) * MM_DEFAULT_GRANULARITY)));
	}


private: // Private methods

	/**
	* @brief Allocates the specified number of segments in the memory block
	* @version 1.0
	* @param[in] aNumOfSeg - Number of segments to be allocated
	* @return int - An index of the free segment(s), -1 if the allocation fails
	**/
	int			allocateSegment(const unsigned aNumOfSeg);

	
	/**
	* @brief Deallocates segment(s) from the specified index
	* @version 1.0
	* @param[in] aSegNum - Index of the segment(s) to be deallocated
    * @param[in] aNumOfSeg - Number of segment(s) to be deallocated
	* @return void
	* @warning Asserts if the segment at index aSegNum is unallocated
	**/
	void        deallocateSegment(const unsigned aSegNum, const unsigned aNumOfSeg);

	
	/**
	* @brief Sets aNumOfSeg segment(s) at aSegNum index as allocated
	* @version 1.0
	* @param[in] aSegNum - The index of the segment(s) that need(s) to be allocated
	* @param[in] aNumOfSeg - The number of segment(s) that need(s) to be allocated
	* @return void
	* @warning Asserts if any of the specified segment(s) is already allocated
	**/
	void		setAllocationMap(const unsigned aSegNum, 
									const unsigned aNumOfSeg)
	{
		SysAssert(!(m_allocMap & MM_ALLOC_MAP(aNumOfSeg, aSegNum)));
		
		m_allocMap |= MM_ALLOC_MAP(aNumOfSeg, aSegNum);
		m_isContiguous |= MM_ALLOC_MAP((aNumOfSeg - 1), aSegNum);
	}


	/**
	* @brief Sets aNumOfSeg segment(s) at aSegNum index as free
	* @version 1.0
	* @param[in] aSegNum - The index of the segment(s) that need(s) to be unallocated
	* @param[in] aNumOfSeg - The number of segment(s) that need(s) to be unallocated
	* @return void
	* @warning Asserts if any of the specified segment(s) is not allocated
	**/
	void		unsetAllocationMap(const unsigned aSegNum, 
									const unsigned aNumOfSeg)
	{
		const unsigned toAllocate = MM_ALLOC_MAP(aNumOfSeg, aSegNum);
		SysAssert(0 == MM_CAN_ALLOCATE(m_allocMap, toAllocate));

		m_allocMap &= ~(MM_ALLOC_MAP(aNumOfSeg, aSegNum));
		m_isContiguous &= ~(MM_ALLOC_MAP((aNumOfSeg - 1), aSegNum));

        // Update the contiguous bit if we deallocate a part of a contiguous segment
        if(0 < aSegNum)
        {
            m_isContiguous &= ~(MM_LSB_NTH_ONE(aSegNum - 1));
        }
	}


private: // Private data members
	// Chunk allocated from the system memory
    void*               m_chunk;
	// The maximum number of segments this memory chunk can hold
    unsigned            m_maxSegments;
	// The bitmap for the allocation of segments
    unsigned			m_allocMap;
	// The bitmap to be set if the segments are contiguous
    unsigned			m_isContiguous;
};



///////////////////////////////////////////////////////////////////////////////
//  Class SysAllocator
///////////////////////////////////////////////////////////////////////////////
class SysAllocator
{
public: // Public methods

	/**
	* @brief Method to initialize the state of SysAllocator
	* @version 1.0
	* @return void
	* @warning This method is currently empty. Initialize class members if any.
	**/
	void		init();

	
	/**
	* @brief Destroys the memory chunks and release the system memory
	* @version 1.0
	* @return void
	**/
	void		deinit();

	
	/**
	* @brief Allocates memory of aSize bytes
	* @version 1.0
	* @param[in] aSize - Number of bytes to be allocated
	* @param[in] aOOMCheck - True if the OOM check is to be done. Default is true.
	* @see Implementation of availableMemory(aSize, bContiguous)
	* @return void* - Returns a valid pointer on successful allocation, NULL otherwise
	* @warning Asserts if aOOMCheck is true and memory allocation fails
	**/
	void*		allocate(const unsigned aSize, const bool aOOMCheck = true);

	
	/**
	* @brief Releases the aPtr, release memory to system if the chunk is empty
	* @version 1.0
	* @param[in] aPtr - Pointer to be released from the memory chunk
    * @param[in] aSize - Number of bytes to be deallocated from the chunk
	* @param[in] aDestroyChunk - True if the chunk is to be released to system.
					Default is true.
	* @see Implementation of availableMemory(aSize, bContiguous)
	* @return void
	* @warning Asserts if aPtr is not a valid allocation
	**/
	void        deallocate(const void* aPtr, const unsigned aSize, const bool aDestroyChunk = true);

	
	/**
	* @brief Returns the number of bytes available with the allocator w.r.t. aSize
	* @version 1.0
	* @param[in] aSize - Number of bytes required
	* @param[in] aContiguous - True if the memory needs to be contiguous
	* @return unsigned - The number of bytes available w.r.t. aSize
	**/
	unsigned    availableMemory(const unsigned aSize, const bool aContiguous);

#ifndef NDEBUG
	/**
	* @brief Returns the total number of bytes the allocator can allocate
	* @version 1.0
	* @return unsigned - The number of bytes the allocator can allocate
    * @warning This is a debug only API
	**/
	unsigned    availableMemory();
#endif // NDEBUG
    
    /**
	* @brief Sets the Out Of Memory Notifier
	* @version 1.0
    * @param[in] aNotifier - Out Of Memory Notifier to be set
	* @return void
	**/
    void        setNotifier(ICMMNotifier* aNotifier) { m_notifier = aNotifier; }


    /**
	* @brief Sets the Memory Collector
	* @version 1.0
    * @param[in] aCollector - Memory Collector to be set
	* @return void
	**/
    void        addCollector(IMemoryCollector* aCollector) 
    { 
        m_collector = aCollector; 
    }


    /**
	* @brief Returns the ECMMCheckResultType if the OOM condition occurs
	* @version 1.0
	* @return bool - The ECMMCheckResultType which reflects the allocator state
	**/
	unsigned    status() { return m_memStatus; }


    /**
	* @brief Sets the allocator state
	* @version 1.0
	* @param[in] aStatus - The state to be set
	* @return void
	**/
    void        setStatus(ECMMCheckResultType aStatus);


    /**
	* @brief Sets the allocator in Out Of Memory state
	* @version 1.0
	* @return void
	**/
    void        setAllocatorInOOMState();
    
    
	/**
	* @brief Resets the Out of Memory state of the allocator
	* @version 1.0
	* @return void
	**/
    void        resetOOMState();


	/**
	* @brief Returns true if the Out of Memory is sent
	* @version 1.0
	* @return True if the Out of Memory is sent
	**/
    bool        isOOMSent() { return m_isOOMMessageSent; }


    /**
	* @brief Collects the dead resources from in-memory cache
	* @version 1.0
	* @return unsigned - Returns the size in bytes of memory collected
	**/
    unsigned    collectDeadResources();


    /**
	* @brief Collects the live image resources from in-memory cache
	* @version 1.0
	* @return unsigned - Returns the size in bytes of memory collected
	**/
    unsigned    collectLiveImageResources(unsigned aRequiredSize);

private: // Private methods

	/**
	* @brief Sets the system into OOM state
	* @version 1.0
	* @return void
	**/
	void		checkIfOOM();


private: // Private data members
	// Array of memory chunks
	MemChunk			m_chunks[MAX_BLOCKS];
    //
    unsigned            m_memStatus;
    //
    bool                m_isOOMMessageSent;
    //
    ICMMNotifier*       m_notifier;
    //
    IMemoryCollector*   m_collector;
};
#endif // _SYS_ALLOCATOR_H_

// End of file
