/*
* ==============================================================================
*  Name         : NavigationShp.cpp
*  Part of      : WEBCORE / Page / SHP
*  Description  : Implementation of Navigation
*  Date         : April 17, 2008
*  Authors      : Vineet Chaudhary (vineet.c@siso.com),
*                 Prashant Nevase (n.prashant@siso.com)
* ==============================================================================
*/

#include "Platform.h"
#include "Page.h"
#include "EventHandler.h"
#include "Frame.h"
#include "FrameView.h"
#include "PlatformKeyboardEvent.h"
#include "PlatformMouseEvent.h"

#include "RenderFrame.h"
#include "RenderFlow.h"
#include "InlineFlowBox.h"

#include "HTMLAreaElement.h"
#include "HTMLNames.h"

#include "MouseEventWithHitTestResults.h"
#include "HitTestRequest.h"

#include "ShpJasmine.h"

#include "Navigation.h"


//1. Virtual Cursor
#ifdef SUPPORT_VIRTUAL_CURSOR
#include "VirtualCursor.h"
#endif
//2. 4-way Navigation
#ifdef SUPPORT_FOUR_WAY_NAVIGATION
#include "FourWayNavigation.h"
#endif
//3. Scroll only navigation
#include "ScrollNavigation.h"


namespace WebCore {

struct NavigationCoreType
    CMM_MODIFIED
{
	ENavigationModeType navigationMode;
	ENavigationModeType lastNavigationMode;
	Vector<CNavigation*> instances;
};

//STATICS
int CNavigation::m_defaultScrollStep = 0;
int CNavigation::m_defaultAccelerationStep = 0;
CNavigation* CNavigation::m_navigation = 0;
NavigationCoreType* CNavigation::m_navigationCore = 0;

#define SMART_TOUCH_DEFAULT_FOCUS_RANGE		20
#define SMART_TOUCH_EDGE_FOCUS_THRESHOLD	2

CNavigation::CNavigation()
{
	m_page = 0;
	m_mainFrameView = 0;
	m_frameView = 0;
	m_hoveredNode = 0;
	m_smartTouchEnabled = true;
	m_smartTouchFocusRange = SMART_TOUCH_DEFAULT_FOCUS_RANGE;
}

CNavigation::~CNavigation()
{
	m_page->setNavigationInstance(0);
	m_page = 0;
	m_mainFrameView = 0;
	m_frameView = 0;
	m_hoveredNode = 0;
	m_smartTouchEnabled = false;
}

void CNavigation::initNavigationStatics()
{
	m_navigationCore = 0;
	m_navigation = 0;
	m_defaultScrollStep = 0;
	m_defaultAccelerationStep = 0;
}

#ifdef SFB_GLOBAL_STATIC_HANDLING
void initCNavigationGlobals()
{
	CNavigation::initNavigationStatics();

	//1. VC statics
#ifdef SUPPORT_VIRTUAL_CURSOR
	CVirtualCursor::initVCStatics();
#endif

	//2. 4-way navigation statics
#ifdef SUPPORT_FOUR_WAY_NAVIGATION 
	CFourWayNavigation::initFWNStatics();
#endif

	//3. scroll only mode statics
	CScrollNavigation::initSNStatics();
}
#endif

void CNavigation::init()
{
	ASSERT(!m_navigationCore);
	
	m_navigationCore = new NavigationCoreType;

	m_navigationCore->navigationMode = NAVIGATION_MODE_SCROLL_ONLY;
	m_navigationCore->lastNavigationMode = NAVIGATION_MODE_SCROLL_ONLY;
	m_navigationCore->instances.clear();

	//1. init VC
#ifdef SUPPORT_VIRTUAL_CURSOR
	CVirtualCursor::init();
#endif
	//2. init 4-way navigation
#ifdef SUPPORT_FOUR_WAY_NAVIGATION
	CFourWayNavigation::init();
#endif

	//3. init scroll only mode
	CScrollNavigation::init();

	//set default navigation mode
	CNavigation::setNavigationMode(m_navigationCore->navigationMode);
}

void CNavigation::finalize()
{
	//1. finalize VC
#ifdef SUPPORT_VIRTUAL_CURSOR
	CVirtualCursor::finalize();
#endif

	//2. finalize 4-way navigation
#ifdef SUPPORT_FOUR_WAY_NAVIGATION
	CFourWayNavigation::finalize();
#endif

	//3. finalize scroll only mode
	CScrollNavigation::finalize();

	ASSERT(m_navigationCore);

	m_navigationCore->instances.clear();

	delete m_navigationCore;
}

CNavigation* CNavigation::createInstance()
{
	CNavigation* instance = 0;

	switch(m_navigationCore->navigationMode)
	{
#ifdef SUPPORT_VIRTUAL_CURSOR
		case NAVIGATION_MODE_VIRTUAL_CURSOR:
		{
			instance = CVirtualCursor::createInstance();
			break;
		}
#endif

#ifdef SUPPORT_FOUR_WAY_NAVIGATION
		case NAVIGATION_MODE_4_WAY_NAVIGATION:
		{
			instance = CFourWayNavigation::createInstance();
			break;
		}
#endif

		case NAVIGATION_MODE_SCROLL_ONLY:
		{
			instance = CScrollNavigation::createInstance();
			break;
		}

		default:
		{
			instance = CScrollNavigation::createInstance();
			break;
		}
	}

	ASSERT(instance);

	m_navigationCore->instances.append(instance);

	return instance;
}

void CNavigation::destroyInstance(CNavigation* instance)
{
	switch(m_navigationCore->lastNavigationMode)
	{
#ifdef SUPPORT_VIRTUAL_CURSOR
		case NAVIGATION_MODE_VIRTUAL_CURSOR:
		{
			CVirtualCursor::destroyInstance(static_cast<CVirtualCursor*>(instance));
			break;
		}
#endif

#ifdef SUPPORT_FOUR_WAY_NAVIGATION
		case NAVIGATION_MODE_4_WAY_NAVIGATION:
		{
			CFourWayNavigation::destroyInstance(static_cast<CFourWayNavigation*>(instance));
			break;
		}
#endif

		case NAVIGATION_MODE_SCROLL_ONLY:
		{
			CScrollNavigation::destroyInstance(static_cast<CScrollNavigation*>(instance));
			break;
		}

		default:
		{
			CScrollNavigation::destroyInstance(static_cast<CScrollNavigation*>(instance));
			break;
		}
	}

	//remove the given instance from the instance vector
	for(int i = 0; i < m_navigationCore->instances.size(); ++i)
	{
		if(instance == m_navigationCore->instances[i])
		{
			m_navigationCore->instances.remove(i);
			break;
		}
	}
}

void CNavigation::setNavigationMode(ENavigationModeType navigationMode)
{
	m_navigationCore->navigationMode = navigationMode;
	doSetNavigationMode();
}

ENavigationModeType CNavigation::getNavigationMode()
{
	return m_navigationCore->navigationMode;
}

void CNavigation::doSetNavigationMode()
{
	if(m_navigationCore->lastNavigationMode != m_navigationCore->navigationMode)
	{
		for(int i = 0; m_navigationCore->instances.size(); ++i)
		{
			//create new instance of new mode set
			CNavigation* newInstance = CNavigation::createInstance();

			//copy stuff from old instance to new instance
			copyToNewInstance(newInstance, m_navigationCore->instances[i]);

			//check if this old instance is set as active instance
			if(m_navigation == m_navigationCore->instances[i])
			{
				m_navigation = newInstance;
			}

			//setframeview
			if(newInstance->m_frameView)
				newInstance->setFrameView(newInstance->m_frameView);
			else
				newInstance->setFrameView(newInstance->m_mainFrameView);

			//do actions newly set page
			newInstance->doPostPaintAction();
			newInstance->doDefaultAction();

			//delete old instance
			CNavigation::destroyInstance(m_navigationCore->instances[i]);
			
			//save this instance at current iteration
			m_navigationCore->instances[i] = newInstance;

			newInstance->m_page->setNavigationInstance(newInstance);
		}

		m_navigationCore->lastNavigationMode = m_navigationCore->navigationMode;
	}
}

void CNavigation::copyToNewInstance(CNavigation* fromInstance, CNavigation* toInstance)
{
	if(fromInstance && toInstance)
	{
		toInstance->m_hGC = fromInstance->m_hGC;
		toInstance->m_page = fromInstance->m_page;
		toInstance->m_mainFrameView = fromInstance->m_mainFrameView;
		toInstance->m_frameView = fromInstance->m_frameView;
	}
}

void CNavigation::switchPage(CNavigation* navigation)
{
	if(navigation)
	{
		//set this navigation pointer as current navigation
		m_navigation = navigation;

		//setframeview
		if(m_navigation->m_frameView)
			m_navigation->setFrameView(m_navigation->m_frameView);
		else
			m_navigation->setFrameView(m_navigation->m_mainFrameView);
		
		//do actions newly set page
		m_navigation->doPostPaintAction();
		m_navigation->doDefaultAction();
	}
}

void CNavigation::setMainFrameView(FrameView* mainFrameView)
{
	m_mainFrameView = mainFrameView;
#ifdef	MULTIPLE_INSTANCE_SUPPORT //Pramod - Multiple instance support
	m_frameView = mainFrameView;
#endif
}

void CNavigation::setGC(int hGC)
{
	m_hGC = hGC;
}

void CNavigation::setPage(Page* page)
{
	m_page = page;
}

void CNavigation::startFastScrolling()
{
	//calc multiplier in fibonacci sequence
	if(m_fastScrolling.isFastScrolling)
	{
		if(m_scrollOffset < m_fastScrolling.scrollOffsetMaxValueThreshold)
		{
			int temp = m_fastScrolling.scrollOffsetMultiplier;
			m_fastScrolling.scrollOffsetMultiplier += m_fastScrolling.scrollOffsetLastMultiplier;
			m_fastScrolling.scrollOffsetLastMultiplier = temp;

			m_scrollOffset += ( m_fastScrolling.scrollOffsetMultiplier * 
								CNavigation::m_defaultAccelerationStep 
#ifdef SFB_SUPPORT_IMAGE_ZOOM
                                * m_frameView->frame()->zoomFactor()
#endif
                                );

			if(m_scrollOffset > m_fastScrolling.scrollOffsetMaxValueThreshold)
				m_scrollOffset = m_fastScrolling.scrollOffsetMaxValueThreshold;
		}
	}
	else
		m_fastScrolling.isFastScrolling = true;
}

void CNavigation::stopFastScrolling()
{
	m_fastScrolling.isFastScrolling = false;
	m_scrollOffset = CNavigation::m_defaultScrollStep;
	m_fastScrolling.scrollOffsetMultiplier = 1;
	m_fastScrolling.scrollOffsetLastMultiplier = 0;
}

void CNavigation::sendVirtualMouseEvent(MouseEventType type, IntPoint wndPoint, IntPoint contentsPoint)
{
	PlatformMouseEvent pme(wndPoint, contentsPoint, LeftButton, type,
						   true, false, false, false, false, 0);

    if(MouseEventReleased == type)
        m_frameView->frame()->eventHandler()->handleMouseReleaseEvent(pme);
    else
        m_frameView->frame()->eventHandler()->handleMousePressEvent(pme);
}

Frame* CNavigation::getSubframeForTargetNode(Node* node)
{
    if(!node)
        return 0;

    RenderObject* renderer = node->renderer();
    if(!renderer || !renderer->isWidget())
        return 0;

    Widget* widget = static_cast<RenderWidget*>(renderer)->widget();
    if(!widget || !widget->isFrameView())
        return 0;

    return static_cast<FrameView*>(widget)->frame();
}

FrameView* CNavigation::frameViewAtWindowPoint(IntPoint wndPoint)
{
	FrameView* view = m_mainFrameView;

	Frame* validFrame = 0;
	Frame* frame = m_mainFrameView->frame();
	Node* node = 0;

	while(frame)
	{
		validFrame = frame;
		IntPoint documentPoint = frame->view()->windowToContents(wndPoint);
		node = frame->document()->elementFromPoint(documentPoint.x(), documentPoint.y());
		frame = getSubframeForTargetNode(node);
	}

	view = validFrame->view();

	return view;
}

float CNavigation::getDistnace(FloatPoint p1, FloatPoint p2)
{
	p2.setX(p2.x() - p1.x());
	p2.setX(p2.x() * p2.x());
	p2.setY(p2.y() - p1.y());
	p2.setY(p2.y() * p2.y());
	p2.setX(p2.x() + p2.y());
	p2.setX(sqrt(p2.x()));

	return p2.x();
}

bool CNavigation::canNodeHaveFocus(Node* node)
{
	//1. ignore null node
	if(!node)
		return false;

	//2. ignore iframes
	if(node->hasTagName(HTMLNames::iframeTag))
		return false;

	//3. if node supports the focus
	if(node->supportsFocus())
		return true;

	//4. if renderer is editable or style is cursor pointer
	RenderObject* renderer = node->renderer();
	if(renderer && (renderer->isEditable() || (renderer->style()->cursor() == CURSOR_POINTER)))
		return true;

	//5. if element is area element having href
	HTMLAreaElement* areaElt = (node->hasTagName(HTMLNames::areaTag)) ? static_cast<HTMLAreaElement*>(node) : 0;
	if(areaElt && !areaElt->noHref())
		return true;

	return false;
}

bool CNavigation::canNodeAtPointHaveFocus(IntPoint documentPoint, Node*& node)
{
	bool result = false;

	Frame* frame = m_frameView->frame();
	Frame* validFrame = 0;

	IntPoint wndPoint = frame->view()->contentsToWindow(documentPoint);
	
	//get the node of innermost subframe
	while(frame)
	{
		validFrame = frame;
		documentPoint = frame->view()->windowToContents(wndPoint);
		node = frame->document()->elementFromPoint(documentPoint.x(), documentPoint.y());
		frame = getSubframeForTargetNode(node);
	}

	if(node)
	{
		const HitTestRequest hitTest(true, false);
		PlatformMouseEvent pme(wndPoint, documentPoint, NoButton, MouseEventMoved,
							false, false, true, false, false, 0);
		MouseEventWithHitTestResults mev = validFrame->document()->prepareMouseEvent(hitTest, documentPoint, pme);
		
		if(mev.isOverLink() || canNodeHaveFocus(node))
		{
			result = true;
		}
	}

	return result;
}

#ifdef SUPPORT_SMART_TOUCH_FOCUSING

void CNavigation::getRenderersAtSmartTouchLocation(Vector<RenderObject*>& renderersList, IntPoint docPoint)
{
	IntRect viewRect;
		
	int vlx = docPoint.x() - m_smartTouchFocusRange;
	int vly = docPoint.y() - m_smartTouchFocusRange;

	if(vlx < 0) vlx = 0;
	if(vly < 0) vly = 0;

	viewRect.setLocation(IntPoint(vlx, vly));

	viewRect.setHeight(m_smartTouchFocusRange * 2);
	viewRect.setWidth(m_smartTouchFocusRange * 2);

//#define USE_OPTIMIZED_RENDERERS_ITERATION
#ifdef USE_OPTIMIZED_RENDERERS_ITERATION
	//This is the optimized implementation. This will be enabled once the
	//function getRenderersInRect gets implemented for all containers correctly.

	//m_frameView->frame()->renderer()->getRenderersInRect(renderersList, viewRect, 0, 0);
#else
	//This is the temporary implementation. This iterates *all* the nodes and
	//finds nodes that fall within the corresponding bounding rect.

	PassRefPtr<HTMLCollection> allNodes = m_frameView->frame()->document()->all();

	Node* node;

	for(int i = 0; i < allNodes->length(); i++)
    {
		node = allNodes->item(i);
		RenderObject* renderer = node->renderer();
		
		if(renderer)
		{
			IntRect rect = renderer->absoluteBoundingBoxRect();
			if(rect.intersects(viewRect))
			{
				renderersList.append(renderer);
			}
		}
	}
#endif

}

IntPoint CNavigation::ClosestPointOfRectFromPoint(IntRect rect, IntPoint fromPoint)
{
	FrameView* view = m_frameView;

	int refX = fromPoint.x();
	int refY = fromPoint.y();

	IntRect wndRect = view->contentsToWindow(rect);
	int leftEdge = wndRect.x();
	int rightEdge = wndRect.x() + wndRect.width();
	int topEdge = wndRect.y();
	int bottomEdge = wndRect.y() + wndRect.height();

	IntPoint currentPoint;

	if(rightEdge < refX)
	{
		if(bottomEdge < refY)
		{
			currentPoint.setX(rightEdge - SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
			currentPoint.setY(bottomEdge - SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
		}
		else
		{
			if(topEdge < refY)
			{
				currentPoint.setX(rightEdge - SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
				currentPoint.setY(refY);
			}
			else
			{
				currentPoint.setX(rightEdge - SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
				currentPoint.setY(topEdge + SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
			}
		}

	}
	else
	{
		if(leftEdge < refX)
		{
			if(bottomEdge <= refY)
			{
				currentPoint.setX(refX);
				currentPoint.setY(bottomEdge - SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
			}
			else
			{
				currentPoint.setX(refX);
				currentPoint.setY(topEdge + SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
			}
		}
		else
		{
			if(bottomEdge < refY)
			{
				currentPoint.setX(leftEdge + SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
				currentPoint.setY(bottomEdge - SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
			}
			else
			{
				if(topEdge < refY)
				{
					currentPoint.setX(leftEdge + SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
					currentPoint.setY(refY);
				}
				else
				{
					currentPoint.setX(leftEdge + SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
					currentPoint.setY(topEdge + SMART_TOUCH_EDGE_FOCUS_THRESHOLD);
				}
			}						
		}
	}

	return currentPoint;
}

IntPoint CNavigation::ClosestPointOfNodeFromPoint(RenderObject* renderer, IntPoint fromPoint)
{
	IntPoint resultPoint(0, 0);

	Node* node = renderer->node();
	String nodeName = node->nodeName();

	//Anchor may have more than one inline flow boxes. Calculate the closest
	//box from the reference point *fromPoint* and return its point
	if(nodeName == "A")
	{
		IntPoint currentPoint;
		float shortestDistance = m_smartTouchFocusRange * 2;

		RenderFlow* renderFlow = reinterpret_cast<RenderFlow*>(renderer);

		IntRect absRect = renderer->absoluteBoundingBoxRect();

		InlineFlowBox* childBox = renderFlow->firstLineBox();

		if(childBox && childBox->nextFlowBox())
		{
			for(; childBox; childBox = childBox->nextFlowBox())
			{
				IntRect r((childBox->xPos() + absRect.x()),
					(childBox->yPos() + absRect.y()),
					childBox->width(),
					childBox->height());

				//for this rect, calculate closest point
				currentPoint = ClosestPointOfRectFromPoint(r, fromPoint);
				float currentDistance = getDistnace(currentPoint, fromPoint);

				if(currentDistance < shortestDistance)
				{
					shortestDistance = currentDistance;
					resultPoint = currentPoint;
				}
			}
		}
		else
		{
			IntRect rect = renderer->absoluteBoundingBoxRect();
			resultPoint = ClosestPointOfRectFromPoint(rect, fromPoint);
		}
	}
	else
	{
		IntRect rect = renderer->absoluteBoundingBoxRect();
		resultPoint = ClosestPointOfRectFromPoint(rect, fromPoint);
	}

	return resultPoint;
}

void CNavigation::setSmartTouchFocusRange(int range)
{
	if(range >= 0 && range <= SMART_TOUCH_DEFAULT_FOCUS_RANGE)
		m_smartTouchFocusRange = range;
	else
		m_smartTouchFocusRange = SMART_TOUCH_DEFAULT_FOCUS_RANGE;
}

IntPoint CNavigation::getSmartTouchLocation(IntPoint wndPoint)
{
	ASSERT(m_smartTouchEnabled);

	int refX = wndPoint.x();
	int refY = wndPoint.y();
	IntPoint resultLocation(refX, refY);

	if(m_smartTouchFocusRange > 0 && m_frameView)
	{
		//TODO:: we need to consider multiple frames case. if the smart focus rect is on two different frames
		//which are adjacent, then we need to check nearest focusable node in both the frames

		FrameView* view = m_frameView;
		Frame* frame = view->frame();
		
		IntPoint docPoint = view->windowToContents(wndPoint);
		Node* node;
		
		//Current point, if it is on focusable node then return immediately
		if(canNodeAtPointHaveFocus(docPoint, node))
		{
			return resultLocation;
		}
		
		Vector<RenderObject*> renderersList;
		SFBalDebug(BAL_PERF, "getRenderersAtSmartTouchLocation :: START");
		getRenderersAtSmartTouchLocation(renderersList, docPoint);
		SFBalDebug(BAL_PERF, "getRenderersAtSmartTouchLocation :: END");

		float distance = m_smartTouchFocusRange * 2;

		RenderObject* renderer;
		IntPoint currentPoint;

		//find the closest focusable renderer to given touch point
		for(int i = 0; i < renderersList.size(); i++)
		{
			renderer = renderersList[i];
			node = renderer->node();
			
			if(canNodeHaveFocus(node))
			{
				//Algorithm to find the closest node to the user's touch point
				
				currentPoint = ClosestPointOfNodeFromPoint(renderer, wndPoint);
				
				float temp = getDistnace(FloatPoint(currentPoint.x(), currentPoint.y()), FloatPoint(refX, refY));
				if(temp < distance)
				{
					distance = temp;
					resultLocation = currentPoint;
				}
			}
		}
	}

	return resultLocation;
}
#endif

} //namespace WebCore