/*
* ==============================================================================
*  Name         : VirtualCursor.cpp
*  Part of      : WEBCORE / Page / SHP
*  Description  : Implementation of Virtual Cursor
*  Date         : February 29, 2008
*  Author       : Prashant Nevase
* ==============================================================================
*/

#ifdef SUPPORT_VIRTUAL_CURSOR

#include "FrameView.h"
#include "EventHandler.h"
#include "Frame.h"

#include "VCSAdjustment.h"
#include "VirtualCursor.h"

#include "Cursor.h"
 
#include "gwes.h"
#include "MouseEventWithHitTestResults.h"
#include "HitTestRequest.h"

#ifdef SMART_FOCUSING
#include "HTMLAreaElement.h"
#include "HTMLNames.h"
#endif

#include "RenderFrame.h"
#include "PlatformScrollBar.h"


#define VIRTUAL_CURSOR_DEFAULT_STEP			(LINE_STEP * 3)
#define VIRTUAL_CURSOR_ACCELERATION_STEP	(LINE_STEP)
#define SCROLL_OFFSET_MAX_VALUE_THRESHOLD	(100)

//locs of bounded rect will start at (height or width)/2.5
//e.g. if height is 240, then top loc for bounded rect will be at 96 and
//bottom loc will be at (240 - 96)
#define	BOUNDED_RECT_FACTOR					(2.5)

//LOC
#define DEFAULT_HEIGHT_BEYOND_LOC			(60)
#define DEFAULT_WIDTH_BEYOND_LOC			(50)

#ifdef SMART_FOCUSING
#define VIRTUAL_CURSOR_VC_JUMP_RANGE				(80)
#define VIRTUAL_CURSOR_JUMP_STEP					(3)
#define VIRTUAL_CURSOR_FOCUSED_OFFSET_CORRECTION	(1)
#define VIRTUAL_CURSOR_JUMP_THRESHOLD				(LINE_STEP * 4)
#endif

#define MAX_TEXT_HEIGHT                      (20)

#define WEBCOREMAX(a, b) ((a) > (b) ? (a) : (b)) 
#define WEBCOREMIN(a, b) ((a) < (b) ? (a) : (b))

namespace WebCore {

enum EVirtualMouseEventType
{
	VIRTUAL_MOUSE_PRESSED, 
	VIRTUAL_MOUSE_RELEASED
};

void CVirtualCursor::initVCStatics()
{ 
}



//class member function definitions
CVirtualCursor::CVirtualCursor() : 	m_updatePointerX(true),
									m_updatePointerY(true),

									m_frameTopX(0),
									m_frameTopY(0),

									m_minFrameViewHeight(0),
									m_minFrameViewWidth(0),

									m_scrollBy(0),

									m_lastCursorRectArea(0, 0, 0, 0),

									m_topLoc(DEFAULT_HEIGHT_BEYOND_LOC),
									m_bottomLoc(DEFAULT_HEIGHT_BEYOND_LOC),
									m_leftLoc(DEFAULT_WIDTH_BEYOND_LOC),
									m_rightLoc(DEFAULT_WIDTH_BEYOND_LOC),

									m_frameMaxHeight(0),
									m_frameMaxWidth(0)
{
	m_pointerX = 0;
	m_pointerY = 0;

	m_frameView = 0;
	m_mainFrameView = 0;
	m_hGC = 0;
	m_hoveredNode = 0;
	m_scrollOffset = VIRTUAL_CURSOR_DEFAULT_STEP;

	m_fastScrolling.scrollOffsetMaxValueThreshold = SCROLL_OFFSET_MAX_VALUE_THRESHOLD;
	m_fastScrolling.scrollOffsetMultiplier = 1;
	m_fastScrolling.scrollOffsetLastMultiplier = 0;
	m_fastScrolling.isFastScrolling = false;

	m_navigationFreezed = NAVIGATION_FREEZED_NONE;


	m_hAdj = new CVCSAdjustment(0.0, 0.0, 0.0, 0.0, 0.0, 0.0);
	m_vAdj = new CVCSAdjustment(0.0, 0.0, 0.0, 0.0, 0.0, 0.0);

	m_minFrameViewHeight = PlatformScrollbar::horizontalScrollbarHeight();
	m_minFrameViewWidth = PlatformScrollbar::verticalScrollbarWidth();
}

CVirtualCursor::~CVirtualCursor()
{
	if(m_hAdj)
	{
		delete m_hAdj;
		m_hAdj = 0;
	}

	if(m_vAdj)
	{
		delete m_vAdj;
		m_vAdj = 0;
	}
}

void CVirtualCursor::init()
{
	m_defaultScrollStep = VIRTUAL_CURSOR_DEFAULT_STEP;
	m_defaultAccelerationStep = VIRTUAL_CURSOR_ACCELERATION_STEP;
}

void CVirtualCursor::finalize()
{
}

CVirtualCursor* CVirtualCursor::createInstance()
{
	CVirtualCursor* virtualCursor = new CVirtualCursor();

	//set width and height of last cursor rect area
	if(virtualCursor)
	{
		virtualCursor->m_lastCursorRectArea.setWidth(CPlatformCursorHelper::GetCursorWidth());
		virtualCursor->m_lastCursorRectArea.setHeight(CPlatformCursorHelper::GetCursorHeight());
	}

	return virtualCursor;
}

void CVirtualCursor::destroyInstance(CVirtualCursor* virtualCursor)
{
	if(virtualCursor)
		delete virtualCursor;
}

void CVirtualCursor::resetPage()
{
	if(m_mainFrameView)
	{
		m_hoveredNode = NULL;
		m_pointerX = 0;
		m_pointerY = 0;
		m_updatePointerX = true;
		m_updatePointerY = true;

		m_navigationFreezed = NAVIGATION_FREEZED_NONE;
		
		m_vAdj->setValue(0);
		m_hAdj->setValue(0);
		
		m_scrollBy = 0;

		setFrameView(m_mainFrameView);

		//set cursor to default one and draw it
		CPlatformCursorHelper::SetCursor(pointerCursor());
		doPostPaintAction();
	}
}

void CVirtualCursor::pageLoadingFinished()
{
}

void CVirtualCursor::calculateVerticalLoc()
{
#ifdef SUPPORT_TOUCH_SCREEN
	//for touch screen user can tap anywhere in the screen area.
	m_topLoc = static_cast<int>(m_vAdj->getLower());
	m_bottomLoc = static_cast<int>(m_vAdj->getUpper());
#else
	m_topLoc = static_cast<int>(m_mainFrameView->visibleHeight() / BOUNDED_RECT_FACTOR);
	m_bottomLoc = m_mainFrameView->visibleHeight() - m_topLoc;

	int lower = static_cast<int>(m_vAdj->getLower());
	int upper = static_cast<int>(m_vAdj->getUpper());
	
	m_topLoc = max(m_topLoc, lower);
	m_bottomLoc = min(m_bottomLoc, upper);
#endif
}

void CVirtualCursor::calculateHorizontalLoc()
{
#ifdef SUPPORT_TOUCH_SCREEN
	//for touch screen user can tap anywhere in the screen area.
	m_leftLoc = static_cast<int>(m_hAdj->getLower());
	m_rightLoc = static_cast<int>(m_hAdj->getUpper());
#else
	m_leftLoc = static_cast<int>(m_mainFrameView->visibleWidth() / BOUNDED_RECT_FACTOR);
	m_rightLoc = m_mainFrameView->visibleWidth() - m_leftLoc;

	int lower = static_cast<int>(m_hAdj->getLower());
	int upper = static_cast<int>(m_hAdj->getUpper());

	m_leftLoc = max(m_leftLoc, lower);
	m_rightLoc = min(m_rightLoc, upper);
#endif
}

void CVirtualCursor::setFrameView(FrameView* frameView)
{
	if(frameView)
	{
		//avoid resetting same frame
		if(m_frameView != frameView)
		{
			m_frameView = frameView;

			//set adjustment handlers
			m_hAdj->setHandlers(
				0,
				reinterpret_cast<cbVCSHandler>(&(CVirtualCursor::scrollAdjustmentChangedCB)),
				reinterpret_cast<void*>(m_frameView)
				);
			m_vAdj->setHandlers(
				0,
				reinterpret_cast<cbVCSHandler>(&(CVirtualCursor::scrollAdjustmentChangedCB)),
				reinterpret_cast<void*>(m_frameView)
				);

			//ask to update co-ordinates
			m_updatePointerX = true;
			m_updatePointerY = true;
			updateVerticalStats();
			updateHorizontalStats();

			//update acceleration values
			m_fastScrolling.scrollOffsetMaxValueThreshold = (min(
				m_frameView->visibleHeight(),
				m_frameView->visibleWidth())) / 3;
			m_fastScrolling.scrollOffsetMultiplier = 1;
			m_fastScrolling.scrollOffsetLastMultiplier = 0;
			m_fastScrolling.isFastScrolling = false;

			//release the scroll freezes
			m_navigationFreezed = NAVIGATION_FREEZED_NONE;

			//smartly move the iframe bringing the focused point in the center
			smartMoveIFrame();
		}
	}
}

int CVirtualCursor::getFrameMaxHeight(FrameView* view)
{
	//set maximum of contentHeight and visibleHeight, if vertical scrollbar is null
	//then consider visibleHeight as max height
	int height = view->visibleHeight();
	if(view->verticalScrollBar())
	{
		int frameContentsHeight = view->contentsHeight();
		if(frameContentsHeight > height)
			height = frameContentsHeight;
	}

	return height;
}

int CVirtualCursor::getFrameMaxWidth(FrameView* view)
{
	//set maximum of contentWidth and visibleWidth, if horizontal scrollbar is null
	//then consider visibleWidth as max width
	int width = view->visibleWidth();
	if(view->horizontalScrollBar())
	{
		int frameContentsWidth = view->contentsWidth();
		if(frameContentsWidth > width)
			width = frameContentsWidth;
	}

	return width;
}

void CVirtualCursor::updateVerticalStats()
{
	if(m_frameView)
	{
		//set frame lefttop
		IntRect rect = m_frameView->frameGeometry();
		m_frameTopY = rect.y();

		if(m_updatePointerY)
		{
			//set y-co of virtual cursor
			m_pointerY = m_frameView->scrollOffset().height() +
											m_vAdj->getValue();
		}

		//set maxframeheight
		m_frameMaxHeight = getFrameMaxHeight(m_frameView);
		

		setVerticalVisibleLimits();

		if(m_updatePointerY)
		{
			//get actual position from visible cursor
			m_pointerY -= m_vAdj->getLower();
			//add top hidden height to y-co
			m_pointerY += getFrameTopHiddenHeight();

			m_updatePointerY = false;
		}

		//set the loc for the frame
		calculateVerticalLoc();
	}
}

void CVirtualCursor::updateHorizontalStats()
{
	if(m_frameView)
	{
		//set frame lefttop
		IntRect rect = m_frameView->frameGeometry();
		m_frameTopX = rect.x();

		if(m_updatePointerX)
		{
			m_pointerX = m_frameView->scrollOffset().width() +
											m_hAdj->getValue();

		}

		//set maxframewidth
		m_frameMaxWidth = getFrameMaxWidth(m_frameView);
		

		setHorizontalVisibleLimits();

		if(m_updatePointerX)
		{
			//get actual position from visible cursor
			m_pointerX -= m_hAdj->getLower();
			//add left hidden height to y-co
			m_pointerX += getFrameLeftHiddenWidth();

			m_updatePointerX = false;
		}

		//set the loc for the frame
		calculateHorizontalLoc();
	}
}

void CVirtualCursor::smartMoveIFrame()
{
	if(m_frameView && m_frameView->isIFrameView())
	{
		//get screen center
		int screenCenterY = m_mainFrameView->visibleHeight() / 2;
		int screenCenterX = m_mainFrameView->visibleWidth() / 2;

		FrameView* view = m_frameView;

		//get the current visual pointer's co-ordinates
		int currentVisualY = m_vAdj->getValue();
		int currentVisualX = m_hAdj->getValue();

		if(currentVisualY != screenCenterY)
		{
			if(currentVisualY < screenCenterY)
			{
				int wantToScrollBy = screenCenterY - currentVisualY;
				//scroll parent up
				handleScrollUp(wantToScrollBy, static_cast<FrameView*>(m_frameView->parent()), true);
			}
			else
			{
				int wantToScrollBy = currentVisualY - screenCenterY;
				//scroll parent down
				handleScrollDown(wantToScrollBy, static_cast<FrameView*>(m_frameView->parent()), true);
			}
		}

		if(currentVisualX != screenCenterX)
		{
			if(currentVisualX <= screenCenterX)
			{
				int wantToScrollBy = screenCenterX - currentVisualX;
				//scroll parent left
				handleScrollLeft(wantToScrollBy, static_cast<FrameView*>(m_frameView->parent()), true);
			}
			else
			{
				int wantToScrollBy = currentVisualX - screenCenterX;
				//scroll parent right
				handleScrollRight(wantToScrollBy, static_cast<FrameView*>(m_frameView->parent()), true);
			}
		}
	}
}

void CVirtualCursor::getPointerLocation(FrameView** frameView, int* x, int* y)
{
	if(m_frameView)
	{
		*x = m_pointerX;
		*y = m_pointerY;
	}
	else
	{
		x = 0;
		y = 0;
	}

	*frameView = m_frameView;
}

void CVirtualCursor::setPointerLocation(int x, int y)
{
	if(x >= 0 && y >= 0)
	{
		if(m_frameView)
		{
			int contentsHeight = m_frameView->contentsHeight();
			int contentsWidth = m_frameView->contentsWidth();
			m_pointerY = ((y > contentsHeight) ? contentsHeight : y);
			m_pointerX = ((x > contentsWidth) ? contentsWidth : x);
			m_hAdj->setValue(x);
			m_vAdj->setValue(y);
			m_navigationFreezed = NAVIGATION_FREEZED_NONE;
		}
	}
}

void CVirtualCursor::scrollBy(int xOffset, int yOffset)
{
	if(m_frameView)
	{
		ENavigationType scrollType = NAVIGATION_UP;

		m_navigationFreezed = NAVIGATION_FREEZED_NONE;

		//consider x-offset
		if(0 != xOffset)
		{
			if(xOffset < 0)
			{
				xOffset = -xOffset;
				scrollType = NAVIGATION_LEFT;
			}
			else
				scrollType = NAVIGATION_RIGHT;

			setCursorInPage(scrollType, xOffset, false);
		}

		//consider y-offset
		if(0 != yOffset)
		{
			if(yOffset < 0)
			{
				yOffset = -yOffset;
				scrollType = NAVIGATION_UP;
			}
			else
				scrollType = NAVIGATION_DOWN;

			setCursorInPage(scrollType, yOffset, false);
		}
	}
}

void CVirtualCursor::handleNavigationEvent(int keyCode, bool isKeyUp)
{
	if(m_frameView)
	{
		if(isKeyUp)
			CVirtualCursor::stopFastScrolling();
		else
			CVirtualCursor::startFastScrolling();
		
		if(!isKeyUp)
		{
			//check for up/down/left/right navigations
			switch(keyCode)
			{
				case VK_HP_LEFT: //left
				{
					setCursorInPage(NAVIGATION_LEFT, m_scrollOffset);
					break;
				}
		        
				case VK_HP_RIGHT: //right
				{
					setCursorInPage(NAVIGATION_RIGHT, m_scrollOffset);
					break;
				}
		        
				case VK_HP_UP: //up
				{
					setCursorInPage(NAVIGATION_UP, m_scrollOffset);
					break;
				}
		        
				case VK_HP_DOWN: //down
				{
					setCursorInPage(NAVIGATION_DOWN, m_scrollOffset);
					break;
				}
		        
				case VK_HP_OK: //enter
				{
					IntPoint contentsPoint(m_pointerX, m_pointerY);
					IntPoint wndPoint = m_frameView->contentsToWindow(contentsPoint);

					sendVirtualMouseEvent(MouseEventPressed, wndPoint, contentsPoint);
					sendVirtualMouseEvent(MouseEventReleased, wndPoint, contentsPoint);
					
					break;
				}
			}
		}
	}
}

void CVirtualCursor::doPostPaintAction()
{
	if(this == CNavigation::m_navigation)
	{
		CVirtualCursor::drawCursor();
	}
}

void CVirtualCursor::drawCursor()
{
	CVirtualCursor* virtualCursor = static_cast<CVirtualCursor*>(CNavigation::m_navigation);
	
	if(virtualCursor && virtualCursor->m_frameView)
	{
		//Invalidate the last cursor position, due to graphics issue
		//we need to update the whole frame
		WmShowGCRectArea(virtualCursor->m_hGC, 
			virtualCursor->m_lastCursorRectArea.x(),
			virtualCursor->m_lastCursorRectArea.y(),
			virtualCursor->m_lastCursorRectArea.width(),
			virtualCursor->m_lastCursorRectArea.height());

		IntPoint wndPoint = virtualCursor->m_frameView->contentsToWindow(
			IntPoint(virtualCursor->m_pointerX, virtualCursor->m_pointerY));

		int x = virtualCursor->m_hAdj->getValue();
		int y = virtualCursor->m_vAdj->getValue();

		CPlatformCursorHelper::DrawCursor(virtualCursor->m_hGC, x, y);

		//update drawn cursor as last cursor
		x = x - CPlatformCursorHelper::GetCursorXOffset();
		y = y - CPlatformCursorHelper::GetCursorYOffset();
		virtualCursor->m_lastCursorRectArea.setX(x);
		virtualCursor->m_lastCursorRectArea.setY(y);
	}
}

void CVirtualCursor::scrollAdjustmentChangedCB(CVCSAdjustment* adjustment, void* _that)
{
	CVirtualCursor* virtualCursor = static_cast<CVirtualCursor*>(CNavigation::m_navigation);

	if(virtualCursor && virtualCursor->m_frameView)
	{
		FrameView* that = reinterpret_cast<FrameView*>(_that);

		if(false == that->areScrollbarsSuppressed())
		{
			ScrollDirection direction = ScrollUp;
			bool doScroll = false;

			if(adjustment == virtualCursor->m_hAdj)
			{
				EChangedValueType cvt = virtualCursor->m_hAdj->getChangedValueType();
				switch(cvt)
				{
					case CHANGED_VALUE_INCREMENTED:
					{
						direction = ScrollRight;
						doScroll = true;
						break;
					}

					case CHANGED_VALUE_DECREMENTED:
					{
						direction = ScrollLeft;
						doScroll = true;
						break;
					}
				}
			}
			else if(adjustment == virtualCursor->m_vAdj)
			{
				EChangedValueType cvt = virtualCursor->m_vAdj->getChangedValueType();
				switch(cvt)
				{
					case CHANGED_VALUE_INCREMENTED:
					{
						direction = ScrollDown;
						doScroll = true;
						break;
					}

					case CHANGED_VALUE_DECREMENTED:
					{
						direction = ScrollUp;
						doScroll = true;
						break;
					}
				}
			}

			if(doScroll)
			{
				that->scroll(direction, ScrollByLine, (virtualCursor->m_scrollBy / LINE_STEP));
				virtualCursor->m_scrollBy = 0;
			}
		}
	}
}

#if 0
void CVirtualCursor::scrollFrame(FrameView* view, ScrollDirection direction, float multiplier)
{
	bool updateFrameViewStats = false;
	//check if this frame has fininshed scrolling, try scolling parent
	while(view)
	{
		IntSize oldOffset = view->scrollOffset();
		view->scroll(direction, ScrollByLine, multiplier);
		IntSize newOffset = view->scrollOffset();

		if(updateFrameViewStats)
		{
			if(ScrollUp == direction || ScrollDown == direction)
				CVirtualCursor::updateVerticalStats();
			else
				CVirtualCursor::updateHorizontalStats();
		}

		if(newOffset == oldOffset)
		{
			view = static_cast<FrameView*>(view->parent());
			updateFrameViewStats = true;
		}
		else
		{
			break;
		}
	}
}

#endif

#ifdef SMART_FOCUSING

bool CVirtualCursor::isNextFocusableNodeFound(IntPoint documentPoint, float& scrollOffset)
{
	bool result = false;
	Node* node = 0;

	if(canNodeAtPointHaveFocus(documentPoint, node))
	{
		if(node != m_hoveredNode)
		{
			m_hoveredNode = node;
			result = true;
		}
		else
		{
			scrollOffset += VIRTUAL_CURSOR_JUMP_STEP;
		}
	}

	return result;
}

bool CVirtualCursor::isFocusableNodeInJumpRange(ENavigationType scrollType, float& scrollOffset)
{
	bool result = false;
	int jumpRange;
	scrollOffset = 0;
	
	jumpRange = VIRTUAL_CURSOR_VC_JUMP_RANGE;

	int xPos = m_pointerX;
	int yPos = m_pointerY;
	IntPoint documentPoint(xPos, yPos);
	bool goingBeyondLimits = false;
	float offset = 0;
		
	
	switch(scrollType)
	{
		case NAVIGATION_UP:
		{
			while(!goingBeyondLimits && (offset < jumpRange))
			{
				yPos = m_pointerY - offset;
				if(yPos < 0)
					goingBeyondLimits = true;

				documentPoint.setY(yPos);

				if(isNextFocusableNodeFound(documentPoint, scrollOffset))
				{
					scrollOffset = offset + VIRTUAL_CURSOR_FOCUSED_OFFSET_CORRECTION;
					break;
				}

				offset += VIRTUAL_CURSOR_JUMP_STEP;
			}

			break;
		}

		case NAVIGATION_DOWN:
		{
			while(!goingBeyondLimits && (offset < jumpRange))
			{
				yPos = m_pointerY + offset;
				if(yPos > m_frameMaxHeight)
					goingBeyondLimits = true;

				documentPoint.setY(yPos);

				if(isNextFocusableNodeFound(documentPoint, scrollOffset))
				{
					scrollOffset = offset + VIRTUAL_CURSOR_FOCUSED_OFFSET_CORRECTION;
					break;
				}

				offset += VIRTUAL_CURSOR_JUMP_STEP;
			}

			break;
		}

		case NAVIGATION_LEFT:
		{
			while(!goingBeyondLimits && (offset < jumpRange))
			{
				xPos = m_pointerX - offset;
				if(xPos < 0)
					goingBeyondLimits = true;

				documentPoint.setX(xPos);

				if(isNextFocusableNodeFound(documentPoint, scrollOffset))
				{
					scrollOffset = offset + VIRTUAL_CURSOR_FOCUSED_OFFSET_CORRECTION;
					break;
				}

				offset += VIRTUAL_CURSOR_JUMP_STEP;
			}

			break;
		}

		case NAVIGATION_RIGHT:
		{
			while(!goingBeyondLimits && (offset < jumpRange))
			{
				xPos = m_pointerX + offset;
				if(xPos > m_frameMaxWidth)
					goingBeyondLimits = true;

				documentPoint.setX(xPos);

				if(isNextFocusableNodeFound(documentPoint, scrollOffset))
				{
					scrollOffset = offset + VIRTUAL_CURSOR_FOCUSED_OFFSET_CORRECTION;
					break;
				}

				offset += VIRTUAL_CURSOR_JUMP_STEP;
			}

			break;
		}
	}

	result = (scrollOffset != 0) ? true : false;

	return result;
}
#endif //SMART_FOCUSING

bool CVirtualCursor::handleScrollUp(int wantToScrollBy, FrameView* view, bool scrollOnly)
{
	bool result = false;

	bool updateFrameViewStats = false;
	int scrolled = wantToScrollBy;

	while((wantToScrollBy > 0) && view)
	{
		int scrollRemained = view->scrollOffset().height();

		if(wantToScrollBy <= scrollRemained)
		{
			m_scrollBy = wantToScrollBy;
			//scroll the respective view
			CVirtualCursor::scrollAdjustmentChangedCB(m_vAdj, view);

			wantToScrollBy = 0;

			result = true;
		}
		else
		{
			updateFrameViewStats = true;

			wantToScrollBy -= scrollRemained;
			m_scrollBy = scrollRemained;
			//scroll the respective view
			CVirtualCursor::scrollAdjustmentChangedCB(m_vAdj, view);

			//now request parent, if he can scroll
			view = static_cast<FrameView*>(view->parent());

			if(0 == scrollRemained)
				result = false;

		}
	}

	if(updateFrameViewStats || scrollOnly)
		CVirtualCursor::updateVerticalStats();

	if(scrollOnly)
	{
		//If wantToScrollBy is not finished, then it means that
		//we can not scroll total, so bring visible point at the
		//location which we've scrolled.
		scrolled -= wantToScrollBy;
		m_vAdj->setValue(m_vAdj->getValue() + scrolled);
	}
	else
		m_vAdj->setValue(m_topLoc - wantToScrollBy);

	return result;
}

bool CVirtualCursor::handleScrollDown(int wantToScrollBy, FrameView* view, bool scrollOnly)
{
	bool result = false;

	bool updateFrameViewStats = false;
	int scrolled = wantToScrollBy;

	while((wantToScrollBy > 0) && view)
	{
		int scrollRemained = (getFrameMaxHeight(view) - view->visibleHeight()) - view->scrollOffset().height();

		if(wantToScrollBy <= scrollRemained)
		{
			m_scrollBy = wantToScrollBy;
			//scroll the respective view
			CVirtualCursor::scrollAdjustmentChangedCB(m_vAdj, view);

			wantToScrollBy = 0;

			result = true;
		}
		else
		{
			updateFrameViewStats = true;

			wantToScrollBy -= scrollRemained;
			m_scrollBy = scrollRemained;
			//scroll the respective view
			CVirtualCursor::scrollAdjustmentChangedCB(m_vAdj, view);

			//now request parent, if he can scroll
			view = static_cast<FrameView*>(view->parent());

			if(0 == scrollRemained)
				result = false;

		}
	}

	if(updateFrameViewStats || scrollOnly)
		CVirtualCursor::updateVerticalStats();

	if(scrollOnly)
	{
		//If wantToScrollBy is not finished, then it means that
		//we can not scroll total, so bring visible point at the
		//location which we've scrolled.
		scrolled -= wantToScrollBy;
		m_vAdj->setValue(m_vAdj->getValue() - scrolled);
	}
	else
		m_vAdj->setValue(m_bottomLoc + wantToScrollBy);

	return result;
}

bool CVirtualCursor::handleScrollLeft(int wantToScrollBy, FrameView* view, bool scrollOnly)
{
	bool result = false;

	bool updateFrameViewStats = false;
	int scrolled = wantToScrollBy;

	while((wantToScrollBy > 0) && view)
	{
		int scrollRemained = view->scrollOffset().width();

		if(wantToScrollBy <= scrollRemained)
		{
			m_scrollBy = wantToScrollBy;
			//scroll the respective view
			CVirtualCursor::scrollAdjustmentChangedCB(m_hAdj, view);

			wantToScrollBy = 0;

			result = true;
		}
		else
		{
			updateFrameViewStats = true;

			wantToScrollBy -= scrollRemained;
			m_scrollBy = scrollRemained;
			//scroll the respective view
			CVirtualCursor::scrollAdjustmentChangedCB(m_hAdj, view);

			//now request parent, if he can scroll
			view = static_cast<FrameView*>(view->parent());

			if(0 == scrollRemained)
				result = false;

		}
	}

	if(updateFrameViewStats || scrollOnly)
		CVirtualCursor::updateVerticalStats();

	if(scrollOnly)
	{
		//If wantToScrollBy is not finished, then it means that
		//we can not scroll total, so bring visible point at the
		//location which we've scrolled.
		scrolled -= wantToScrollBy;
		m_hAdj->setValue(m_hAdj->getValue() + scrolled);
	}
	else
		m_hAdj->setValue(m_leftLoc - wantToScrollBy); 

	return result;
}

bool CVirtualCursor::handleScrollRight(int wantToScrollBy, FrameView* view, bool scrollOnly)
{
	bool result = false;

	bool updateFrameViewStats = false;
	int scrolled = wantToScrollBy;

	while((wantToScrollBy > 0) && view)
	{
		int scrollRemained = (getFrameMaxWidth(view) - view->visibleWidth()) - view->scrollOffset().width();

		if(wantToScrollBy <= scrollRemained)
		{
			m_scrollBy = wantToScrollBy;
			//scroll the respective view
			CVirtualCursor::scrollAdjustmentChangedCB(m_hAdj, view);

			wantToScrollBy = 0;

			result = true;
		}
		else
		{
			updateFrameViewStats = true;

			wantToScrollBy -= scrollRemained;
			m_scrollBy = scrollRemained;
			//scroll the respective view
			CVirtualCursor::scrollAdjustmentChangedCB(m_hAdj, view);

			//now request parent, if he can scroll
			view = static_cast<FrameView*>(view->parent());

			if(0 == scrollRemained)
				result = false;

		}
	}

	if(updateFrameViewStats || scrollOnly)
		CVirtualCursor::updateVerticalStats();

	if(scrollOnly)
	{
		//If wantToScrollBy is not finished, then it means that
		//we can not scroll total, so bring visible point at the
		//location which we've scrolled.
		scrolled -= wantToScrollBy;
		m_hAdj->setValue(m_hAdj->getValue() - scrolled);
	}
	else
		m_hAdj->setValue(m_rightLoc + wantToScrollBy);

	return result;
}

bool CVirtualCursor::setCursorInPage(ENavigationType scrollType, float scrollOffset, bool informWebCore)
{
	bool result = false;

	//check whether requested scroll is freezed
	if(m_navigationFreezed & scrollType)
		return result;

	bool hoveredNodeSet = false;

#ifdef SMART_FOCUSING
	if(VIRTUAL_CURSOR_JUMP_THRESHOLD > scrollOffset)
	{
		float offset = 0;
		if(isFocusableNodeInJumpRange(scrollType, offset))
		{
			scrollOffset = offset;
			hoveredNodeSet = true;
		}
	}
#endif //SMART_FOCUSING

    switch(scrollType)
    {
        case NAVIGATION_UP:
        {
            //release down freeze
			m_navigationFreezed &= ~(NAVIGATION_FREEZED_DOWN);

			m_vAdj->setChangedValueType(CHANGED_VALUE_DECREMENTED);

			if(m_pointerY == 0)
			{
				m_navigationFreezed |= NAVIGATION_FREEZED_UP;
			}
			else
			{
				m_pointerY -= scrollOffset;

				if(m_pointerY < 0)
					m_pointerY = 0;

				int scrollHeight = m_frameView->scrollOffset().height();

				FrameView* view = m_frameView;
				int newYInWindow = m_pointerY;
				
				while(view)
				{
					newYInWindow += view->frameGeometry().y() - view->scrollOffset().height();
					view = static_cast<FrameView*>(view->parent());
				}
				
				//decide whether to scroll
				if(newYInWindow >= m_topLoc)
				{
					//we can move the cursor until loc
					// vprajan: max, min taken from Webcore namespace
					int newValue = WEBCOREMAX(((m_vAdj->getValue()) - scrollOffset), m_topLoc);
					m_vAdj->setValue(newValue);
				}
				else
				{
					int wantToScrollBy = m_topLoc - newYInWindow;
					result = handleScrollUp(wantToScrollBy, m_frameView);
				}
			}

			break;
        }
        
        case NAVIGATION_DOWN:
        {
			//release up freeze
			m_navigationFreezed &= ~(NAVIGATION_FREEZED_UP);

			m_vAdj->setChangedValueType(CHANGED_VALUE_INCREMENTED);

			if(m_pointerY == m_frameMaxHeight)
			{
				m_navigationFreezed |= NAVIGATION_FREEZED_DOWN;
			}
			else
			{
				m_pointerY += scrollOffset;

				if(m_pointerY > m_frameMaxHeight)
					m_pointerY = m_frameMaxHeight;

				int visibleHeight = m_frameView->visibleHeight();
				int scrollHeight = m_frameView->scrollOffset().height();

				FrameView* view = m_frameView;
				int newYInWindow = m_pointerY;
				
				while(view)
				{
					newYInWindow += view->frameGeometry().y() - view->scrollOffset().height();
					view = static_cast<FrameView*>(view->parent());
				}

				//decide whether to scroll
				if(m_bottomLoc > newYInWindow)
				{
					//we can move the cursor until loc
					int newValue = WEBCOREMIN(((m_vAdj->getValue()) + scrollOffset), m_bottomLoc);
					m_vAdj->setValue(newValue);
				}
				else
				{
					int wantToScrollBy = newYInWindow - m_bottomLoc;
					result = handleScrollDown(wantToScrollBy, m_frameView);
				}
			}

			break;
        }
        
        case NAVIGATION_LEFT:
        {
			//release right freeze
			m_navigationFreezed &= ~(NAVIGATION_FREEZED_RIGHT);

			m_hAdj->setChangedValueType(CHANGED_VALUE_DECREMENTED);

			if(m_pointerX == 0)
			{
				m_navigationFreezed |= NAVIGATION_FREEZED_LEFT;
			}
			else
			{
				m_pointerX -= scrollOffset;

				if(m_pointerX < 0)
					m_pointerX = 0;

				int scrollWidth = m_frameView->scrollOffset().width();

				FrameView* view = m_frameView;
				int newXInWindow = m_pointerX;
				
				while(view)
				{
					newXInWindow += view->frameGeometry().x() - view->scrollOffset().width();
					view = static_cast<FrameView*>(view->parent());
				}
				
				//decide whether to scroll
				if(newXInWindow >= m_leftLoc)
				{
					//we can move the cursor until loc
					int newValue = WEBCOREMAX(((m_hAdj->getValue()) - scrollOffset), m_leftLoc);
					m_hAdj->setValue(newValue);
				}
				else
				{
					int wantToScrollBy = m_leftLoc - newXInWindow;
					result = handleScrollLeft(wantToScrollBy, m_frameView);
				}
			}

			break;
        }
        
        case NAVIGATION_RIGHT:
        {
            //release left freeze
			m_navigationFreezed &= ~(NAVIGATION_FREEZED_LEFT);

			m_hAdj->setChangedValueType(CHANGED_VALUE_INCREMENTED);

			if(m_pointerX == m_frameMaxWidth)
			{
				m_navigationFreezed |= NAVIGATION_FREEZED_RIGHT;
			}
			else
			{
				m_pointerX += scrollOffset;

				if(m_pointerX > m_frameMaxWidth)
					m_pointerX = m_frameMaxWidth;

				int visibleWidth = m_frameView->visibleWidth();
				int scrollWidth = m_frameView->scrollOffset().width();

				FrameView* view = m_frameView;
				int newXInWindow = m_pointerX;
				
				while(view)
				{
					newXInWindow += view->frameGeometry().x() - view->scrollOffset().width();
					view = static_cast<FrameView*>(view->parent());
				}

				//decide whether to scroll
				if(m_rightLoc > newXInWindow)
				{
					//we can move the cursor until loc
					int newValue = WEBCOREMIN(((m_hAdj->getValue()) + scrollOffset), m_rightLoc);
					m_hAdj->setValue(newValue);
				}
				else
				{
					int wantToScrollBy = newXInWindow - m_rightLoc;
					result = handleScrollRight(wantToScrollBy, m_frameView);
				}
			}

			break;
        }
    }

	if(informWebCore && !(m_navigationFreezed & scrollType))
	{
		IntPoint wndPoint = m_frameView->contentsToWindow(IntPoint(m_pointerX, m_pointerY));
		
		//Inform the webcore
		PlatformMouseEvent pme(IntPoint(wndPoint.x(), wndPoint.y()), IntPoint(m_pointerX, m_pointerY), NoButton, MouseEventMoved,
							   false, false, true, false, false, 0);

		m_frameView->frame()->eventHandler()->mouseMoved(pme);

		//get the mouse hovered node
		const HitTestRequest hitTest(false, true);
		IntPoint documentPoint(m_pointerX, m_pointerY);
		if(!hoveredNodeSet)
			m_hoveredNode = m_frameView->frame()->document()->prepareMouseEvent(hitTest, documentPoint, pme).targetNode(); 	

		//if we are scrolling then draw cursor after main GC is shown,
		//otherwise draw here
		if(!result)
		{
			doPostPaintAction();
		}
	}

	return (result & !(m_navigationFreezed & scrollType));
}

int CVirtualCursor::getFrameTopHiddenHeight()
{
	return (getFrameVisibleHeight() - (m_vAdj->getUpper() - m_vAdj->getLower()) - getFrameBottomHiddenHeight());
}

int CVirtualCursor::getFrameBottomHiddenHeight()
{
	int hiddenHeight = 0;
	int myFrameVisibleHeight = getFrameVisibleHeight();
	FrameView* parentView = static_cast<FrameView*>(m_frameView->parent());
	int myFrameTopY = m_frameView->frameGeometry().y();

	while(parentView)
	{
		int parentVisibleHeight = parentView->visibleHeight();
		int parentScrollHeight = parentView->scrollOffset().height();

		int tempHeight = (myFrameVisibleHeight - ((parentScrollHeight + parentVisibleHeight) - myFrameTopY));
		hiddenHeight = (tempHeight > hiddenHeight) ? tempHeight : hiddenHeight;

		myFrameTopY += parentView->frameGeometry().y();
		parentView = static_cast<FrameView*>(parentView->parent());
	}

	return hiddenHeight;
}

int CVirtualCursor::getFrameLeftHiddenWidth()
{
	return (getFrameVisibleWidth() - (m_hAdj->getUpper() - m_hAdj->getLower()) - getFrameRightHiddenWidth());
}

int CVirtualCursor::getFrameRightHiddenWidth()
{
	int hiddenWidth = 0;
	int myFrameVisibleWidth = getFrameVisibleWidth();
	FrameView* parentView = static_cast<FrameView*>(m_frameView->parent());
	int myFrameTopX = m_frameView->frameGeometry().x();

	while(parentView)
	{
		int parentVisibleWidth = parentView->visibleWidth();
		int parentScrollWidth = parentView->scrollOffset().width();

		int tempWidth = (myFrameVisibleWidth - ((parentScrollWidth + parentVisibleWidth) - myFrameTopX));
		hiddenWidth = (tempWidth > hiddenWidth) ? tempWidth : hiddenWidth;

		myFrameTopX += parentView->frameGeometry().x();
		parentView = static_cast<FrameView*>(parentView->parent());
	}

	return hiddenWidth;
}

void CVirtualCursor::setVerticalVisibleLimits()
{
	//m_virtualCursor->m_vAdj->setPage_size(contentsHeight);
	m_vAdj->setStep_increment(10);

	FrameView* currentView = m_frameView;
	FrameView* parentView = static_cast<FrameView*>(currentView->parent());
	int myFrameRelativeTopY = m_frameView->frameGeometry().y();
	int myFrameRelativeHeight = getFrameVisibleHeight();

	while(parentView)
	{
		myFrameRelativeTopY -= parentView->scrollOffset().height();
		int parentVisibleHeight = parentView->visibleHeight();
		int parentTopY = parentView->frameGeometry().y();

		if(myFrameRelativeTopY < 0)
		{
			myFrameRelativeHeight += myFrameRelativeTopY;
			myFrameRelativeTopY = parentTopY;
		}

		myFrameRelativeHeight = ((myFrameRelativeTopY + myFrameRelativeHeight) > parentVisibleHeight) ? (parentVisibleHeight - myFrameRelativeTopY) : myFrameRelativeHeight;

		myFrameRelativeTopY += parentTopY;

		parentView = static_cast<FrameView*>(parentView->parent());
	}

	if(myFrameRelativeHeight > 0)
	{
		m_vAdj->setLower(myFrameRelativeTopY);
		m_vAdj->setUpper(myFrameRelativeTopY + myFrameRelativeHeight);
		m_vAdj->adjustmentChanged();
	}
}

void CVirtualCursor::setHorizontalVisibleLimits()
{
	//m_hAdj->setPage_size(contentsWidth);
	m_hAdj->setStep_increment(10);

	FrameView* currentView = m_frameView;
	FrameView* parentView = static_cast<FrameView*>(currentView->parent());
	int myFrameRelativeTopX = m_frameView->frameGeometry().x();
	int myFrameRelativeWidth = getFrameVisibleWidth();

	while(parentView)
	{
		myFrameRelativeTopX -= parentView->scrollOffset().width();
		int parentVisibleWidth = parentView->visibleWidth();
		int parentTopX = parentView->frameGeometry().x();

		if(myFrameRelativeTopX < 0)
		{
			myFrameRelativeWidth += myFrameRelativeTopX;
			myFrameRelativeTopX = parentTopX;
		}

		myFrameRelativeWidth = ((myFrameRelativeTopX + myFrameRelativeWidth) > parentVisibleWidth) ? (parentVisibleWidth - myFrameRelativeTopX) : myFrameRelativeWidth;

		myFrameRelativeTopX += parentTopX;

		parentView = static_cast<FrameView*>(parentView->parent());
	}

	if(myFrameRelativeWidth > 0)
	{
		m_hAdj->setLower(myFrameRelativeTopX);
		m_hAdj->setUpper(myFrameRelativeTopX + myFrameRelativeWidth);
		m_hAdj->adjustmentChanged();
	}
}

int CVirtualCursor::getFrameVisibleHeight()
{
	return m_frameView->visibleHeight();
}

int CVirtualCursor::getFrameVisibleWidth()
{
	return m_frameView->visibleWidth();
}

#ifdef SUPPORT_TOUCH_SCREEN
void CVirtualCursor::touchPressed(IntPoint wndPoint)
{
	if(m_frameView)
	{
		IntPoint contentsPoint = m_frameView->windowToContents(wndPoint);
		scrollBy((contentsPoint.x() - m_pointerX), (contentsPoint.y() - m_pointerY));
	}
}

void CVirtualCursor::touchMoved(IntPoint wndPoint)
{
	if(m_frameView)
	{
		IntPoint contentsPoint = m_frameView->windowToContents(wndPoint);

		FrameView* view = frameViewAtWindowPoint(wndPoint);

		scrollBy((contentsPoint.x() - m_pointerX), (contentsPoint.y() - m_pointerY));

		if(view != m_frameView)
		{
			setFrameView(view);
		}
	}
}

void CVirtualCursor::touchReleased(IntPoint wndPoint)
{
	if(m_frameView)
	{
		IntPoint contentsPoint = m_frameView->windowToContents(wndPoint);
		scrollBy((contentsPoint.x() - m_pointerX), (contentsPoint.y() - m_pointerY));
	}
}
#endif //SUPPORT_TOUCH_SCREEN

} //namespace WebCore

#endif //SUPPORT_VIRTUAL_CURSOR - Compile only if this flag is enabled
