/*
 * Copyright (C) 2007, 2008 Apple Inc. All rights reserved.
 * Copyright (C) 2008 Collabora, Ltd. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1.  Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2.  Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 * 3.  Neither the name of Apple Computer, Inc. ("Apple") nor the names of
 *     its contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE AND ITS CONTRIBUTORS "AS IS" AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL APPLE OR ITS CONTRIBUTORS BE LIABLE FOR ANY
 * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef FileSystem_h
#define FileSystem_h

#if PLATFORM(GTK)
#include <gmodule.h>
#endif

#include <time.h>

#include <wtf/Platform.h>

#if PLATFORM(SHP)
#include "FileMgr.h"
#endif // PLATFORM(SHP)

#if PLATFORM(SHP)
// Set this flag to 1 if you want to test the File APIs
#define TEST_FILE_APIS (0)
#endif // PLATFORM(SHP)

typedef const struct __CFData* CFDataRef;

namespace WebCore {

class CString;
class String;

#if PLATFORM(WIN)
typedef HANDLE PlatformFileHandle;
typedef FILETIME PlatformFileTime;
typedef HMODULE PlatformModule;
const PlatformFileHandle invalidPlatformFileHandle = INVALID_HANDLE_VALUE;
#elif PLATFORM(SHP)
typedef HFile       PlatformFileHandle;
struct              TmDateTime;
typedef TmDateTime  PlatformFileTime;
const PlatformFileHandle invalidPlatformFileHandle = -1;
typedef void* PlatformModule;
#else
typedef int PlatformFileHandle;
typedef time_t PlatformFileTime;
#if PLATFORM(GTK)
typedef GModule* PlatformModule;
#else
typedef void* PlatformModule;
#endif
const PlatformFileHandle invalidPlatformFileHandle = -1;
#endif

bool fileExists(const String&);
bool deleteFile(const String&);
bool deleteEmptyDirectory(const String&);
bool getFileSize(const String&, long long& result);
bool getFileModificationTime(const String&, time_t& result);
String pathByAppendingComponent(const String& path, const String& component);
bool makeAllDirectories(const String& path);
String homeDirectoryPath();
String pathGetFileName(const String&);

CString fileSystemRepresentation(const String&);

inline bool isHandleValid(const PlatformFileHandle& handle) { return handle != invalidPlatformFileHandle; }

// Prefix is what the filename should be prefixed with, not the full path.
CString openTemporaryFile(const char* prefix, PlatformFileHandle&);
void closeFile(PlatformFileHandle&);
int writeToFile(PlatformFileHandle, const char* data, int length);

// Methods for dealing with loadable modules
bool unloadModule(PlatformModule);

#if PLATFORM(WIN)
String localUserSpecificStorageDirectory();
String roamingUserSpecificStorageDirectory();

bool safeCreateFile(const String&, CFDataRef);
#endif

#if PLATFORM(SHP)

/*The FileMode is as follows*/

#define FILE_MODE_CREATE    (0x0001)
#define FILE_MODE_READ      (0x0002)
#define FILE_MODE_WRITE     (0x0004)
#define FILE_MODE_TRUNCATE	(0x0008)
#define FILE_MODE_APPEND    (0x0010)
#define FILE_MODE_DIR		(0x00000200)
#define FILE_MODE_READONLY	(0x00000400)
#define FILE_MODE_HIDDEN	(0x00000800)
#define FILE_MODE_SYSTEM	(0x00001000)

/* File seek Mode*/
#define	FILE_SEEK_SET	    0
#define	FILE_SEEK_CUR	    1	
#define	FILE_FM_SEEK_END	2

/*
 * @desc    openFile : function called open a file
 * @param   path     : File path
 * @param   fileName : Name of the file
 * @param   mode     : File open mode. For e.g. FILE_MODE_CREATE, FILE_MODE_READ, 
					   FILE_MODE_CREATE | FILE_MODE_READ | FILE_MODE_WRITE, etc.
 * @param   handle   : File Handle
 * @return  File path
 */
CString openFile(const char* path,const char* fileName, UINT32 mode, PlatformFileHandle& handle);

/*
 * @desc    readFile  : function called read data from a file
 * @param   handle    : File Handle
 * @param   buffer    : Buffer to read
 * @param   bufferLen : Length to be read
 * @param   bytesRead : Total length read 
 * @return  True/False
 */
bool readFile(PlatformFileHandle handle, char* buffer, long long bufferLen, unsigned int* bytesRead);

/*
 * @desc    readFileWithSeek  : Read data from a file with a seek position
 * @param   handle            : File Handle
 * @param   buffer            : Buffer to read
 * @param   bufferLen         : Length to be read
 * @param   bytesRead         : Total length read
 * @param   offset            : Seek position
 * @param   mode              : File seek mode For e.g FILE_SEEK_SET, FILE_SEEK_CUR, FILE_FM_SEEK_END.
                                The default mode is FILE_SEEK_CUR
 * @return  True/False
 */
bool readFileWithSeek(PlatformFileHandle handle, char* buffer, long long bufferLen, unsigned int* bytesRead, unsigned int offset, UINT32 mode = FILE_SEEK_CUR);

#endif

#if PLATFORM(SHP) && TEST_FILE_APIS
void testFileAPIs();
#endif // PLATFORM(SHP) && TEST_FILE_APIS

} // namespace WebCore

#endif // FileSystem_h
