/*
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "AffineTransform.h"

#include "IntRect.h"
#include "FloatRect.h"
#include "NotImplemented.h"

namespace WebCore {

static const double deg2rad = 0.017453292519943295769; // pi/180

AffineTransform::AffineTransform()
{

#if defined(SFB_GWES_TRANSFORM)
	m_xx = 1;
	m_xy = 0;
	m_tx = 0;
	m_yx = 0;
	m_yy = 1;
	m_ty = 0;    
#endif
}

AffineTransform::AffineTransform(double a, double b, double c, double d, double e, double f)
{
#if defined(SFB_GWES_TRANSFORM)
	setMatrix( a, b, c, d, e, f );
#endif

}

void AffineTransform::setMatrix(double a, double b, double c, double d, double e, double f)
{
#if defined(SFB_GWES_TRANSFORM)
	m_xx = (SFBGWESTransformType)a;
	m_xy = (SFBGWESTransformType)b;
	m_yx = (SFBGWESTransformType)c;
	m_yy = (SFBGWESTransformType)d;
	m_tx = (SFBGWESTransformType)e;
	m_ty = (SFBGWESTransformType)f;
#endif
}

#if defined(SFB_GWES_TRANSFORM)
void
affine_matrix_transform_distance (const AffineTransform *matrix, SFBGWESTransformType *dx, SFBGWESTransformType *dy)
{

    SFBGWESTransformType new_x, new_y;

    new_x = (matrix->m_xx * (SFBGWESTransformType)*dx + matrix->m_xy * (SFBGWESTransformType)*dy);
    new_y = (matrix->m_yx * (SFBGWESTransformType)*dx + matrix->m_yy * (SFBGWESTransformType)*dy);

    *dx = new_x;
    *dy = new_y;
}
#endif

void AffineTransform::map(double x, double y, double* x2, double* y2) const
{

#if defined(SFB_GWES_TRANSFORM)

	//*x2 = x;
	//*y2 = y;

	SFBGWESTransformType xx2, yy2;
	xx2 = x;
	yy2 = y;

    affine_matrix_transform_distance (this, &xx2, &yy2);

	*x2 = xx2;
	*y2 = yy2;
    *x2 += m_tx;
    *y2 += m_ty;

#else
    *x2 = x;
    *y2 = y;
    notImplemented();
#endif
}

IntRect AffineTransform::mapRect(const IntRect &rect) const
{
    FloatRect floatRect(rect);
    FloatRect enclosingFloatRect = this->mapRect(floatRect);

	enclosingFloatRect.setWidth( enclosingFloatRect.width() - 0.5f );
	enclosingFloatRect.setHeight( enclosingFloatRect.height() - 0.5f );

    return enclosingIntRect(enclosingFloatRect);
}

FloatRect AffineTransform::mapRect(const FloatRect &rect) const
{

#if defined(SFB_GWES_TRANSFORM)
	SFBGWESTransformType rectMinX = rect.x();
    SFBGWESTransformType rectMaxX = rect.x() + rect.width();
    SFBGWESTransformType rectMinY = rect.y();
    SFBGWESTransformType rectMaxY = rect.y() + rect.height();

    SFBGWESTransformType px = rectMinX;
    SFBGWESTransformType py = rectMinY;
    //cairo_matrix_transform_point(&m_transform, &px, &py);
	double ppx;
	double ppy;

	ppx = px;
	ppy = py;
	map( ppx, ppy, &ppx, &ppy );
	px = ppx;
	py = ppy;

    SFBGWESTransformType enclosingRectMinX = px;
    SFBGWESTransformType enclosingRectMinY = py;
    SFBGWESTransformType enclosingRectMaxX = px;
    SFBGWESTransformType enclosingRectMaxY = py;

    px = rectMaxX;
    py = rectMinY;
    //cairo_matrix_transform_point(&m_transform, &px, &py);
	ppx = px;
	ppy = py;
	map( ppx, ppy, &ppx, &ppy );
	px = ppx;
	py = ppy;



    if (px < enclosingRectMinX)
        enclosingRectMinX = px;
    else if (px > enclosingRectMaxX)
        enclosingRectMaxX = px;
    if (py < enclosingRectMinY)
        enclosingRectMinY = py;
    else if (py > enclosingRectMaxY)
        enclosingRectMaxY = py;

    px = rectMaxX;
    py = rectMaxY;
    //cairo_matrix_transform_point(&m_transform, &px, &py);
	ppx = px;
	ppy = py;
	map( ppx, ppy, &ppx, &ppy );
	px = ppx;
	py = ppy;

    if (px < enclosingRectMinX)
        enclosingRectMinX = px;
    else if (px > enclosingRectMaxX)
        enclosingRectMaxX = px;
    if (py < enclosingRectMinY)
        enclosingRectMinY = py;
    else if (py > enclosingRectMaxY)
        enclosingRectMaxY = py;

    px = rectMinX;
    py = rectMaxY;
    //cairo_matrix_transform_point(&m_transform, &px, &py);
	ppx = px;
	ppy = py;
	map( ppx, ppy, &ppx, &ppy );
	px = ppx;
	py = ppy;

    if (px < enclosingRectMinX)
        enclosingRectMinX = px;
    else if (px > enclosingRectMaxX)
        enclosingRectMaxX = px;
    if (py < enclosingRectMinY)
        enclosingRectMinY = py;
    else if (py > enclosingRectMaxY)
        enclosingRectMaxY = py;


    SFBGWESTransformType enclosingRectWidth = enclosingRectMaxX - enclosingRectMinX;
    SFBGWESTransformType enclosingRectHeight = enclosingRectMaxY - enclosingRectMinY;

    return FloatRect(enclosingRectMinX, enclosingRectMinY, enclosingRectWidth, enclosingRectHeight);

#else
	notImplemented();
	return rect;
#endif
}


bool AffineTransform::isIdentity() const
{
	notImplemented();
	return false;
}

double AffineTransform::a() const
{
#if defined(SFB_GWES_TRANSFORM)
	return m_xx;
#else
    return 0;
#endif
}

void AffineTransform::setA(double a)
{
	notImplemented();
}

double AffineTransform::b() const
{
#if defined(SFB_GWES_TRANSFORM)
	return m_xy;
#else
    return 0;
#endif
}

void AffineTransform::setB(double b)
{
	notImplemented();
}

double AffineTransform::c() const
{
#if defined(SFB_GWES_TRANSFORM)
	return m_yx;
#else
    return 0;
#endif
}

void AffineTransform::setC(double c)
{
    notImplemented();
}

double AffineTransform::d() const
{
#if defined(SFB_GWES_TRANSFORM)
	return m_yy;
#else
    return 0;
#endif
}

void AffineTransform::setD(double d)
{
    notImplemented();
}

double AffineTransform::e() const
{
#if defined(SFB_GWES_TRANSFORM)
	return m_tx;
#else
    return 0;
#endif
}

void AffineTransform::setE(double e)
{
    notImplemented();
}

double AffineTransform::f() const
{
#if defined(SFB_GWES_TRANSFORM)
	return m_ty;
#else
    return 0;
#endif
}

void AffineTransform::setF(double f)
{
	notImplemented();
}

void AffineTransform::reset()
{
    notImplemented();
}


#if defined(SFB_GWES_TRANSFORM)
void
affine_matrix_multiply (AffineTransform *result, const AffineTransform *a, const AffineTransform *b)
{
    AffineTransform r;

    r.m_xx = a->m_xx * b->m_xx + a->m_yx * b->m_xy;
    r.m_yx = a->m_xx * b->m_yx + a->m_yx * b->m_yy;

    r.m_xy = a->m_xy * b->m_xx + a->m_yy * b->m_xy;
    r.m_yy = a->m_xy * b->m_yx + a->m_yy * b->m_yy;

    r.m_tx = a->m_tx * b->m_xx + a->m_ty * b->m_xy + b->m_tx;
    r.m_ty = a->m_tx * b->m_yx + a->m_ty * b->m_yy + b->m_ty;

    *result = r;
}
#endif

AffineTransform &AffineTransform::scale(double sx, double sy)
{
#if defined(SFB_GWES_TRANSFORM)
    AffineTransform tmp;

    tmp.setMatrix ( sx, 0, 0, sy, 0, 0);

    affine_matrix_multiply ( this, &tmp, this );

	return *this;
#else
    notImplemented();
    return *this;

#endif
}

AffineTransform &AffineTransform::rotate(double d)
{
    notImplemented();
    return *this;
}

AffineTransform &AffineTransform::translate(double tx, double ty)
{

#if defined(SFB_GWES_TRANSFORM)
    AffineTransform tmp;

    tmp.setMatrix ( 1, 0, 0, 1, tx, ty);

    affine_matrix_multiply ( this, &tmp, this );

	return *this;
#else
    notImplemented();
    return *this;

#endif
}

AffineTransform &AffineTransform::shear(double sx, double sy)
{
    notImplemented();
    return *this;
}

double AffineTransform::det() const
{
	notImplemented();
    return 0;
}

AffineTransform AffineTransform::inverse() const
{
    notImplemented();
    return AffineTransform();
}



bool AffineTransform::operator== (const AffineTransform &m2) const
{
	notImplemented();
    return false;
}

AffineTransform &AffineTransform::operator*= (const AffineTransform &m2)
{
    notImplemented();
    return *this;
}

AffineTransform AffineTransform::operator* (const AffineTransform &m2)
{
    notImplemented();
    return AffineTransform();
}

}

// vim: ts=4 sw=4 et
