/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2007 Alp Toker <alp@atoker.com>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "GraphicsContext.h"
#include "AffineTransform.h"

#if PLATFORM(SHP)

#include "FloatRect.h"
#include "Font.h"
#include "SimpleFontData.h"
#include "IntRect.h"
#include "NotImplemented.h"
#include "Path.h"
#include "GWES.h"
#include "CMMDefs.h"
#ifdef USE_NATIVE_WIDGET_SUPPORT
#include "NativeEditField.h"
#endif

#ifndef M_PI
#define M_PI 3.14159265358979323846
#endif

#define DOT_COUNT	1
#define SPACE_COUNT 1

using std::swap;

namespace WebCore {

struct GraphicsContextPlatformState {
    GraphicsContextPlatformState()
    : m_xShift(0)
	, m_yShift(0)
#ifdef SFB_GWES_TRANSFORM
	, m_CTM( 1, 0, 0, 1, 0, 0 ) // identity matrix
#endif
    {}
	int m_xShift;
	int m_yShift;
	IntRect m_clipRect;

#ifdef SFB_GWES_TRANSFORM
	AffineTransform	m_CTM;
#endif

};

	
class GraphicsContextPlatformPrivate 
CMM_MODIFIED
{
public:
    GraphicsContextPlatformPrivate();
    ~GraphicsContextPlatformPrivate();

    HGC* cr;
	HWIN* hWin;
    Vector<float> layers;
	GraphicsContextPlatformState state;
	Vector<GraphicsContextPlatformState> stack;
#ifdef SFB_PROGRESSIVE_RENDERING
	WmBitmap* placeholderImage; // not-owned
#endif // SFB_PROGRESSIVE_RENDERING
};


GraphicsContextPlatformPrivate::GraphicsContextPlatformPrivate()
    :  cr(0), hWin(0)
#ifdef SFB_PROGRESSIVE_RENDERING
    ,placeholderImage(NULL)
#endif // SFB_PROGRESSIVE_RENDERING
{
}

GraphicsContextPlatformPrivate::~GraphicsContextPlatformPrivate()
{
}


GraphicsContext::GraphicsContext(PlatformGraphicsContext* cr, PlatformWindowContext* hWin)
    : m_common(createGraphicsContextPrivate())
    , m_data(new GraphicsContextPlatformPrivate)
{
    m_data->cr = cr;
    setPaintingDisabled(!cr);

	m_data->hWin = hWin;
}

void GraphicsContext::setPlatformContext(PlatformGraphicsContext* cr)
{
	m_data->cr = cr;
	setPaintingDisabled(!cr);
}

void GraphicsContext::setWindowHandle(PlatformWindowContext* hWin)
{
	m_data->hWin = hWin;
}


GraphicsContext::~GraphicsContext()
{
    destroyGraphicsContextPrivate(m_common);
    delete m_data;
	m_data = NULL;
}

HGC* GraphicsContext::platformContext() const
{
    return m_data->cr;
}

HWIN* GraphicsContext::getWindowHandle() const
{
	return m_data->hWin;
}

void GraphicsContext::savePlatformState()
{
       /*HQ fix start 19 Aug 2008*/
	WmRect rect;
	if( !WmGetClipRect( *(m_data->cr), &rect ) )
	{
		//assert
		SysNeverGetHere();
	}

	m_data->state.m_clipRect.setX( rect.x );
	m_data->state.m_clipRect.setY( rect.y );
	m_data->state.m_clipRect.setWidth( rect.w );
	m_data->state.m_clipRect.setHeight( rect.h );
        /*HQ fix end 19 Aug 2008*/

	m_data->stack.append(m_data->state);
}

void GraphicsContext::restorePlatformState()
{
    if (m_data->stack.isEmpty()) {
        LOG_ERROR("ERROR void GraphicsContext::restore() stack is empty");
        /*HQ fix  19 Aug 2008*/
	SysNeverGetHere();
        return;
    }
    m_data->state = m_data->stack.last();
        /*HQ fix start 19 Aug 2008*/
	int x, y, w, h;
	x = m_data->state.m_clipRect.x();
	y = m_data->state.m_clipRect.y();
	w = m_data->state.m_clipRect.width();
	h = m_data->state.m_clipRect.height();

    if(w > 0 && h > 0)
    {
	if( WmSetClipRect( 	*(m_data->cr), x, y, w, h ) == CLIP_REJECT )
		SysNeverGetHere();
    }

    m_data->stack.removeLast();
	//WmSetClipRect(*(m_data->cr), 0, 0, 2000, 2000);
 /*HQ fix end 19 Aug 2008*/
}

static inline void setColor(HGC hGC, const Color& col)
{
    WmSetFgColor(hGC, RGB565Ex( col.rgb() ) );
}

// Draws a filled rectangle with a stroked border.
void GraphicsContext::drawRect(const IntRect& ori_rect)
{
        
	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );
	
	if( ori_rect.width() == 1 && rect.width() == 0 )
		rect.setWidth( 1 );

	if( ori_rect.height() == 1 && rect.height() == 0 )
		rect.setHeight( 1 );
#endif

    if (paintingDisabled() || 0 >= rect.width() || 0 >= rect.height() )
        return;

    HGC hGC = *m_data->cr;
    
	StrokeStyle strokeStyle = this->strokeStyle();
	setPlatformStrokeStyle(strokeStyle);

	float strokeThickness = 0;

	// SHP limitation - we can draw dotted line only if the thickness is 1
	if (strokeStyle == DottedStroke)
		strokeThickness = 1.00;
	else if (strokeStyle != NoStroke)
		strokeThickness = this->strokeThickness();

	setPlatformStrokeThickness(strokeThickness);
	setColor(hGC, strokeColor());

	if (fillColor().alpha())
	{
		WmDrawFilledRect(hGC, RGB565Ex( fillColor().rgb() ),
						 translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
	}
	else
	{
		WmDrawRect (hGC, translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
	}
}

#ifdef _SFB_PAGE_PREVIEW	//vamsi Mar 03 - Page Overview
// Draws a filled rectangle with a stroked border.
void GraphicsContext::drawBorderRect(const IntRect& ori_rect)
{
	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );
#endif

    if (paintingDisabled() || 0 == rect.width() || 0 == rect.height() )
        return;

    HGC hGC = *m_data->cr;
       
	StrokeStyle strokeStyle = this->strokeStyle();
	setPlatformStrokeStyle(strokeStyle);

	float strokeThickness = 0;

	// SHP limitation - we can draw dotted line only if the thickness is 1
	if (strokeStyle == DottedStroke)
		strokeThickness = 1.00;
	else if (strokeStyle != NoStroke)
		strokeThickness = this->strokeThickness();

	setPlatformStrokeThickness(strokeThickness);
	//Color Color(255, 0, 0);
	//setColor(hGC, strokeColor());
	setColor(hGC, Color(255, 0, 0));

	WmDrawRect (hGC, translateX(rect.x() + 1), translateY(rect.y() + 1), (rect.width() - 2), (rect.height() - 2));
	WmDrawRect (hGC, translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
	setColor(hGC, strokeColor());
}
#endif

// FIXME: Now that this is refactored, it should be shared by all contexts.
static void adjustLineToPixelBoundaries(FloatPoint& p1, FloatPoint& p2, float strokeWidth, StrokeStyle style)
{

#ifdef SFB_GWES_TRANSFORM
	//p1 = m_data->state.m_CTM.mapPoint( p1 );
	//p2 = m_data->state.m_CTM.mapPoint( p2 );
	SysNeverGetHere();
#endif

    // For odd widths, we add in 0.5 to the appropriate x/y so that the float arithmetic
    // works out.  For example, with a border width of 3, KHTML will pass us (y1+y2)/2, e.g.,
    // (50+53)/2 = 103/2 = 51 when we want 51.5.  It is always true that an even width gave
    // us a perfect position, but an odd width gave us a position that is off by exactly 0.5.
    if (style == DottedStroke || style == DashedStroke) {
        if (p1.x() == p2.x()) {
            p1.setY(p1.y() + strokeWidth);
            p2.setY(p2.y() - strokeWidth);
        }
        else {
            p1.setX(p1.x() + strokeWidth);
            p2.setX(p2.x() - strokeWidth);
        }
    }

    if (static_cast<int>(strokeWidth) % 2) {
        if (p1.x() == p2.x()) {
            // We're a vertical line.  Adjust our x.
            p1.setX(p1.x() + 0.5);
            p2.setX(p2.x() + 0.5);
        }
        else {
            // We're a horizontal line. Adjust our y.
            p1.setY(p1.y() + 0.5);
            p2.setY(p2.y() + 0.5);
        }
    }
}

// This is only used to draw borders.
void GraphicsContext::drawLine(const IntPoint& ori_point1, const IntPoint& ori_point2)
{
        
	IntPoint point1 = ori_point1;
	IntPoint point2 = ori_point2;
#ifdef SFB_GWES_TRANSFORM
	point1 = m_data->state.m_CTM.mapPoint( point1 );
	point2 = m_data->state.m_CTM.mapPoint( point2 );
#endif

	StrokeStyle strokeStyle = this->strokeStyle();

    if (paintingDisabled())
        return;
	
	setPlatformStrokeStyle(strokeStyle);

	if (strokeStyle == DottedStroke)
		setPlatformStrokeThickness(1.00);
	else
		setPlatformStrokeThickness(strokeThickness());

	setColor(*(m_data->cr), strokeColor());
	//verifying the value of hGC before passing it to WmDrawLine
	if (( *(m_data->cr) != INVALID_HOBJ) && (WmVerifyGC(*(m_data->cr)) == TRUE ))
	{	if(translateX(point1.x()) != translateX(point2.x()))
		WmDrawLine(*(m_data->cr), translateX(point1.x()), translateY(point1.y()), translateX(point2.x()), translateY(point2.y()));
		else
	  {
		if(translateY(point1.y()) !=  translateY(point2.y()))
		WmDrawLine(*(m_data->cr), translateX(point1.x()), translateY(point1.y()), translateX(point2.x()), translateY(point2.y()));
	  }
	}
}

// This method is only used to draw the little circles used in lists.
void GraphicsContext::drawEllipse(const IntRect& ori_rect)
{

	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );
#endif

    if (paintingDisabled() || 0 >= rect.width() || 0 >= rect.height()  )
        return;

    HGC hGC = *(m_data->cr);

	StrokeStyle strokeStyle = this->strokeStyle();
	setPlatformStrokeStyle(strokeStyle);

	float strokeThickness = 0;

	// SHP limitation - we can draw dotted line only if the thickness is 1
	if (strokeStyle == DottedStroke)
		strokeThickness = 1.00;
	else if (strokeStyle != NoStroke)
		strokeThickness = this->strokeThickness();

	setPlatformStrokeThickness(strokeThickness);
	setColor(hGC, strokeColor());

	if (fillColor().alpha()) 
	{
		WmDrawFilledEllipse (hGC, RGB565Ex( fillColor().rgb() ),
							 translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
    }
	else
	{
		WmDrawEllipse(hGC, translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
	}
}

// FIXME: This function needs to be adjusted to match the functionality on the Mac side.
void GraphicsContext::strokeArc(const IntRect& ori_rect, int startAngle, int angleSpan)
{
	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );
#endif

    if (paintingDisabled() || 0 >= rect.width() || 0 >= rect.height()  )
        return;

    HGC hGC = *(m_data->cr);

	StrokeStyle strokeStyle = this->strokeStyle();
	setPlatformStrokeStyle(strokeStyle);

	float strokeThickness = 0;

	// SHP limitation - we can draw dotted line only if the thickness is 1
	if (strokeStyle == DottedStroke)
		strokeThickness = 1.00;
	else if (strokeStyle != NoStroke)
		strokeThickness = this->strokeThickness();

	setPlatformStrokeThickness(strokeThickness);
	setColor(hGC, strokeColor());

	WmDrawArc(hGC, translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height(),
			  startAngle, angleSpan, ARC_ONLY);
}

void GraphicsContext::drawConvexPolygon(size_t npoints, const FloatPoint* points, bool shouldAntialias)
{
    if (paintingDisabled())
        return;

    HGC hGC = *(m_data->cr);
	WmPoint* pPoint = (WmPoint*) MemoryManager::Alloc( sizeof(WmPoint) * npoints);

	for(unsigned int i = 0; i < npoints; i++ )
	{

		pPoint[i].x = translateX(points[i].x());
		pPoint[i].y = translateY(points[i].y());

#ifdef SFB_GWES_TRANSFORM
		FloatPoint tempPoint;
		tempPoint = m_data->state.m_CTM.mapPoint(  points[i]  );
		pPoint[i].x = tempPoint.x();
		pPoint[i].y = tempPoint.y();
#endif


		//vamsi - Google edit field overlap on main indicator issue
		if(pPoint[i].x < 0) pPoint[i].x = 0;
		if(pPoint[i].y < 0) pPoint[i].y = 0;
	}

	WmSetAntialiasingFlag( hGC, shouldAntialias );

	COLOR color = BLACK;

    if (fillColor().alpha()) 
	{
		color = RGB565Ex(fillColor().rgb());
	}

	if (strokeStyle() != NoStroke) 
	{
		color = RGB565Ex(strokeColor().rgb());
	}

	WmDrawFilledPolygon( hGC, color, pPoint, npoints );

    MemoryManager::Free(pPoint);
	pPoint = NULL;
}

void GraphicsContext::fillRect(const IntRect& ori_rect, const Color& color)
{

	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );
#endif

    if (paintingDisabled() || 0 >= rect.width() || 0 >= rect.height() )
        return;
	WmGC* pGC = WmGetGCPointer(*m_data->cr);
	if(pGC->gcAttrib.bitsPerPixel != 32)
	{
		
			if (color.alpha())
			{
				WmDrawFilledRect(*m_data->cr, RGB565Ex( color.rgb() ),
								 translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
			}
		
	}
	else
	{
		NativeEditField* pNativeEdit = NativeEditField::getNativeEditFieldInstance();
		if(!pNativeEdit->m_isEditFeild )
		{
			if (color.alpha())
				{
					WmDrawFilledRect(*m_data->cr, RGB565Ex( color.rgb() ),
									 translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
				}
		}
		else
		{
			if (color.alpha())
			{
				WmDrawFilledRect32(*m_data->cr,  color.rgb() ,
									 translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
			}
		}
		pNativeEdit->m_isEditFeild = false;
	}
}

void GraphicsContext::fillRect(const FloatRect& ori_rect, const Color& color)
{

	FloatRect rect = ori_rect;

#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );
#endif

    if (paintingDisabled()  || 0 >= (int) rect.width() || 0 >= (int) rect.height()  )
        return;

    if (color.alpha())
	{
		WmDrawFilledRect(*m_data->cr, RGB565Ex( color.rgb() ),
						 translateX(rect.x()), translateY(rect.y()), (int)rect.width(), 
						 (int)rect.height());
	}
}

void GraphicsContext::clip(const IntRect& ori_rect)
{

	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );
	if( ori_rect.width() > 0 && rect.width() <= 0 )
		rect.setWidth( 1 );
	if( ori_rect.height() > 0 && rect.height() <= 0 )
		rect.setHeight( 1 );
//for iframe
	WmRect oldClip;
	WmGetClipRect( *(m_data->cr), &oldClip );
	IntRect rcOldClip( oldClip.x, oldClip.y, oldClip.w, oldClip.h );
	rect.intersect( rcOldClip );

#endif

    if (paintingDisabled()  || 0 >= rect.width() || 0 >= rect.height() )
        return;

	WmClip clipRes;

	clipRes = WmSetClipRect(*(m_data->cr), translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
#ifdef SFB_GWES_TRANSFORM
	if( clipRes == CLIP_REJECT )
		WmSetClipRect(*(m_data->cr), translateX(rcOldClip.x()), translateY(rcOldClip.y()), rcOldClip.width(), rcOldClip.height());
#endif

}

void GraphicsContext::clipTextBox(const IntRect& ori_rect)
{
	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );

	WmRect oldClip;
	WmGetClipRect( *(m_data->cr), &oldClip );
	IntRect rcOldClip( oldClip.x, oldClip.y, oldClip.w, oldClip.h );

#endif

    if (paintingDisabled()  || 0 >= rect.width() || 0 >= rect.height() )
        return;

	WmClip clipRes;
	
	clipRes = WmSetClipRect(*(m_data->cr), translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());

#ifdef SFB_GWES_TRANSFORM
	if( clipRes == CLIP_REJECT )
		WmSetClipRect(*(m_data->cr), translateX(rcOldClip.x()), translateY(rcOldClip.y()), rcOldClip.width(), rcOldClip.height());
#endif
}

void GraphicsContext::setClipBounds(const IntRect& ori_rect)
{
	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );

	WmRect oldClip;
	WmGetClipRect( *(m_data->cr), &oldClip );
	IntRect rcOldClip( oldClip.x, oldClip.y, oldClip.w, oldClip.h );
#endif

    if (paintingDisabled()  || 0 >= rect.width() || 0 >= rect.height() )
        return;

	WmClip clipRes;
	clipRes = WmSetClipRect(*(m_data->cr), rect.x(), rect.y(), rect.width(), rect.height());

#ifdef SFB_GWES_TRANSFORM
	if( clipRes == CLIP_REJECT )
		WmSetClipRect(*(m_data->cr), translateX(rcOldClip.x()), translateY(rcOldClip.y()), rcOldClip.width(), rcOldClip.height());
#endif

}

bool GraphicsContext::extendedPoint(IntPoint point1, IntPoint point2, IntPoint point3, IntPoint point4, IntPoint& crossingPoint)
{	
	if(point3.x() == point4.x() && point1.y() == point2.y())
	{
		crossingPoint = IntPoint(point4.x(), point2.y());
		return true;		
	}

	if(point1.x() == point2.x() && point3.y() == point4.y())
	{
		crossingPoint = IntPoint(point1.x(), point3.y());
		return true;		
	}

	return false;
}

bool GraphicsContext::crossingPoint(IntPoint point1, IntPoint point2, IntPoint point3, IntPoint point4, IntPoint& crossingPoint)
{
	if(point1.y()!= point2.y())
	{
        swap(point1, point3);
        swap(point2, point4);		
    }
    if(point2.x() < point1.x())
        swap(point1, point2);
    if(point4.y() < point3.y())
        swap(point3, point4);
    if((point1.x() <= point3.x() && point3.x() <= point2.x()) &&
        point3.y() <= point1.y() && point1.y() <= point4.y() ) 
	{
        crossingPoint = IntPoint(point3.x(), point1.y());
        return true;
    }

	// Now find any extended point
    return extendedPoint(point1, point2, point3, point4, crossingPoint);
}

bool GraphicsContext::drawFocusRingLine(IntPoint ori_point1, IntPoint ori_point2, const Color& color)
{
	IntPoint point1 = ori_point1;
	IntPoint point2 = ori_point2;
        
#ifdef SFB_GWES_TRANSFORM
	point1 = m_data->state.m_CTM.mapPoint( point1 );
	point2 = m_data->state.m_CTM.mapPoint( point2 );
#endif

	if (paintingDisabled())
        return false;

	if(point1 == point2)
		return false;

	if(point1.x() == point2.x() )
	{
		if(point1.y() > point2.y())
			return false;
	}

	if(point1.y() == point2.y())
	{
		if(point1.x() > point2.x())
			return false;
	}

	//Get the platform stroke thikness
	float oldStrokeThikness = strokeThickness();

	//Set thickness for the ring
	setStrokeThickness(2);// focus ring strokethickness change

	setColor(*(m_data->cr), color);
	//verifying the value of hGC before passing it to WmDrawLine
	if (( *(m_data->cr) != INVALID_HOBJ) && (WmVerifyGC(*(m_data->cr)) == TRUE ))
	{	if(translateX(point1.x()) != translateX(point2.x()))
	WmDrawLine(*(m_data->cr), translateX(point1.x()), translateY(point1.y()), translateX(point2.x()), translateY(point2.y()));
		else
	{
	if(translateY(point1.y()) !=  translateY(point2.y()))
	WmDrawLine (*(m_data->cr), translateX(point1.x()), translateY(point1.y()), translateX(point2.x()), translateY(point2.y()) );
	}
	}

	
	//Restore the platform thikness
	setStrokeThickness(oldStrokeThikness);
	return true;
}

void GraphicsContext::drawFocusRing(const Color& color)
{
    if (paintingDisabled())
        return;

	// Get the rect for any focussable elements, except text of an anchor element.
	// We will get the rect for the text of anchor element from GraphicsContext::anchorRects
	Vector<IntRect> rects = focusRingRects();
	
#ifdef ENABLE_FOCUS_RING_DRAWING

	// Get the rects for the text of anchor element
	Vector<IntRect> linkRect = anchorRects();

	//Merge the anchor rect and any other focus ring rects ( rect for image etc...)
	for( int i = 0; i < linkRect.size(); i++)
	{
		bool rectAdded = false;
		IntRect temp = linkRect[i];

		// If 2 rects are on same line , merge them
		for(int j = 0; j < rects.size(); j++)
		{
			if((temp.y() <= rects[j].y() && temp.bottom() >= rects[j].bottom()) || 
				(temp.y() >= rects[j].y() && temp.bottom() <= rects[j].bottom()))
			{
				temp.unite(rects[j]);
				rects[j] = temp;
				rectAdded = true;
				break;
			}
		}

		if(!rectAdded)
			rects.append(temp);
	}

	int rectCount = rects.size();
    
	int padding = focusRingWidth();
	int inflateVal = 4;	
	IntRect temp;
	
	//Sort the rects based on y co-ordinates(rect having less y value on top)
	int min = 0;
	
	if(rectCount > 2 )
	{
		for(int i = 0; i < rectCount - 2 ; i++)
		{
			min = i;
			for(int j = i+1;j <  rectCount - 1 ; j++)
			{
				if(rects[j].y() < rects[min].y())
				{
					min = j;

					//swap values
					temp = rects[i];
					rects[i] = rects[min];
					rects[min] = temp;
				}
			}
		}
	}
	else if(rectCount == 2)
	{
		if(rects[0].y() > rects[1].y())
		{
			temp = rects[0];
			rects[0] = rects[1];
			rects[1] = temp;
		}
	}	

	//Draw the focus ring for focussable elements
	for(int i = 0; i < rects.size() ; i++)
	{
		IntRect currRect = rects[i];

		int inflatVal = 1;//focus ring inflate change
		currRect.inflate(inflatVal);

		bool lineDrawn = false;
		bool retVal = false;

		IntPoint temp(0 ,0);
		IntPoint crossPoint(0, 0);
		IntRect nextRect(0, 0, 0, 0);
		IntRect prevRect(0, 0, 0, 0);

		//Top Horizontal Line
		if(i == 0)
		{
			retVal = drawFocusRingLine(currRect.topLeftPoint(), currRect.topRightPoint(), color);
		}
		else
		{
			prevRect = rects[i-1];
			prevRect.inflate(inflatVal);

			// Find for any intersection points with left vertical line of prev rect
			if(crossingPoint(currRect.topLeftPoint(), currRect.topRightPoint(), prevRect.topLeftPoint(), prevRect.bottomLeftPoint(), crossPoint))
			{
				if(currRect.right() < prevRect.x())
					crossPoint.setX(currRect.right());

				if(drawFocusRingLine(currRect.topLeftPoint(), crossPoint, color))
					lineDrawn = true;
			}

			//Now find any point of intersection with right vertical line of prev rect
			if(crossingPoint(currRect.topLeftPoint(), currRect.topRightPoint(), prevRect.topRightPoint(), prevRect.bottomRightPoint(), crossPoint))
			{
				if(drawFocusRingLine(crossPoint, currRect.topRightPoint(), color))
					lineDrawn = true;
			}
			if(!lineDrawn && currRect.topRightPoint().x() < prevRect.x())
			{
				retVal = drawFocusRingLine(currRect.topLeftPoint(), currRect.topRightPoint(), color);
				lineDrawn = false;
			}
		}

		//Bottom Horizontal Line
		if(i == rects.size() - 1 )
		{
			if(rects.size() > 1)
			{				
				prevRect = rects[i-1];
				prevRect.inflate(inflatVal);

				if(currRect.bottom() > prevRect.bottom())
				{
					retVal = drawFocusRingLine(currRect.bottomLeftPoint(), currRect.bottomRightPoint(), color);
				}
				else
				{
					// Find for any intersection points with left vertical line of prev rect
					if(crossingPoint(currRect.bottomLeftPoint(), currRect.bottomRightPoint(), prevRect.topLeftPoint(), prevRect.bottomLeftPoint(), crossPoint))
					{
						if(drawFocusRingLine(currRect.bottomLeftPoint(), crossPoint, color))
							lineDrawn = true;
					}

					//Now find any point of intersection with right vertival line of prev rect
					if(crossingPoint(currRect.bottomLeftPoint(), currRect.bottomRightPoint(), prevRect.topRightPoint(), prevRect.bottomRightPoint(), crossPoint))
					{
						if(drawFocusRingLine(crossPoint, currRect.bottomRightPoint(), color))
							lineDrawn = true;
					}
				}
			}
			else
			{
				retVal = drawFocusRingLine(currRect.bottomLeftPoint(), currRect.bottomRightPoint(), color);
			}
		}
		else
		{
			nextRect = rects[i+1];
			nextRect.inflate(inflatVal);

			// Find for any intersection points with left vertical line of next rect
			if(crossingPoint(currRect.bottomLeftPoint(), currRect.bottomRightPoint(), nextRect.topLeftPoint(), nextRect.bottomLeftPoint(), crossPoint))
			{
				if(drawFocusRingLine(currRect.bottomLeftPoint(), crossPoint, color))
					lineDrawn = true;
			}

			//Now find any point of intersection with right vertival line of next rect
			if(crossingPoint(currRect.bottomLeftPoint(), currRect.bottomRightPoint(), nextRect.topRightPoint(), nextRect.bottomRightPoint(), crossPoint))
			{
				if(nextRect.right() < currRect.x())
					crossPoint.setX(currRect.x());

				if(drawFocusRingLine(crossPoint, currRect.bottomRightPoint(), color))
					lineDrawn = true;
			}			
		}

		//Left Vertical Line
		if(rects.size() > 1)
		{
			if( i == rects.size() - 1)
			{
				prevRect = rects[i-1];
				prevRect.inflate(inflatVal);
				
				// Find for any intersection points with bottom horizontal line of prev rect
				if(crossingPoint(currRect.topLeftPoint(), currRect.bottomLeftPoint(), prevRect.bottomLeftPoint(), prevRect.bottomRightPoint(), crossPoint))
				{
					if(currRect.x() < prevRect.x() || currRect.right() < prevRect.x())
					{
						if(crossPoint.y() != currRect.y())
							crossPoint.setY(currRect.y());
					}
					retVal = drawFocusRingLine(crossPoint, currRect.bottomLeftPoint(), color);
				}			
			}
			else
			{
				if(i > 0)
				{
					prevRect = rects[i-1];
					prevRect.inflate(inflatVal);
				}

				nextRect = rects[i+1];
				nextRect.inflate(inflatVal);

				// Find for any intersection points with top horizontal line of next rect
				if(crossingPoint(currRect.topLeftPoint(), currRect.bottomLeftPoint(), nextRect.topLeftPoint(), nextRect.topRightPoint(), crossPoint))
				{
					temp = currRect.topLeftPoint();

					if(!(currRect.x() > nextRect.x() && currRect.x() < nextRect.right()))
					{
						if(crossPoint.y() != currRect.bottom())
							crossPoint.setY(currRect.bottom());
					}

					if(currRect.x() > prevRect.x() && currRect.x() < prevRect.right())
					{
						if(temp.y() != prevRect.bottom())
							temp.setY(prevRect.bottom());
					}

					retVal = drawFocusRingLine(temp, crossPoint, color);
				}

				// Find for any intersection points with bottom horizontal line of next rect
				if(crossingPoint(currRect.topLeftPoint(), currRect.bottomLeftPoint(), nextRect.bottomLeftPoint(), nextRect.bottomRightPoint(), crossPoint))
				{
					retVal = drawFocusRingLine(crossPoint, currRect.bottomLeftPoint(), color);
				}
			}
		}
		else
		{
			retVal = drawFocusRingLine(currRect.topLeftPoint(), currRect.bottomLeftPoint(), color);
		}

		//Right vertical Line
		if(rects.size() > 1 )
		{
			if( i == rects.size() - 1)
			{				
				prevRect = rects[i-1];
				prevRect.inflate(inflatVal);

				// Find for any intersection points with bottom horizontal line of prev rect
				if(crossingPoint(currRect.topRightPoint(), currRect.bottomRightPoint(), prevRect.bottomLeftPoint(), prevRect.bottomRightPoint(), crossPoint))
				{
					if(currRect.right() > prevRect.right() || currRect.right() < prevRect.x())
					{
						if(crossPoint.y() != currRect.y())
							crossPoint.setY(currRect.y());
					}
					retVal = drawFocusRingLine(crossPoint, currRect.bottomRightPoint(), color);
				}
			}
			else
			{
				if(i > 0)
				{
					prevRect = rects[i-1];
					prevRect.inflate(inflatVal);
				}

				nextRect = rects[i+1];
				nextRect.inflate(inflatVal);

				// Find for any intersection points with top horizontal line of next rect			
				if(crossingPoint(currRect.topRightPoint(), currRect.bottomRightPoint(), nextRect.topLeftPoint(), nextRect.topRightPoint(), crossPoint))
				{
					temp = currRect.topRightPoint();

					if(!(currRect.right() > nextRect.x() && currRect.right() < nextRect.right()))
					{
						if(crossPoint.y() != currRect.bottom())
							crossPoint.setY(currRect.bottom());
					}

					if(currRect.right() > prevRect.x() && currRect.right() < prevRect.right())
					{
						if(temp.y() != prevRect.bottom())
							temp.setY(prevRect.bottom());
					}

					retVal = drawFocusRingLine(temp, crossPoint, color);
				}

				// Find for any intersection points with bottom horizontal line of next rect
				if(crossingPoint(currRect.topRightPoint(), currRect.bottomRightPoint(), nextRect.bottomLeftPoint(), nextRect.bottomRightPoint(), crossPoint))
				{
					retVal = drawFocusRingLine(crossPoint, currRect.bottomRightPoint(), color);
				}
			}
		}
		else
		{
			retVal = drawFocusRingLine(currRect.topRightPoint(), currRect.bottomRightPoint(), color);
		}		
	}

	//Clear the anchor rects
	clearAnchorRect();

#endif //ENABLE_FOCUS_RING_DRAWING

#ifdef ENABLE_FOCUS_RECT_DRAWING
	int inflateVal = 1;
	int rectCount = rects.size();
	
	for(int i = 0; i < rectCount ; i++)
	{
		IntRect currRect = rects[i];
		currRect.inflate(inflateVal);
		bool retVal = false;

		//Top Horizontal Line
		retVal = drawFocusRingLine(currRect.topLeftPoint(), currRect.topRightPoint(), color);
		
		//Bottom Horizontal Line
		retVal = drawFocusRingLine(currRect.bottomLeftPoint(), currRect.bottomRightPoint(), color);

		//Left Vertical Line
		retVal = drawFocusRingLine(currRect.topLeftPoint(), currRect.bottomLeftPoint(), color);

		//Right vertical Line
		retVal = drawFocusRingLine(currRect.topRightPoint(), currRect.bottomRightPoint(), color);
	}
#endif //ENABLE_FOCUS_RECT_DRAWING
}

#if 0
void GraphicsContext::setFocusRingClip(const IntRect& rect)
{
	WmSetClipRect(*(m_data->cr), translateX(rect.x()), translateY(rect.y()), rect.width(), rect.height());
}

void GraphicsContext::clearFocusRingClip()
{
	WmSetClipRect(*(m_data->cr), 0, 0, 1000, 1000);
}
#endif

void GraphicsContext::drawLineForText(const IntPoint& origin, int width)
{
    if (paintingDisabled())
        return;

    IntPoint endPoint = origin + IntSize(width, 0);
    drawLine(origin, endPoint);
}

void GraphicsContext::drawLineForMisspellingOrBadGrammar(const IntPoint&, int width, bool grammar)
{
    notImplemented();
}

FloatRect GraphicsContext::roundToDevicePixels(const FloatRect& frect)
{
    notImplemented();
	return frect;
}

void GraphicsContext::translate(float x, float y)
{
    if (paintingDisabled())
        return;

#ifdef SFB_GWES_TRANSFORM
	m_data->state.m_CTM.translate( x, y );	
#else
	m_data->state.m_xShift += x;
	m_data->state.m_yShift += y;
#endif
}

IntPoint GraphicsContext::origin()
{
	notImplemented();
    return IntPoint(0, 0);
}

void GraphicsContext::setPlatformFillColor(const Color& col)
{
    // FIXME: this is probably a no-op but I'm not sure
    // notImplemented(); // commented-out because it's chatty and clutters output
}

void GraphicsContext::setPlatformStrokeColor(const Color& col)
{
    // FIXME: this is probably a no-op but I'm not sure
    //notImplemented(); // commented-out because it's chatty and clutters output
}

void GraphicsContext::setPlatformStrokeThickness(float strokeThickness)
{
    if (paintingDisabled() || 0 >= strokeThickness )
        return;

	WmSetLineWidth (*m_data->cr, (int)strokeThickness);
}

void GraphicsContext::setPlatformStrokeStyle(const StrokeStyle& strokeStyle)
{
    if (paintingDisabled())
        return;

    switch (strokeStyle) {
    case NoStroke: //fall throu
        // FIXME: We are currently ignoring the NoStroke condition?
    case SolidStroke:
        WmSetLineType(*m_data->cr, LINE_SOLID );
        break;
    case DottedStroke:
		WmSetLineType(*m_data->cr, LINE_DOT);
        WmSetDotLineInfo(*m_data->cr, DOT_COUNT, SPACE_COUNT);
        break;
    case DashedStroke:
		{
			UCHAR pattern[20];
			AcMemset( pattern, 0, sizeof( pattern ) );
			char mask = ( char )0x80;
			int index = 0;
			int length = strokeThickness() * 3;

			for ( int i = 0; i < strokeThickness()*2; i++ )
			{
				pattern[index] |= ( char )( mask >> ( i % 8 ) );
				if ( i > 0 && ( i % 8 ) == 0 ) 
					index++;
			}
		if( length > 0 && (length < (WM_DASH_LINE_PATTERN_ARRAY_MAX * 8) ))
		{
			WmSetLineType(*m_data->cr, LINE_DASH );
			WmSetDashLineInfo(*m_data->cr, length, pattern);
		}
		}
        break;
    default:
        notImplemented();
        break;
    }
}

void GraphicsContext::setPlatformFont(const Font& font)
{
    if (paintingDisabled())
        return;

    const SimpleFontData *fontData = font.primaryFont();
    fontData->setFont(m_data->cr);
}

void GraphicsContext::setURLForRect(const KURL& link, const IntRect& destRect)
{
    notImplemented();
}

void GraphicsContext::concatCTM(const AffineTransform& transform)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::addInnerRoundedRectClip(const IntRect& rect, int thickness)
{
    notImplemented();
}

void GraphicsContext::setShadow(IntSize const&, int, Color const&)
{
    notImplemented();
}

void GraphicsContext::clearShadow()
{
    notImplemented();
}

void GraphicsContext::beginTransparencyLayer(float opacity)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::endTransparencyLayer()
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::clearRect(const FloatRect& ori_rect)
{

	FloatRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = m_data->state.m_CTM.mapRect( rect );	
#endif



    if (paintingDisabled()  || 0 >= (int) rect.width() || 0 >= (int) rect.height() )
        return;
	
	WmClearGCRectArea(*m_data->cr, translateX(rect.x()), translateY(rect.y()), 
					  (int)rect.width(), (int)rect.height());
}

void GraphicsContext::strokeRect(const FloatRect&, float)
{
    notImplemented();
}

void GraphicsContext::setLineCap(LineCap lineCap)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::setLineJoin(LineJoin lineJoin)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::setMiterLimit(float miter)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::setAlpha(float)
{
    notImplemented();
}

void GraphicsContext::setCompositeOperation(CompositeOperator op)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::beginPath()
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::addPath(const Path& path)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::clip(const Path& path)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::clipOut(const Path&)
{
    notImplemented();
}

void GraphicsContext::rotate(float radians)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

void GraphicsContext::scale(const FloatSize& size)
{
    if (paintingDisabled())
        return;

#ifdef SFB_GWES_TRANSFORM

	m_data->state.m_CTM.scale( size.width(), size.height() );
#else
    notImplemented();
#endif
}

void GraphicsContext::clipOut(const IntRect&)
{
    notImplemented();
}

void GraphicsContext::clipOutEllipseInRect(const IntRect&)
{
    notImplemented();
}

void GraphicsContext::fillRoundedRect(const IntRect&, const IntSize& topLeft, const IntSize& topRight, const IntSize& bottomLeft, const IntSize& bottomRight, const Color&)
{
    notImplemented();
}
IntPoint GraphicsContext::translatePoint(IntPoint& p)
{
#ifdef SFB_GWES_TRANSFORM
	return p;
#else
	return IntPoint(p.x() + m_data->state.m_xShift, p.y() + m_data->state.m_yShift);
#endif
}

int GraphicsContext::translateX(int x)
{

#ifdef SFB_GWES_TRANSFORM
	return x;
#else
	return x + m_data->state.m_xShift;
#endif
}

int GraphicsContext::translateY(int y)
{
#ifdef SFB_GWES_TRANSFORM
	return y;
#else
	return y + m_data->state.m_yShift;
#endif
}

#ifdef SFB_PROGRESSIVE_RENDERING
void GraphicsContext::setPlaceholderImage(WmBitmap* image)
{
	m_data->placeholderImage = image;
}

WmBitmap* GraphicsContext::getPlaceHolderImage() const
{
	return m_data->placeholderImage;
}
#endif // SFB_PROGRESSIVE_RENDERING

void GraphicsContext::paintBuffer(ImageBuffer* buffer, const IntRect& r)
{
    if (paintingDisabled())
        return;

    notImplemented();
}

AffineTransform GraphicsContext::getCTM() const
{

#ifdef SFB_GWES_TRANSFORM

	return m_data->state.m_CTM;
#else
    notImplemented();
    return AffineTransform();
#endif
}


} // namespace WebCore

#endif // PLATFORM(SHP)
