/*
 * Copyright (C) 2006 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2006 Michael Emmel mike.emmel@gmail.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "Font.h"

#include "SimpleFontData.h"
#include "GraphicsContext.h"
#include "NotImplemented.h"
#include "GlyphBuffer.h"
#include "GWES.h"

#ifdef SFB_GWES_TRANSFORM
#include "AffineTransform.h"
#endif

namespace WebCore {

void Font::drawGlyphs(GraphicsContext* graphicsContext, const SimpleFontData* font, const GlyphBuffer& glyphBuffer,
                      int from, int numGlyphs, const FloatPoint& original_point) const
{

	FloatPoint point =original_point ;
#ifdef SFB_GWES_TRANSFORM
	point = graphicsContext->getCTM().mapPoint( original_point );	

	FontPlatformData fontData = font->platformData();
	float fontsize = fontData.m_fontAttr.iSize;
	fontsize *= graphicsContext->getCTM().a();
	fontData.m_fontAttr.iSize = fontsize;

#endif

	HGC* context = graphicsContext->platformContext();

    // Set the text color to use for drawing.
    Color penColor = graphicsContext->fillColor();
	WmSetFgColor(*context, RGB565Ex( penColor.rgb() ) );
	
#ifdef SFB_GWES_TRANSFORM
    SFBalGwesSetFontAttr(fontData.m_fontAttr);
    HFNT hFont = SFBalGwesGetHFontSreenInfo();
	WmSetFont(*context, hFont);
	WmSetFontUnderline(*context, graphicsContext->textUnderline());
    int fontHeight = WmGetFontMaxHeight(hFont);
    int fontDescent = WmGetFontDescender( hFont, fontHeight );
#else
    // This was commented out as it made "some text invisible" but seems to work now.
    font->setFont(context);
#endif // SFB_GWES_TRANSFORM


    GlyphBufferGlyph* glyphs = (GlyphBufferGlyph*) glyphBuffer.glyphs(from);

	MCHAR *text = (MCHAR*) MemoryManager::Alloc( sizeof(MCHAR) * (numGlyphs + 1) );

	AcMemcpy(text, glyphs, ( sizeof(MCHAR) * numGlyphs) );
	text[numGlyphs] = (MCHAR)('\0');

    int offset = point.x();
    for(int i = 0; i < numGlyphs; i++)
    {
#ifndef SFB_GWES_TRANSFORM
		WmDrawTextN(*context, graphicsContext->translateX(offset), graphicsContext->translateY(point.y()) - font->xHeight() + font->descent(), (text + i), 1);
        offset += glyphBuffer.advanceAt(from + i);
#else
		WmDrawTextN(*context, graphicsContext->translateX(offset), graphicsContext->translateY(point.y()) - fontHeight + fontDescent, (text + i), 1);
		//float advanceAt = glyphBuffer.advanceAt(from + i);
		//advanceAt *= (float)(graphicsContext->getCTM().a());
		//offset += advanceAt;

		int	width, height;
		WmGetTextExtentN(hFont, &text[i], 1, &width, &height);
		offset = offset + width + m_letterSpacing;

#endif
    }

    MemoryManager::Free(text);
	text = NULL;
}

void Font::drawComplexText(GraphicsContext* context, const TextRun& run, const FloatPoint& point, int from, int to) const
{

    // This glyph buffer holds our glyphs+advances+font data for each glyph.
    GlyphBuffer glyphBuffer;

    float startX = point.x();
    WidthIterator it(this, run);
    it.advance(from);
    float beforeWidth = it.m_runWidthSoFar;
    it.advance(to, &glyphBuffer);
    
    // We couldn't generate any glyphs for the run.  Give up.
    if (glyphBuffer.isEmpty())
        return;
    
    float afterWidth = it.m_runWidthSoFar;

    if (run.rtl()) {
        float finalRoundingWidth = it.m_finalRoundingWidth;
        it.advance(run.length());
        startX += finalRoundingWidth + it.m_runWidthSoFar - afterWidth;
    } else
        startX += beforeWidth;

    // Swap the order of the glyphs if right-to-left.
    if (run.rtl())
        for (int i = 0, end = glyphBuffer.size() - 1; i < glyphBuffer.size() / 2; ++i, --end)
            glyphBuffer.swap(i, end);

    // Calculate the starting point of the glyphs to be displayed by adding
    // all the advances up to the first glyph.
    FloatPoint startPoint(startX, point.y());
    drawGlyphBuffer(context, glyphBuffer, run, startPoint);
}

FloatRect Font::selectionRectForComplexText(const TextRun&, const IntPoint&, int, int, int) const
{
	notImplemented();
	return FloatRect();
}

float Font::floatWidthForComplexText(const TextRun& run) const
{
    GlyphBuffer glyphBuffer;
    WidthIterator it(this, run);

    it.advance(run.length(), &glyphBuffer);

    return it.m_runWidthSoFar;
}

int Font::offsetForPositionForComplexText(const TextRun&, int, bool) const
{
	notImplemented();
	return 0;
}

}
