/*
 * This file is part of the WebKit project.
 *
 * Copyright (C) 2006 Apple Computer, Inc.
 * Copyright (C) 2006 Michael Emmel mike.emmel@gmail.com 
 * Copyright (C) 2007 Holger Hans Peter Freyther
 * Copyright (C) 2007 Alp Toker <alp.toker@collabora.co.uk>
 * All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 *
 */

#include "config.h"
#include "RenderThemeShp.h"

#include "NotImplemented.h"
#include "RenderObject.h"

#include "RsrcMgr.h" 
#include "WsRsrcCheck.h"
#include "WsRsrcRadio.h"
#include "WsRsrcButton.h"
#include "SFBrowserRsrc.h"
#include "GWES.h"
#include "SFBDrawWidget.h"

#define SFB_RSRC_FILENAME "SFBrowserRsrc.rbm"
//#define COLOR_HIGHLIGHT RGB565Ex( 0x00FF9900 )

#define THEME_COLOR 204
#define THEME_FONT  210

// Button constants
#define BP_BUTTON    1
#define BP_RADIO     2
#define BP_CHECKBOX  3

// Textfield constants
#define TFP_TEXTFIELD 1
#define TFS_READONLY  6
#define SfbMin(X, Y)  ((X) < (Y) ? (X) : (Y))
/*
 * Approach to theming:
 *  a) keep one copy of each to be drawn widget, GtkEntry, GtkButton, Gtk...
 *     + the button will look like the native control
 *     + we don't need to worry about style updates and loading the right GtkStyle
 *     - resources are wasted. The native windows will not be used, we might have issues
 *       with
 *
 *  b) Use GtkStyle directly and copy and paste Gtk+ code
 *
 *
 * We will mix a and b
 *
 * - Create GtkWidgets to hold the state (disabled/enabled), selected, not selected.
 * - Use a GdkPixmap to make the GtkStyle draw to and then try to convert set it the
 *   source of the current operation.
 *
 */

namespace WebCore {

#ifdef SFB_SUPPORT_IMAGE_ZOOM
    extern float g_zoomMultiplier;
#endif

RenderTheme* theme()
{
    static RenderThemeShp shpTheme;
    return &shpTheme;
}

RenderThemeShp::RenderThemeShp()
{
}

void RenderThemeShp::close()
{
}

void RenderThemeShp::addIntrinsicMargins(RenderStyle* style) const
{
    // Cut out the intrinsic margins completely if we end up using a small font size
    if (style->fontSize() < 11)
        return;

    // Intrinsic margin value.
    const int m = 2;

    // FIXME: Using width/height alone and not also dealing with min-width/max-width is flawed.
    if (style->width().isIntrinsicOrAuto()) {
        if (style->marginLeft().quirk())
            style->setMarginLeft(Length(m, Fixed));
        if (style->marginRight().quirk())
            style->setMarginRight(Length(m, Fixed));
    }

    if (style->height().isAuto()) {
        if (style->marginTop().quirk())
            style->setMarginTop(Length(m, Fixed));
        if (style->marginBottom().quirk())
            style->setMarginBottom(Length(m, Fixed));
    }
}

bool RenderThemeShp::supportsFocus(EAppearance appearance)
{
    switch (appearance) {
        case PushButtonAppearance:
        case ButtonAppearance:
        case TextFieldAppearance:
            return true;
        default:
            return false;
    }
}

static WmBitmap* loadBitmap(int nvItemId, int rsrcId)
{
	char pszFile[64]= "";
	NvGetString(nvItemId, pszFile);

	return __WsLoadBitmap(pszFile, rsrcId);
}

void RenderThemeShp::setCheckboxSize(RenderStyle* style) const 
{ 
    setRadioSize(style);
}

bool RenderThemeShp::paintCheckbox(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& ori_rect)
{

	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = i.context->getCTM().mapRect( rect );	
#endif
    // FIXME: is it the right thing to do?
	
	char		 szFileSetupWizard[FM_FILEPATH_LEN_MAX];
	WmBitmap* pBitmap = NULL;

	int rsrcId = IDB_SFB_CHECKBOX_OFF;
	if ( isEnabled(o) )
	{
		if ( isChecked(o) )
			rsrcId = IDB_SFB_CHECHBOX_ON;
	}
	else
	{
		if ( isChecked(o) ) 
			rsrcId = IDB_CHECK_ON_DIM;
		else
			rsrcId = IDB_CHECK_OFF_DIM;
	}

    IntRect zoomedRect = rect;
#ifdef SFB_SUPPORT_IMAGE_ZOOM
    zoomedRect.setWidth(rect.width() * g_zoomMultiplier);
    zoomedRect.setHeight(rect.height() * g_zoomMultiplier);
#endif


	//pBitmap = loadBitmap( NV_SI_BITMAP_FILE_NAME_CHECK, rsrcId ); 
	pBitmap = RsrcLoadBitmap( SFB_RSRC_FILENAME, rsrcId );
	if (pBitmap)
	{
		WmDrawScaledBitmap( *((i.context)->platformContext()), i.context->translateX(rect.x()), i.context->translateY(rect.y()),
						zoomedRect.width(), zoomedRect.height(), pBitmap );
		__WsFreeBitmap(pBitmap);
	}

    return false;
}

void RenderThemeShp::setRadioSize(RenderStyle* style) const 
{ 
    notImplemented(); 
    // If the width and height are both specified, then we have nothing to do.
    if (!style->width().isIntrinsicOrAuto() && !style->height().isAuto())
        return;


    // FIXME:  A hard-coded size of 13 is used.  This is wrong but necessary for now.  It matches Firefox.
    // At different DPI settings on Windows, querying the theme gives you a larger size that accounts for
    // the higher DPI.  Until our entire engine honors a DPI setting other than 96, we can't rely on the theme's
    // metrics.
    const int ff = 13;
    if (style->width().isIntrinsicOrAuto())
        style->setWidth(Length(ff, Fixed));

    if (style->height().isAuto())
        style->setHeight(Length(ff, Fixed));
}

bool RenderThemeShp::paintRadio(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& ori_rect)
{ 
	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = i.context->getCTM().mapRect( rect );	
#endif

    // FIXME: is it the right thing to do?

	char		 szFileSetupWizard[FM_FILEPATH_LEN_MAX];
	WmBitmap* pBitmap = NULL;
	int height=0;
	int width=0;

	int rsrcId = IDB_SFB_RADIO_BUTTON_OFF;
	if ( isEnabled(o) )
	{
		if ( isChecked(o) )
			rsrcId = IDB_SFB_RADIO_BUTTON_ON;
			
	}
	else
	{
		
		if ( isChecked(o) ) 
			rsrcId = IDB_SFB_RADIO_BUTTON_ON;
		else
			rsrcId = IDB_SFB_RADIO_BUTTON_OFF;
	}

	//pBitmap = loadBitmap( NV_SI_BITMAP_FILE_NAME_RADIO, rsrcId ); 
	//NvGetString( NV_SI_BITMAP_FILE_NAME_SETUP_WIZARD, szFileSetupWizard );
	
	//pBitmap = __WsLoadBitmap( szFileSetupWizard, rsrcId );
	pBitmap = RsrcLoadBitmap( SFB_RSRC_FILENAME, rsrcId );

	if(rect.width()!=rect.height())
		width=height = SfbMin(rect.width(),rect.height()) ;
	else
	{
		width=rect.width();
		height=rect.height();
	}
#ifdef SFB_SUPPORT_IMAGE_ZOOM
    width *= g_zoomMultiplier;
    height *= g_zoomMultiplier;
#endif

	WmDrawScaledBitmap( *((i.context)->platformContext()), i.context->translateX(rect.x()), i.context->translateY(rect.y()),
						width, height, pBitmap );
	if (pBitmap && (isPressed(o)))
	{
	/*WmDrawFilledRect(*((i.context)->platformContext()),COLOR_HIGHLIGHT, (i.context->translateX(rect.x())-5), i.context->translateY(rect.y()),
						(rect.width()+10), rect.height());*/

		WmDrawScaledBitmap( *((i.context)->platformContext()), i.context->translateX(rect.x()), i.context->translateY(rect.y()),
						width, height, pBitmap );
			
	}
__WsFreeBitmap(pBitmap);
    return false;
}

bool RenderThemeShp::paintButton(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& ori_rect) 
{ 
	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = i.context->getCTM().mapRect( rect );	
#endif

	HGC hGC;
	
	int x1,y1,width,height,x2,y2;
    // FIXME: is it the right thing to do?
	hGC=*((i.context)->platformContext());
		
	x1=i.context->translateX(rect.x());
	y1=i.context->translateY(rect.y());

	width=rect.width();
	height=rect.height();

	x2=(width+x1);
	y2=(y1+height);

	SFBDrawButton(hGC, x1,  y1,  x2,  y2, "", 0, BLACK);
	if (isPressed(o))
		SFBDrawButton(hGC, x1,  y1,  x2,  y2, "", 1, WHITE);
	
		
		
	/*WmBitmap* pBitmap = NULL;

	int rsrcId = IDB_BUTTON_GRAY_SIDE_TOP;
	if (isPressed(o))
		rsrcId = IDB_BUTTON_GRAY_SIDE_TOP_DOWN;

	pBitmap = loadBitmap( NV_SI_BITMAP_FILE_NAME_BUTTON, rsrcId ); 

	if (pBitmap)
	{
		
		WmDrawScaledBitmap( *((i.context)->platformContext()), i.context->translateX(rect.x()), i.context->translateY(rect.y()),
						rect.width(), rect.height(), pBitmap );
		__WsFreeBitmap(pBitmap);
	}*/

    return false;
}

void RenderThemeShp::adjustTextFieldStyle(CSSStyleSelector*, RenderStyle*, Element* e) const 
{ 
    notImplemented(); 
}

bool RenderThemeShp::paintTextField(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& r)
{
    // FIXME: should use theme-aware drawing
	notImplemented();
    return true;
}

bool RenderThemeShp::paintTextArea(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& r)
{
    return paintTextField(o, i, r);
}

void RenderThemeShp::adjustButtonStyle(CSSStyleSelector* selector, RenderStyle* style, WebCore::Element* e) const
{
    addIntrinsicMargins(style);
}

void RenderThemeShp::systemFont(int propId, FontDescription&) const
{
}

void RenderThemeShp::adjustMenuListStyle(CSSStyleSelector*, RenderStyle*, Element*) const
{
}

bool RenderThemeShp::paintMenuList(RenderObject* o, const RenderObject::PaintInfo& i, const IntRect& ori_rect) 
{ 
	WmBitmap* pBitmap = NULL;
   	IntRect rect = ori_rect;
#ifdef SFB_GWES_TRANSFORM
	rect = i.context->getCTM().mapRect( rect );	
#endif
	WmGC* pGC = WmGetGCPointer(*((i.context)->platformContext()));

	int rectX = i.context->translateX(rect.x());
	int rectY = i.context->translateY(rect.y());
	int endX = rectX + rect.width();
	int endY = rectY + rect.height();
	/*Sneha:To fix Bug id 1711*/
	COLOR color=RGB565Ex(o->selectionForegroundColor().rgb());
	// Partho : Fix for CQ 2372
	COLOR bcolor = RGB565Ex(o->style()->borderTopColor().rgb());
	int bwidth = o->style()->borderTopWidth();
	//applying bg colour to menulist(CQ 2373)
	if(!color)
		color=RGB565Ex(o->style()->backgroundColor().rgb());
	
	pBitmap=RsrcLoadBitmap(SFB_RSRC_FILENAME,IDB_SFB_COMBO_BOX_BORDER);
	// Partho : Fix for CQ 2372
	if(bwidth)
		WmSetLineWidth(*((i.context)->platformContext()), bwidth );
	else
	    WmSetLineWidth(*((i.context)->platformContext()), 1);
	WmSetLineType(*((i.context)->platformContext()), LINE_SOLID );
	/*Draw rectangle*/
	// Partho : Fix for CQ 2372
	if(bcolor)
		WmSetFgColor( *((i.context)->platformContext()), bcolor);
	else
	    WmSetFgColor( *((i.context)->platformContext()), BLACK ); 


//verifying the value of hGC before passing it to WmDrawLine
	if (( *((i.context)->platformContext()) != INVALID_HOBJ) && (WmVerifyGC(*((i.context)->platformContext())) == TRUE ))
	{
		if(rectY != endY)
		WmDrawLine( *((i.context)->platformContext()), rectX, rectY, rectX, endY );  
		if(rectY != endY)
		WmDrawLine( *((i.context)->platformContext()), endX, rectY, endX, endY );  
		if(rectX != endX )
		WmDrawLine( *((i.context)->platformContext()), rectX, rectY, endX, rectY );
		if(rectX != endX )
		WmDrawLine( *((i.context)->platformContext()), rectX, endY, endX, endY );
		if(rectY != endY )
		WmDrawLine( *((i.context)->platformContext()), endX - SHAPE_EDITFIELD_COMBO_RECT_WIDTH, rectY, endX - SHAPE_EDITFIELD_COMBO_RECT_WIDTH, endY );  
	}

	endX -= SHAPE_EDITFIELD_COMBO_RECT_WIDTH;
	/* draw triangle */
	WmDrawScaledBitmap(*((i.context)->platformContext()),endX, rectY, SHAPE_EDITFIELD_COMBO_RECT_WIDTH,rect.height()+1,pBitmap);
	if(pGC->gcAttrib.bitsPerPixel != 32)
	{
		if(!color)
			WmDrawFilledRect(*((i.context)->platformContext()),WHITE,i.context->translateX(rect.x()) + 1,i.context->translateY(rect.y()) + 1,(rect.width()-SHAPE_EDITFIELD_COMBO_RECT_WIDTH) - 2,rect.height() - 2);
		else
			WmDrawFilledRect(*((i.context)->platformContext()),color,i.context->translateX(rect.x()) + 1,i.context->translateY(rect.y()) + 1,(rect.width()-SHAPE_EDITFIELD_COMBO_RECT_WIDTH) - 2,rect.height() - 2);
	}
	else
	{
		if(!color)
			WmDrawFilledRect32(*((i.context)->platformContext()),0xffffffff,i.context->translateX(rect.x()) + 1,i.context->translateY(rect.y()) + 1,(rect.width()-SHAPE_EDITFIELD_COMBO_RECT_WIDTH) - 2,rect.height() - 2);
		else
			WmDrawFilledRect32(*((i.context)->platformContext()),o->style()->backgroundColor().rgb(),i.context->translateX(rect.x()) + 1,i.context->translateY(rect.y()) + 1,(rect.width()-SHAPE_EDITFIELD_COMBO_RECT_WIDTH) - 2,rect.height() - 2);
	}


	__WsFreeBitmap(pBitmap);
	return false;

}

void RenderThemeShp::adjustMenuListButtonStyle(CSSStyleSelector*, RenderStyle*, Element*) const
{
}

bool RenderThemeShp::paintMenuListButton(RenderObject*, const RenderObject::PaintInfo&, const IntRect&) 
{ 
	return true; 
}

#ifdef _SFB_SEARCH_STRING_IN_PAGE //Jan-16 Vamsi/Pramod Default colors are not good
Color RenderThemeShp::platformActiveSelectionBackgroundColor() const
{
    // Use red color by default if the platform theme doesn't define anything.
    return Color(255, 0, 0);
}

Color RenderThemeShp::platformInactiveSelectionBackgroundColor() const
{
    // Use orange color by default if the platform theme doesn't define anything.
    return Color(255, 165, 0);
}
#endif

}
