/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		TextBreakIteratorShp.cpp
 * @brief		Used by Jasmine Browser to iterate through text. (characters, words, lines etc.)
 *@author	Zafar Hussain (zafar.h@samsung.com)
 * @version		1.0
 * 
 */

#include "config.h"
#if USE(SHP_UNICODE)
#include "TextBreakIterator.h"
#include "GWES.h"
#include "ShpJasmine.h"
#include "allocs.h"

#if PLATFORM(SHP)
#include "CMMDefs.h"
#endif // PLATFORM(SHP)

namespace WebCore 
{

    class TextBreakIterator
        CMM_MODIFIED
    {
    public:
        virtual int first() = 0;
        virtual int next() = 0;
        virtual int previous() = 0;
        inline int following(int pos) {
            currentPos = pos;
            return next();
        }
        inline int preceding(int pos) {
            currentPos = pos;
            return previous();
        }
        int currentPos;
        const UChar *string;
        int length;
    };

    class WordBreakIterator : public TextBreakIterator
    {
    public:
        virtual int first();
        virtual int next();
        virtual int previous();
    };

//I have implemented only '\n' as the line break character.
//Practically, I doubt if it will any time be used as we can't enter the '\n' through mobile phones.
//Also I doubt if it will any time be used as QT has not impelemented it.
	class LineBreakIterator : public TextBreakIterator
    {
    public:
        virtual int first();
        virtual int next();
        virtual int previous();
    };
//I have implemented only '\n' as the line break character.
//Practically, I doubt if it will any time be used as QT has not impelemented it.
	class SentenceBreakIterator : public TextBreakIterator
    {
    public:
        virtual int first();
        virtual int next();
        virtual int previous();
    };
	/*
	ZAFAR: This class is created with the sole purpose of meeting the needs of LATIN charset.
	It is assumed that each character is of sizeof(UChar) in length.
	This is done since currently we are supporting only 2 byte characters.
	Wikipedia: (About UTF-8)
	UTF-8 encodes each character in one to four octets (8-bit bytes):

   1. One byte is needed to encode the 128 US-ASCII characters (Unicode range U+0000 to U+007F).
   2. Two bytes are needed for Latin letters with diacritics and for characters from Greek, Cyrillic, Armenian, Arabic, Syriac and Thaana alphabets (Unicode range U+0080 to U+07FF).
   3. Three bytes are needed for the rest of the Basic Multilingual Plane (which contains virtually all characters in common use).
   4. Four bytes are needed for characters in the other planes of Unicode, which are rarely used in practice.
	*/
	
	
	class TextLayout
	{
	public:
		TextLayout():m_pStr(NULL)
		{
			//m_pStr=NULL;
		}

		~TextLayout()
		{
			SFBalMemFree(m_pStr);
		}
		bool setText(UChar *Strg,int length)
		{
			if(Strg==NULL||length==0)
				return false;
			int len=sizeof(UChar)*(length+1);						
			if(m_pStr)
			{
				SFBalMemFree(m_pStr);
				m_pStr=NULL;
			}
			m_pStr=(UChar *) SFBalMemAlloc(len);			
			if(NULL==m_pStr)
				return false;
			m_pStr[length]=NULL;			
			len=len-sizeof(UChar);
			AcMemcpy(m_pStr,Strg,len);
			return true;
		}
		int nextCursorPosition(int CurPos)
		{
			CurPos++;
			return CurPos;
		}
		int previousCursorPosition(int PrevPos)
		{
			PrevPos--;
			return PrevPos;
		}

	private:
		UChar *m_pStr;		

	};

    class CharBreakIterator : public TextBreakIterator
    {
    public:
        virtual int first();
        virtual int next();
        virtual int previous();
        TextLayout layout;
    };

	static CharBreakIterator *charIterator = 0;
    static WordBreakIterator *wordIterator = 0;
    static LineBreakIterator *lineIterator = 0;
    static SentenceBreakIterator *sentenceIterator = 0;

#ifdef SFB_GLOBAL_STATIC_HANDLING
	void initTextBreakIteratorShp()
	{
	    charIterator = 0;
	    wordIterator = 0;
	    lineIterator = 0;
	    sentenceIterator = 0;
	}

	void deInitTextBreakIteratorShp()
	{
		if(charIterator)
			delete charIterator;
		if(wordIterator)
			delete wordIterator;
		if(lineIterator)
			delete lineIterator;
		if(sentenceIterator)
			delete sentenceIterator;
	}
#endif


	TextBreakIterator* characterBreakIterator(const UChar* string, int length)
	{
        
	    if (!charIterator)
			charIterator = new CharBreakIterator;
	
	    if(!charIterator->layout.setText((UChar *)string,length))
		{
			//delete iterator;
			return NULL;
		}
	    charIterator->string = string;
	    charIterator->length = length;
	    charIterator->currentPos = 0;  
	    return charIterator;
	}   

    int CharBreakIterator::first() {
        currentPos = 0;
        return currentPos;
    }

	int CharBreakIterator::next() {
        if (currentPos == length)
            return -1;
        currentPos = layout.nextCursorPosition(currentPos);
        return currentPos;
    }
    int CharBreakIterator::previous() {
        if (currentPos == 0)
            return -1;
        currentPos = layout.previousCursorPosition(currentPos);
        return currentPos;
    }



	TextBreakIterator* wordBreakIterator(const UChar* string, int length)
	{
	    if (!wordIterator)
			wordIterator = new WordBreakIterator;
	
	    wordIterator->string = string;
	    wordIterator->length = length;
	    wordIterator->currentPos = 0;
	
	    return wordIterator;
	}

	int WordBreakIterator::first() 
	{
		currentPos = 0;
		return currentPos;
	}

    int WordBreakIterator::next() 
	{
        if (currentPos == length) 
		{
            currentPos = -1;
            return currentPos;
        }
        bool haveSpace = false;
        while (currentPos < length) 
		{
            if (haveSpace && !WTF::Unicode::isWideCharSpace(string[currentPos]))
                break;
			  if (WTF::Unicode::isWideCharSpace(string[currentPos]))
                haveSpace = true;
            ++currentPos;
        }
        return currentPos;
    }
    int WordBreakIterator::previous() 
	{
        if (currentPos == 0) 
		{
            currentPos = -1;
            return currentPos;
        }
        bool haveSpace = false;
        while (currentPos > 0) 
		{
            if (haveSpace && !WTF::Unicode::isWideCharSpace(string[currentPos]))
                break;
			  if (WTF::Unicode::isWideCharSpace(string[currentPos]))
                haveSpace = true;
            --currentPos;
        }
        return currentPos;
    }
//I have implemented only '\n' as the line break character.
	TextBreakIterator* lineBreakIterator(const UChar* string, int length)
	{
	    if (!lineIterator)
			lineIterator = new LineBreakIterator;

	    lineIterator->string = string;
	    lineIterator->length = length;
	    lineIterator->currentPos = 0;
	
		return lineIterator;
}

//I have implemented only '\n' as the line break character.
	int LineBreakIterator::first() 
	{
        currentPos = 0;
        return currentPos;
    }

//I have implemented only '\n' as the line break character.
	int LineBreakIterator::next() 
	{
        if (currentPos == length) 
		{
            currentPos = -1;
            return currentPos;
        }
        bool haveLiineBreak = false;
        while (currentPos < length) 
		{
            if (haveLiineBreak && !(string[currentPos]=='\n'))
                break;
			  if (string[currentPos]=='\n')
                haveLiineBreak = true;
            ++currentPos;
        }
        return currentPos;
    }

//I have implemented only '\n' as the line break character.
    int LineBreakIterator::previous() 
	{
        if (currentPos == 0) 
		{
            currentPos = -1;
            return currentPos;
        }
        bool haveLiineBreak= false;
        while (currentPos > 0) 
		{
            if (haveLiineBreak && !(string[currentPos]=='\n'))
                break;
			  if (string[currentPos]=='\n')
                haveLiineBreak = true;
            --currentPos;
        }
        return currentPos;
    }



//I have implemented only a single '.' as the sentence break character.
	TextBreakIterator* sentenceBreakIterator(const UChar* string, int length)
	{
		if (!sentenceIterator)
			sentenceIterator = new SentenceBreakIterator ;

		sentenceIterator->string = string;
		sentenceIterator->length = length;
		sentenceIterator->currentPos = 0;

	    return sentenceIterator;
	}

//I have implemented only a single '.' as the sentence break character.
	int SentenceBreakIterator ::first() 
	{
        currentPos = 0;
        return currentPos;
    }

//I have implemented only a single '.' as the sentence break character.
	int SentenceBreakIterator ::next() 
	{
        if (currentPos == length) 
		{
            currentPos = -1;
            return currentPos;
        }
        bool haveLiineBreak = false;
        while (currentPos < length) 
		{
            if (haveLiineBreak && !(string[currentPos]=='.'))
                break;
			  if (string[currentPos]=='.')
                haveLiineBreak = true;
            ++currentPos;
        }
        return currentPos;
    }

//I have implemented only a single '.' as the sentence break character.
    int SentenceBreakIterator ::previous() 
	{
        if (currentPos == 0) 
		{
            currentPos = -1;
            return currentPos;
        }
        bool haveLiineBreak= false;
        while (currentPos > 0) 
		{
            if (haveLiineBreak && !(string[currentPos]=='.'))
                break;
			  if (string[currentPos]=='.')
                haveLiineBreak = true;
            --currentPos;
        }
        return currentPos;
    }


	int textBreakFirst(TextBreakIterator* bi)
	{
		if(!bi)
			return -1;
	    return bi->first();
	}

	int textBreakNext(TextBreakIterator* bi)
	{	
		if(!bi)
			return -1;
		return bi->next();
	}

	int textBreakPreceding(TextBreakIterator* bi, int pos)
	{
		if(!bi)
			return -1;
		return bi->preceding(pos);
	}

	int textBreakFollowing(TextBreakIterator* bi, int pos)
	{
		if(!bi)
			return -1;
		return bi->following(pos);
	}

	int textBreakCurrent(TextBreakIterator* bi)
	{
		if(!bi)
			return -1;
		return bi->currentPos;
	}

	bool isTextBreak(TextBreakIterator *bi, int pos)
	{
		if(!bi)
			return false;
		else//TBD: To be implemented.
		return true;
	}

}
#endif 
