#ifndef LOADLISTENERS_H
#define LOADLISTENERS_H

//  INCLUDES
#include "BridgeTypes.h"
#include "ContentListener.h"
#include "ResourceLoader.h"

// CONSTANTS

// MACROS

// DATA TYPES

// FUNCTION PROTOTYPES

// FORWARD DECLARATIONS
class UrlResponseInfo;
class IContentLoader;
class WebKitLoader;

#ifdef SUPPORT_CONTENT_DOWNLOADER
class ContentDownloadListener;
#endif // SUPPORT_CONTENT_DOWNLOADER
// CLASS DECLARATION

/**
*
*  @lib resLoader.lib
*/
class PageLoadListener : public IContentListener
    {
    public:  // Constructors and destructor

        /**
        * Constructor
        */
        PageLoadListener( WebKitLoader* _webKitLoader, char* requetUrl );

        /**
        * Destructor.
        */
        virtual ~PageLoadListener();

    public: // new functions

        WebCore::ResourceLoader* GetResourceLoader() const
            { 
            return m_resourceLoader.get(); 
            }
        
        void SetResourceLoader( WebCore::ResourceLoader* _resourceLoader )
            {
            m_resourceLoader = _resourceLoader;
            }

#ifdef SUPPORT_CONTENT_DOWNLOADER
    private: // new functions

        eContentDownloadType GetDownloadType( WebCore::String contentType );
#endif // SUPPORT_CONTENT_DOWNLOADER

    public: // from IContentListener


        /**
        * This function is called when the header information is available
        * @param _transactionId Observed transaction id.
        * @param _headers Header information
        * @return ErrNotSupported if it is not a native browser content
        */
        virtual eErrorCode Headers( int _transactionId, UrlResponseInfo* _headers );

        /**
        * This function is called when a new partial body is available
        * @param _transactionId Observed transaction id.
        * @param _response Response information
        * @return
        */
        virtual void Response( int _transactionId, UrlResponseInfo* _response );

        /**
        * This function is called when the transaction is complete
        * @param _transactionId Observed transaction id.
        * @param _error HTTP error (404) / cancel error
        * @return
        */
        virtual void Complete( int _transactionId, UrlResponseInfo* _response, eErrorCode _error );

        void HandleError( int /*aTransactionId*/, int _error );


    protected:
        /**
        * This function is to check if WebCore is able to render a particular
        * MIME type
        * @param _mimeType
        * @return bool - returns true if WebCore is able to render a MIME type
        * returns false otherwise.
        */
        bool CanShowMIMEType(const WebCore::String& _mimeType) const;


        /**
        * This function is called to get a dummy HTML source which asks plugin to rebder
        * the data that cannot be rendered by WebCore
        * @param _requestUrl - The URL to be requested by the plugin
        * @param _mimeType - The MIME type of the plugin
        * @return WebCore::String - returns the HTML source to be sent to WebCore
        * @Sample HTML:
        * <HTML><BODY>
        * <EMBED width="100%" height="100%" name="plugin" src="xyz.swf" type="application/x-shockwave-flash" />
        * </BODY></HTML>
        */
        WebCore::String CreatePluginHTMLSource( const WebCore::String& _requestUrl,
                                                const WebCore::String& _MIMEType );

    protected:    // Data

        WebKitLoader*               m_webKitLoader;   // not owned
        RefPtr<WebCore::ResourceLoader>     m_resourceLoader; // not owned
		WebCore::String m_requestUrl;
		int m_totalBodyLen;

#ifdef SUPPORT_CONTENT_DOWNLOADER
        // ContentDownloader handle
        ContentDownloadListener*     m_contentDownloader; // owned
#endif // SUPPORT_CONTENT_DOWNLOADER
    };

/**
*
*  @lib resLoader.lib
*/
class ResourceLoadListener : public IContentListener
    {
    public:  // Constructors and destructor

        /**
        * Constructor
        */
        ResourceLoadListener( WebKitLoader* _webKitLoader );

        /**
        * Destructor.
        */
        virtual ~ResourceLoadListener();

    public: // New functions

        WebCore::ResourceLoader* GetResourceLoader() const
            { 
            return m_resourceLoader.get(); 
            }
        
        void SetResourceLoader( WebCore::ResourceLoader* _resourceLoader )
            {
            m_resourceLoader = _resourceLoader;
            }

        /**
        *
        * @param
        * @return
        */
        inline void SetTransactionId( int _transId ) { m_transId = _transId; }

    public: // from IContentListener

        /**
        *
        * @param
        * @return KErrNotSupported if it is not a native browser content
        */
        eErrorCode Headers( int _transactionId, UrlResponseInfo* _response );

        /**
        *
        * @param
        * @return
        */
        void Response( int _transactionId, UrlResponseInfo* _response );

        /**
        *
        * @param
        * @return
        */
        void Complete( int _transactionId, UrlResponseInfo* _response, eErrorCode _error );

        void HandleError( int /*aTransactionId*/, int _error );
        
    private:    // Data

        //
        RefPtr<WebCore::ResourceLoader>     m_resourceLoader;  // not owned
        //
        WebKitLoader*               m_webKitLoader;   // not owned
        //
        Vector<char*>               m_data;
        //
        Vector<unsigned>            m_dataLength;
        //
        unsigned                    m_totalLength;
        //
        int                         m_transId;

    };

#ifdef SUPPORT_CONTENT_DOWNLOADER
class ContentDownloadListener : public IContentListener
    {
    public:  // Constructors and destructor

        /**
        * Constructor
        */
		ContentDownloadListener( WebKitLoader* _webKitLoader,
                                int _transactionId,
                                eContentDownloadType _cdType );

        /**
        * Destructor.
        */
		virtual ~ContentDownloadListener();

    public: // New functions

		/**
        *
        * @param
        * @return
        */
        inline void SetTransactionId( int _transId ) { m_transId = _transId; }

		/**
        *
        * @param
        * @return
        */
        inline int GetTransactionId() { return m_transId; }

        /**
        *
        * @param
        * @return
        */
        inline WebKitLoader* GetLoader() { return m_webKitLoader; }

    public: // from IContentListener

        /**
        *
        * @param
        * @return KErrNotSupported if it is to be 
		*			handled completely by the plugin manager
        */
        eErrorCode Headers( int _transactionId, UrlResponseInfo* _response );

        /**
        *
        * @param
        * @return
        */
        void Response( int _transactionId, UrlResponseInfo* _response );

        /**
        *
        * @param
        * @return
        */
        void Complete( int _transactionId, UrlResponseInfo* _response, eErrorCode _error );


        void HandleError( int _transactionId, int _error );
        
    private:    // Data

        //
        int						    m_transId;
		//
		TransportHandle             m_transportHandle;
        // Holds the type of the download 
        eContentDownloadType        m_CDType;
        // Data store for MIDP and OMA download
        WebCore::String             m_collatedData;
        // Handle of the WebKitLoader
        WebKitLoader*               m_webKitLoader; // not owned
    }; // class ContentDownloadListener

#endif // SUPPORT_CONTENT_DOWNLOADER

#endif      // LOADLISTENERS_H

// End of File

