/*
* ==============================================================================
*  Name        : BaseTransaction.cpp
*  Part of     : WebKit / ResourceLoader
*  Interface   :
*  Description : Implementation of BaseTransaction
*  Version     : %Version%
* ==============================================================================
*/

// INCLUDE FILES
#include "types.h"
#include "BaseTransaction.h"
#include "UrlRequestInfo.h"
#include "UrlResponseInfo.h"
#include "UrlResponseHeaderInfo.h"
#include "UrlLoader.h"

#if PLATFORM(SHP)
#include "TransactionManager.h"
#endif //#if PLATFORM(SHP)

#ifdef __CMM__
#include <AnsiCLib.h>
#include <allocs.h>
#else
#include "Shp.h"
#endif

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES

// CONSTANTS

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================


// ============================ MEMBER FUNCTIONS ===============================


// -----------------------------------------------------------------------------
// BaseTransaction::BaseTransaction
// C++ default constructor
// -----------------------------------------------------------------------------
//
BaseTransaction::BaseTransaction(    UrlRequestInfo* _urlRequestInfo,
    ITransactionObserver* _transactionObserver )
    : m_status( NO_RESPONSE_YET ),
    m_urlRequestInfo( _urlRequestInfo ),
    m_transactionObserver( _transactionObserver ),
    m_responseBuffer( NULL ),
    m_isTransCancelled(false),
    m_refCount( 1 ) // Transaction is created so by default it is refered
    {
    // lock
    m_transactionObserver->GetUrlLoader()->Ref();

    m_responseInfo = new UrlResponseInfo();
    m_responseInfo->SetRequestInfo(m_urlRequestInfo);

    // Check for a fragment
    char* fragLocation = AcStrstr( m_urlRequestInfo->CurrentUrl(), "#" );
    if( NULL != fragLocation )
        {
        int fragLen = AcStrlen(fragLocation);
        
        // fragLen aready has an extra character
        m_fragment = (char *)malloc( fragLen );
        AcStrcpy(m_fragment, fragLocation + 1);

        }
    else
        {
        m_fragment = NULL;
        }
    }


// Destructor
BaseTransaction::~BaseTransaction()
    {
    free(m_fragment);

    delete m_urlRequestInfo;
    m_urlRequestInfo = NULL;

    // reference counting we don't need the header info anymore
    delete m_responseInfo;
    // and the temporary response buffer
    PurgeData();
    // unlock
    m_transactionObserver->GetUrlLoader()->Deref();
    }

// -----------------------------------------------------------------------------
// BaseTransaction::SubmitTransaction
//
//
// -----------------------------------------------------------------------------
//
eErrorCode BaseTransaction::SubmitTransaction()
    {
    // check first if this is a data url
    // submit simply calls async submit (webcore "requirement")
    DoSubmit();
    return ERR_OK;
    }


// -----------------------------------------------------------------------------
// BaseTransaction::CancelTransaction
//
//
// -----------------------------------------------------------------------------
//
void BaseTransaction::CancelTransaction( eErrorCode _error )
    {
    // and let the engine know about the cancellation
    // set KErrCancel unless aError says differently
    m_isTransCancelled = true;
    if( ERR_OK == _error )
        {
        _error = ERR_CANCEL;
        }
    if( SEND_RESPONSE_COMPLETE != m_status )
        {
        m_transactionObserver->Complete( this, m_responseInfo, _error );
        }
    }

// -----------------------------------------------------------------------------
// BaseTransaction::ResponseBody
//
//
// -----------------------------------------------------------------------------
//
char* BaseTransaction::ResponseBody()
    {
    char* responseBody = m_responseBuffer;
    // once the loader takes the chunk, set it to NULL to not to
    // take the same chunk again
    m_responseBuffer = NULL;
    return responseBody;
    }

// -----------------------------------------------------------------------------
// BaseTransaction::PurgeData
//
//
// -----------------------------------------------------------------------------
//
void BaseTransaction::PurgeData()
    {
    // and the response cache. -it should be empty though.
    MemoryManager::Free(m_responseBuffer);
    m_responseBuffer = NULL;
    }

// -----------------------------------------------------------------------------
// BaseTransaction::Run
//
//
// -----------------------------------------------------------------------------
//
void BaseTransaction::Run(eErrorCode _error )
    {
    switch( m_status )
        {
        case SEND_PARTIAL_RESPONSE:
            {
            // create response object
            SetBodyResponse();
            // and pass it to the loader
            m_transactionObserver->Response( this, m_responseInfo );
            break;
            }
        case SEND_RESPONSE_COMPLETE:
            {
            m_transactionObserver->Complete( this, m_responseInfo, _error );
            // note: the transaction is destroyed at this point (by the loader)
            // pay attention to what you put in here
            break;
            }
        default:
            {
            break;
            }
        }
    }

// -----------------------------------------------------------------------------
// BaseTransaction::SendPartialResponse
//
//
// -----------------------------------------------------------------------------
//
void BaseTransaction::SendPartialResponse()
    {
    // call RunL() sync which in turn sends the content off to the
    // the transaction obeserver
    m_status = SEND_PARTIAL_RESPONSE;
    Run();
    }

// -----------------------------------------------------------------------------
// BaseTransaction::TransactionComplete
//
//
// -----------------------------------------------------------------------------
//
void BaseTransaction::TransactionComplete(eErrorCode _error )
    {
    // there musn't be anything in the chunk buffer (sync sending)
    m_status = SEND_RESPONSE_COMPLETE;
    Run( _error );
    }

// -----------------------------------------------------------------------------
// BaseTransaction::HandleError
//
//
// -----------------------------------------------------------------------------
//
bool BaseTransaction::HandleError( int _error )
    {
    // let the loader handle the error
    return m_transactionObserver->HandleError( this, _error );
    // check if the error report is html
    }

// -----------------------------------------------------------------------------
// BaseTransaction::AddBodyData
//
// -----------------------------------------------------------------------------
//
void BaseTransaction::AddBodyData(const void* _chunkPtr, int _len )
    {
    m_responseBuffer = (char*)_chunkPtr;
    m_responseBufferLen = _len;
    // and send it to the transaction obeserver
    SendPartialResponse();
    }

// -----------------------------------------------------------------------------
// BaseTransaction::SetBodyResponse()
//
//
// -----------------------------------------------------------------------------
//
void BaseTransaction::SetBodyResponse()
    {
    // body
    m_responseInfo->SetBody( ResponseBody(), m_responseBufferLen );
    }

#if PLATFORM(SHP)
void BaseTransaction::StopTransTimer()
{
	// Stop Timer
	TransactionManager::sharedInstance()->StopTimer();	
}
#endif

//  End of File

