/*
* ==============================================================================
*  Name        : ContentDispatcher.cpp
*  Part of     : WebKit / ResourceLoader
*  Interface   :
*  Description : Implementation of ContentDispatcher
*  Version     : %Version%
* ==============================================================================
*/

// INCLUDE FILES
#include "types.h"
#include "ContentDispatcher.h"
#include "ContentListener.h"
#include "UrlResponseInfo.h"
#include "UrlResponseHeaderInfo.h"
#include "BaseTransaction.h"

#ifdef __CMM__
#include <AnsiCLib.h>
#include <allocs.h>
#else
#include "Shp.h"
#endif

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES

// CONSTANTS
#define KContentTypeApplicationWapXhtml "application/vnd.wap.xhtml+xml"
#define KContentTypeApplicationOctetStream "application/octet-stream"

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================

// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// ContentDispatcher::ContentDispatcher
// C++ default constructor can NOT contain any code, that
// might leave.
// -----------------------------------------------------------------------------
//
ContentDispatcher::ContentDispatcher()
    : m_refCount( 1 )
    {
    }

// -----------------------------------------------------------------------------
// Destructor
//
// -----------------------------------------------------------------------------
ContentDispatcher::~ContentDispatcher()
    {
    // destroy transaction data
    ClearTransactionList();
    }

// -----------------------------------------------------------------------------
// ContentDispatcher::SetContentListener
//
//
// -----------------------------------------------------------------------------
//
eErrorCode ContentDispatcher::SetContentListener(
    const int _transactionId,
    IContentListener* _contentListener )
    {
    return AddTransactionEntry( _transactionId, _contentListener );
    }


// -----------------------------------------------------------------------------
// ContentDispatcher::RemoveContentListener
//
//
// -----------------------------------------------------------------------------
//
eErrorCode ContentDispatcher::RemoveContentListener(
    const int _transactionId )
    {
		TransactionEntry* transactionEntry  = FindTransactionEntry( _transactionId );
		if( transactionEntry == NULL )
			return ERR_CANCEL;

		return RemoveTransactionEntry( _transactionId );
    }

// -----------------------------------------------------------------------------
// ContentDispatcher::Headers
//
//
// -----------------------------------------------------------------------------
//
eErrorCode ContentDispatcher::Headers(
    BaseTransaction* _transaction,
    UrlResponseInfo* _response )
    {
    int transactionId (_transaction->TransactionId());
    // locate transaction entry record
    TransactionEntry* transactionEntry = FindTransactionEntry( transactionId );
    // verify that we found the transaction

    return transactionEntry->m_contentListener->Headers( transactionId, _response );
    }

// -----------------------------------------------------------------------------
// ContentDispatcher::Response
//
//
// -----------------------------------------------------------------------------
//
void ContentDispatcher::Response(
    BaseTransaction* _transaction,
    UrlResponseInfo* _response )
    {

    int transactionId (_transaction->TransactionId());
    // locate transaction entry record
    TransactionEntry* transactionEntry = FindTransactionEntry( transactionId );
    // verify that we found the transaction

    transactionEntry->m_contentListener->Response( transactionId, _response );
    }

// -----------------------------------------------------------------------------
// ContentDispatcher::Complete
//
//
// -----------------------------------------------------------------------------
//
void ContentDispatcher::Complete(
    BaseTransaction* _transaction,
    UrlResponseInfo* _response,
    eErrorCode _error )
    {
    // locate transaction entry record
    int transactionId (_transaction->TransactionId());
    TransactionEntry* transactionEntry = FindTransactionEntry( transactionId );

    transactionEntry->m_contentListener->Complete( transactionId, _response, _error );
    // then remove transaction entry
    RemoveTransactionEntry( transactionId );
    }

// -----------------------------------------------------------------------------
// ContentDispatcher::HandleError
//
//
// -----------------------------------------------------------------------------
//
void ContentDispatcher::HandleError(
    const int _transactionId,
    int _error )
    {

    // locate transaction entry record
    TransactionEntry* transactionEntry = FindTransactionEntry( _transactionId );

    if ( transactionEntry )
        {
        transactionEntry->m_contentListener->HandleError( _transactionId, _error );
        RemoveTransactionEntry( _transactionId );
        }

    }

// -----------------------------------------------------------------------------
// ContentDispatcher::AddTransactionEntry
//
//
// -----------------------------------------------------------------------------
//
eErrorCode ContentDispatcher::AddTransactionEntry( const int _transactionId,
        IContentListener* _contentListener )
    {
    TransactionEntry* newEntry = new TransactionEntry;
    // set fields of transaction entry to insert
    newEntry->m_transactionId = _transactionId;
    newEntry->m_contentListener = _contentListener;
    // insert entry
    m_transactionEntries.append( newEntry );
    return ERR_OK;
    }

// -----------------------------------------------------------------------------
// ContentDispatcher::FindTransactionEntry
//
//
// -----------------------------------------------------------------------------
//
TransactionEntry* ContentDispatcher::FindTransactionEntry(
    const int _transactionId  )
    {
    TransactionEntry* retEntry = NULL;

    // search for matching transaction entry
    for( int i = 0; i < m_transactionEntries.size(); i++ )
    {
        if( m_transactionEntries[i]->m_transactionId == _transactionId)
        {
            retEntry = m_transactionEntries[i];
            break;
        }
    }
    return retEntry;
    }

// -----------------------------------------------------------------------------
// ContentDispatcher::FindTransactionEntry
//
//
// -----------------------------------------------------------------------------
//
TransactionEntry* ContentDispatcher::FindTransactionEntry(
    const WebCore::ResourceLoader* _resourceLoader  )
    {
    TransactionEntry* retEntry = NULL;

    // search for matching transaction entry
    for( int i = 0; i < m_transactionEntries.size(); i++ )
    {
        if( m_transactionEntries[i]->m_contentListener->GetResourceLoader() == _resourceLoader)
        {
            retEntry = m_transactionEntries[i];
            break;
        }
    }
    return retEntry;
    }

// -----------------------------------------------------------------------------
// ContentDispatcher::RemoveTransactionEntry
//
//
// -----------------------------------------------------------------------------
//
eErrorCode ContentDispatcher::RemoveTransactionEntry(
    const int _transactionId )
    {
    for( int i = 0; i < m_transactionEntries.size(); i++ )
    {
        if( m_transactionEntries[i]->m_transactionId == _transactionId )
        {
            TransactionEntry* trans = m_transactionEntries[i];
            m_transactionEntries.remove(i);
            delete trans;
            break;
        }
    }
    return ERR_OK;
    }

// -----------------------------------------------------------------------------
// ContentDispatcher::ClearTransactionList
//
//
// -----------------------------------------------------------------------------
//
eErrorCode ContentDispatcher::ClearTransactionList()
    {
    for( int i = 0; i < m_transactionEntries.size(); i++ )
    {
        delete m_transactionEntries[i];
    }
    m_transactionEntries.clear();
    return ERR_OK;
    }

//  End of File

