/*
* ==============================================================================
*  Name        : UrlLoader.cpp
*  Part of     : WebKit / ResourceLoader
*  Interface   :
*  Description : Implementation of UrlLoader
*  Version     : %Version%
*
* ==============================================================================
*/

// INCLUDE FILES
#include "types.h"
#include "UrlLoader.h"
#include "BaseTransaction.h"
#include "ContentDispatcher.h"

// EXTERNAL DATA STRUCTURES

// EXTERNAL FUNCTION PROTOTYPES

// CONSTANTS

// MACROS

// LOCAL CONSTANTS AND MACROS

// MODULE DATA STRUCTURES

// LOCAL FUNCTION PROTOTYPES

// FORWARD DECLARATIONS

// ============================= LOCAL FUNCTIONS ===============================


// ============================ MEMBER FUNCTIONS ===============================

// -----------------------------------------------------------------------------
// UrlLoader::UrlLoader
// C++ default constructor can NOT contain any code, that
// might leave.
// -----------------------------------------------------------------------------
//
UrlLoader::UrlLoader(
    ContentDispatcher* _contentDispatcher )
    : m_contentDispatcher( _contentDispatcher ),
      m_refCount( 1 )
    {
    }

// Destructor
UrlLoader::~UrlLoader()
    {
    // cancel and delete pending transactions
    CancelAll();
    // ignore error
    }

// -----------------------------------------------------------------------------
// UrlLoader::Request
//
//
// -----------------------------------------------------------------------------
//
eErrorCode UrlLoader::Request(
    UrlRequestInfo* _requestInfo )
    {
    BaseTransaction* transaction = CreateTransaction( _requestInfo );
    if( transaction )
        {
        return DoSubmitTransaction( transaction );
        }
        
    return ERR_OK;
    }

// -----------------------------------------------------------------------------
// UrlLoader::Cancel
//
//
// -----------------------------------------------------------------------------
//
eErrorCode UrlLoader::Cancel(
    const int _transactionId,
    eErrorCode _error )
    {
    eErrorCode status = ERR_OK;

    BaseTransaction* trans = FindTransactionByTid( _transactionId );

    if( trans )
        {
        Cancel( trans, _error );
        status = ERR_OK;
        }

    return status;
    }

// -----------------------------------------------------------------------------
// UrlLoader::Cancel
//
//
// -----------------------------------------------------------------------------
//
void UrlLoader::Cancel(
    BaseTransaction* _transaction,
    eErrorCode _error )
    {
    // do not cancel completed transaction
    if( _transaction && _transaction->Status() != SEND_RESPONSE_COMPLETE )
        {
        // cancel completes the transaction
        _transaction->CancelTransaction( _error );
        }
    }

// -----------------------------------------------------------------------------
// UrlLoader::CancelAll
//
//
// -----------------------------------------------------------------------------
//
eErrorCode UrlLoader::CancelAll( eErrorCode _error )
    {
    // cancel both active and pending
    // transactions
    eErrorCode status =  ERR_NOT_READY;
    BaseTransaction* trans;

#if PLATFORM(SHP)

	bool bTimerStopped = false;

	if(m_activeTransList.size() > 0)
	{
		m_activeTransList[0]->StopTransTimer();
	}

#endif //#if PLATFORM(SHP)

    // Iterator for the active queue
    while( m_activeTransList.size() > 0 )
        {
            // Cancel the first from the list
            // Cancel removes the transaction from the list
			if( !m_activeTransList[0] || m_activeTransList[0]->Status() == SEND_RESPONSE_COMPLETE )
		    {
				m_activeTransList.remove(0);
			}
			else
			{
                Cancel( m_activeTransList[0], _error );
            }
            status = ERR_OK;
        }
    return status;
    }

// -----------------------------------------------------------------------------
// UrlLoader::Headers
//
//
// -----------------------------------------------------------------------------
//
eErrorCode UrlLoader::Headers(BaseTransaction* _transaction, UrlResponseInfo* _response )
    {
    return m_contentDispatcher->Headers( _transaction, _response );
    }

// -----------------------------------------------------------------------------
// UrlLoader::Response
//
//
// -----------------------------------------------------------------------------
//
void UrlLoader::Response(BaseTransaction* _transaction, UrlResponseInfo* _response )
    {
    m_contentDispatcher->Response( _transaction, _response );
    }

// -----------------------------------------------------------------------------
// UrlLoader::Complete
//
//
// -----------------------------------------------------------------------------
//
void UrlLoader::Complete(BaseTransaction* _transaction, UrlResponseInfo* _response, eErrorCode _error )
    {
    ContentDispatcher* contentDispatcher = m_contentDispatcher;
    int transId( _transaction->TransactionId() );
    // lock
    contentDispatcher->Ref();
    //
    contentDispatcher->Complete( _transaction, _response, _error );
    // close transaction when the last chunk is passed.
    // unless the transaction has been cancelled
    if( RemoveTransaction( transId ) )
        {
        _transaction->Deref();
        }
    // unlock
    contentDispatcher->Deref();
    }

// -----------------------------------------------------------------------------
// UrlLoader::HandleError
//
//
// -----------------------------------------------------------------------------
//
bool UrlLoader::HandleError(BaseTransaction* _transaction, int _error )
    {
    ContentDispatcher* contentDispatcher = m_contentDispatcher;
    int transId( _transaction->TransactionId() );
    // lock
    contentDispatcher->Ref();
    //
    contentDispatcher->HandleError( transId, _error );
    // close transaction since we failed to complete it
    if( RemoveTransaction( transId ) )
        {
        _transaction->Deref();
        }
    // unlock
    contentDispatcher->Deref();
    return FALSE;
    }

// -----------------------------------------------------------------------------
// UrlLoader::UrlLoader
//
//
// -----------------------------------------------------------------------------
//
UrlLoader* UrlLoader::GetUrlLoader()
    {
    return this;
    }


// -----------------------------------------------------------------------------
// UrlLoader::FindTransactionByTid
//
//
// -----------------------------------------------------------------------------
//
BaseTransaction* UrlLoader::FindTransactionByTid(
    const int _transactionId )
    {
    // loop through the both the active linked list

    BaseTransaction* trans = NULL;

    for( int i = 0; i < m_activeTransList.size(); i++)
        {
            trans = m_activeTransList[i];
            if( trans->TransactionId() == _transactionId )
                {
                return trans;
                }
        }

    return NULL;
    }

// -----------------------------------------------------------------------------
// UrlLoader::DoSubmitTransaction
//
//
// -----------------------------------------------------------------------------
//
eErrorCode UrlLoader::DoSubmitTransaction(BaseTransaction* _trans )
    {
    eErrorCode error = ERR_OK;
    
    if( _trans )
        {
        // add it to the active list
        m_activeTransList.append( _trans );
        
        error = _trans->SubmitTransaction();                
        }
        
    return error;        
    }
    
BaseTransaction* UrlLoader::RemoveTransaction( int _transactionId)
{
    BaseTransaction* trans = NULL;
    
    for( int i = 0; i < m_activeTransList.size(); i++)
        {
            trans = m_activeTransList.at( i );
            if( trans->TransactionId() == _transactionId )
                {
                m_activeTransList.remove( i );
                return trans;
                }
        }

    return NULL;
}

//  End of File

