#ifndef URLREQUESTINFO_H
#define URLREQUESTINFO_H

#include "BridgeTypes.h"
#include "HttpLoaderDefs.h"
#include "HTTPHeaderMap.h"
#include "PlatformString.h"

#include "ResourceRequestBase.h"
#include "CMMDefs.h"

/**
*  This calls holds information about the
*  request. It is created by the client and processed
*  by the loaders.
*  @lib resLoader.lib
*/
using namespace WebCore;

class UrlRequestInfo CMM_MODIFIED
    {

    public:  // Constructors and destructor

        /**
        * Constructor.
        */
        UrlRequestInfo();



        /**
        * Destructor.
        */
        virtual ~UrlRequestInfo();

    public: // New functions

        /**
        * Set request headers in name-value pair format
        * @param _name of the header field e.g. "content type"
        * @param _value of the header field e.g. "text/html"
        * @return void
        */
        void SetRequestHeaders( const HTTPHeaderMap& _fieldList )
        {
            m_requestHeaders = _fieldList;
        }

        /**
        * Set request headers in name-value pair format
        * @param _name of the header field e.g. "content type"
        * @param _value of the header field e.g. "text/html"
        * @return void
        */
        HTTPHeaderMap GetRequestHeaders() const { return m_requestHeaders; }

        /**
        * Set request header field in name-value pair format
        * @param _name of the header field e.g. "content type"
        * @param _value of the header field e.g. "text/html"
        * @return void
        */
        void SetRequestHeaderField( String& _name, String& _value )
        {   m_requestHeaders.set( _name, _value ); }

        /**
        * Get request header field value
        * @param _name of the header field e.g. "content type"
        * @return _value corresponding to the _name in the map
        */
        String GetRequestHeaderField( String& _name ) const
        {   return m_requestHeaders.get( _name ); }

        /**
        * Set the request url
        * @param _url Request url.
        * @return void
        */
        void SetUrl( const char* _url );

        /**
        * Get the request url
        * @return Request url.
        */
        inline char* Url() const { return m_url; }

        /**
        * Set the current page's url
        * @since 3.1
        * @param aCurrentUrl current page's url
        * @return
        */
        void SetCurrentUrl( const char* _currentUrl );

        /**
        * Get the current page's url
        * @since 3.1
        * @param
        * @return current page's url
        */
        inline char* CurrentUrl() const { return m_currentUrl; }

        /**
        * Set the redirected url
        * @since 3.1
        * @param aCurrentUrl current page's url
        * @return
        */
        void SetRedirectedUrl( const char* _redirectedUrl );

        /**
        * Get the current page's url
        * @since 3.1
        * @param
        * @return current page's url
        */
        inline char* RedirectedUrl() const { return m_redirectedUrl; }
        /**
        * Set transaction id
        * @param aTransId Transaction id
        * @return void
        */
        inline void SetTransId( int _transId ) { m_transId = _transId; }

        /**
        * Get transaction id
        * @return Transaction id
        */
        inline int TransId() const { return m_transId; }

        /**
        * Set load method ( get, head, post )
        * @param _loadMethod Load method
        * @return void
        */
        inline void SetLoadMethod( eUrlLoadMethod _loadMethod ) { m_loadMethod = _loadMethod; }

        /**
        * Get load method ( get, head, post )
        * @return Load method
        */
        inline eUrlLoadMethod LoadMethod() const { return m_loadMethod; }

        /**
        * Set load type ( image, css, pluging etc )
        * @param _loadType Load type
        * @return void
        */
        inline void SetLoadType( eUrlLoadType _loadType ) { m_loadType = _loadType; }

        /**
        * Get load type ( image, css, pluging etc )
        * @return Load type
        */
        inline eUrlLoadType LoadType() const { return m_loadType; }

        /**
        * Set cache mode ( normal, no cahce, only cache )
        * @param _cacheMode Cache mode
        * @return void
        */
        inline void SetCacheMode( WebCore::ResourceRequestCachePolicy _cacheMode ) 
            { m_cacheMode = _cacheMode; }

        /**
        * Get cache mode ( normal, no cahce, only cache )
        * @return Cache mode
        */
        inline WebCore::ResourceRequestCachePolicy CacheMode() const { return m_cacheMode; }

        /**
        * Set top level(page load) flag
        * @param _topLevel Top level flag
        * @return void
        */
        inline void SetTopLevel( bool _topLevel ) { m_isTopLevel = _topLevel; }

        /**
        * Get top level(page load) flag
        * @return Top level flag
        */
        inline bool TopLevel() const { return m_isTopLevel; }

        /**
        * Set true if the request is initiated by a child frame
        * @param _childFrame child frame flag
        * @return void
        */
        inline void SetChildFrameLoad( bool _childFrame ) { m_childFrame = _childFrame; }

        /**
        * Get child frame flag
        * @return Child frame flag
        */
        inline bool ChildFrameLoad() const { return m_childFrame; }

        /**
        * Set the default header mask -which header
        * has to be set
        * @param _headerMaskType Which header has to be set
        * @return void
        */
        inline void SetDefaultHeaderMask( eUrlDefaultHeaderType _headerMaskType )
            { m_defaultHeadersMask = _headerMaskType; }


        /**
        * Get the default header mask -which header
        * has to be set
        * @return Which header has to be set
        */
        inline eUrlDefaultHeaderType DefaultHeaderMask() const { return m_defaultHeadersMask; }

        /**
        * Set predefined headers (defined by the client)
        * @param _headers Predefined headers
        * @return void
        */
        void SetAcceptHeader( const char* _acceptHeader );

        /**
        * Get predefined headers (defined by the client)
        * @return Predefined headers
        */
        inline char* AcceptHeader() const { return m_acceptHeader; }

        /**
        * Set accept character set
        * @param _charset Charset info
        * @return void
        */
        inline void SetCharacterSet( int _charset) { m_characterSet = _charset; }

        /**
        * Get accept character set
        * @return Charset info
        */
        inline int CharacterSet() const { return m_characterSet; }

        /**
        * Set whether or not IMEI notify is enabled
        * @param aEnabled whether or not IMEI notify is enabled
        * @return void
        */
        inline void SetIMEINotifyEnabled( bool _enabled ) { m_IMEINotifyEnabled = _enabled; }

        /**
        * Get whether or not IMEI notify is enabled
        * @return True if IMEI notify is enabled
        */
        inline bool IMEINotifyEnabled() const { return m_IMEINotifyEnabled; }

        /**
        * Set the referer url
        * @param _refererUrl HTTP Referer url
        * @return
        */
        void SetRefererUrl( const char* _refererUrl );

        /**
        * Get http referer url
        * @return Referer url
        */
        inline char* RefererUrl() const { return m_refererUrl; }

        /**
        * Set whether or not to set http referer header
        * @param _enabled whether or not to set http referer header
        * @return void
        */
        inline void SetSendRefererHeaderEnabled( bool _enabled ) { m_sendRefererHeaderEnabled = _enabled; }

        /**
        * Get whether or not to set http referer header
        * @return True if we need to set http referer header
        */
        inline bool SendRefererHeaderEnabled() const { return m_sendRefererHeaderEnabled; }

        /**
        * Set whether or not http cookies are enabled
        * @param _enabled whether or not http cookies are enabled
        * @return
        */
        inline void SetCookiesEnabled( bool _enabled ) { m_cookiesEnabled = _enabled; }

        /**
        * Get whether or not http cookies are enabled
        * @return True, if http cookies are enabled
        */
        inline bool CookiesEnabled() const { return m_cookiesEnabled; }

        /**
        * Indicates whether request is from a Saved Page
        * @return bool
        */
        inline bool IsSavedPageRequest() { return m_savedPageRequest; }


        /**
        * Sets the  request is from a Saved Page
        * @param _enabled whether or not the request is from saved page
        * @return
        */

        inline void SetSavedPageRequest( bool _isSavedPage ) { m_savedPageRequest = _isSavedPage; }

        /**
        * Set the request priority. -1 means no priority
        * @since 3.1
        * @param _priority priority
        * @return
        */
        void SetPriority( int _priority ) { m_priority = _priority; }

        /**
        * Get the request priority. -1 means no priority
        * @since 3.1
        * @param
        * @return priority
        */
        int Priority() const { return m_priority; }

        /**
        * Set if the client resolves the url
        * @since 3.1
        * @param _resolve Wheter or not the client
        * resolves the url
        * @return
        */
        inline void SetClientResolvesUrl( bool _resolve ) { m_clientResolvesUrl = _resolve; }

        /**
        * Get if the client resolves the url
        * @since 3.1
        * @param
        * @return ETrue if the client
        * resolves the url
        */
        inline bool ClientResolvesUrl() const { return m_clientResolvesUrl; }

        /**
        * Set the content type of the post body
        * @since 3.1
        * @param _postContentType Content type of the post body
        * @return
        */
        void SetPostContentType( const char* _postContentType );


        /**
        * Get the content type of post body
        * @since 3.1
        * @param
        * @return Content type of post body
        */
        inline char* PostContentType() const { return m_postContentType; }

        /**
        * Set the post body buffer
        * @since 3.1
        * @param _postData Body data
        * @return
        */
        void SetPostDataStr( String _postDataStr );
        String GetPostDataStr() const { return m_postDataStr; }

        /**
        * Set the post boundary
        * @since 3.1
        * @param _postBoundry data
        * @return
        */
        void SetPostBoundary( const char* _postBoundry );

        /**
        * Get the post boundry
        * @since 3.1
        * @param
        * @return boundry
        */
        inline char* PostBoundary() const { return m_postBoundary; }



	/**
	* shkim
	* get/set WebControl which start this request
	*/
	void SetWebControl( void* pWebControl ) { m_pWebControl = pWebControl; }
	void* GetWebControl() const { return m_pWebControl; }

#if HTTP_POST_UPLOAD_SUPPORT
		void SetPostDataElemets(FormData* formData){ m_postDataElemets = formData; }
        const Vector<FormDataElement>& GetPostDataElemets() const { return (m_postDataElemets) ? m_postDataElemets->elements() : Vector<FormDataElement>(); }
#endif

      
    private: // Data

        // The request headers list
        HTTPHeaderMap m_requestHeaders;
        // request url
        char*                m_url;
        // referer url
        char*               m_refererUrl;
        // current url
        char*               m_currentUrl;
        // redirected url
        char*               m_redirectedUrl;
        // transaction id
        int                   m_transId;
        // Load method ( get, head, post )
        eUrlLoadMethod        m_loadMethod;
        // Load type ( image, css, pluging etc )
        eUrlLoadType          m_loadType;
        // cahce mode ( normal, no cahce, only cache )
        WebCore::ResourceRequestCachePolicy m_cacheMode;
        // ETrue if this is a page load, EFale if a resource load
        bool                 m_isTopLevel;
        // ETrue if the request is initiated by a child frame
        bool                 m_childFrame;
        // Boundary string on post body
        char*               m_acceptHeader;
        // Indicates which header has to be set
        eUrlDefaultHeaderType m_defaultHeadersMask;
        // charset
        int                  m_characterSet;
        // IMEI notify setting
        bool                 m_IMEINotifyEnabled;
        // send refer header setting
        bool                 m_sendRefererHeaderEnabled;
        // cookies setting
        bool                 m_cookiesEnabled;
        //Is it save page request.
        bool                 m_savedPageRequest;

        bool                 m_clientResolvesUrl;
        int                  m_priority;
        char*                m_postContentType;
        String               m_postDataStr;
        char*                m_postBoundary;
		void*					m_pWebControl;
  
#if HTTP_POST_UPLOAD_SUPPORT
		RefPtr<FormData> m_postDataElemets;		
#endif

    };

#endif      // URLREQUESTINFO_H

// End of File

