
#include "TransactionManager.h"
#include "CString.h"
#include "BaseTransaction.h"
#include "HttpTransaction.h"
#include "HTTPParsers.h"

using namespace WebCore;
const int selectTimeoutMS = 5;
const double pollTimeSeconds = 0.05;

TransactionManager::TransactionManager()
    : m_downloadTimer(this, &TransactionManager::downloadTimerCallback)
    , m_cookieJarFileName(0)
    , m_TransactionListHead(0)
{
    curl_global_init(CURL_GLOBAL_ALL);
    m_curlMultiHandle = curl_multi_init();
    m_curlShareHandle = curl_share_init();
    curl_share_setopt(m_curlShareHandle, CURLSHOPT_SHARE, CURL_LOCK_DATA_COOKIE);
    curl_share_setopt(m_curlShareHandle, CURLSHOPT_SHARE, CURL_LOCK_DATA_DNS);
}

TransactionManager::~TransactionManager()
{
    curl_multi_cleanup(m_curlMultiHandle);
    curl_share_cleanup(m_curlShareHandle);
    if (m_cookieJarFileName)
        free(m_cookieJarFileName);
}

void TransactionManager::setCookieJarFileName(const char* cookieJarFileName)
{ 
    m_cookieJarFileName = strdup(cookieJarFileName);
}

TransactionManager* TransactionManager::sharedInstance()
{
    static TransactionManager* sharedInstance = 0;
    if (!sharedInstance)
        sharedInstance = new TransactionManager();
    return sharedInstance;
}

// called with data after all headers have been processed via headerCallback
static size_t writeCallback(void* ptr, size_t size, size_t nmemb, void* data)
{

    BaseTransaction* job = static_cast<BaseTransaction*>(data);
    int totalSize = size * nmemb;

    // this shouldn't be necessary but apparently is. CURL writes the data
    // of html page even if it is a redirect that was handled internally
    // can be observed e.g. on gmail.com
    CURL* h = job->GetTransportHandle();
    long httpCode = 0;
    CURLcode err = curl_easy_getinfo(h, CURLINFO_RESPONSE_CODE, &httpCode);
    if (CURLE_OK == err && httpCode >= 300 && httpCode < 400)
        return totalSize;

    // <TODO> : Abstarct HandleResponseBodyData to avoid type cast
    if (job)
        ((HttpTransaction*)job)->HandleResponseBodyData(ptr, totalSize);
    return totalSize;
}

/*
 * This is being called for each HTTP header in the response. This includes '\r\n'
 * for the last line of the header.
 *
 * We will add each HTTP Header to the ResourceResponse and on the termination
 * of the header (\r\n) we will parse Content-Type and Content-Disposition and
 * update the ResourceResponse and then send it away.
 *
 */
static size_t headerCallback(char* ptr, size_t size, size_t nmemb, void* data)
{

    BaseTransaction* job = static_cast<BaseTransaction*>(data);

    unsigned int totalSize = size * nmemb;
    if (!job) {
        return totalSize;
    }
    
    // Create the response header info
    // <TODO> : Abstract HandleGotResponseHeaders to avoid type cast
    if(!job->ResponseHeaders()) {
        ((HttpTransaction*)job)->CreateHeaderResponse();
        
    }
    
    String header(static_cast<const char*>(ptr), totalSize);

    /*
     * a) We can finish and send the ResourceResponse
     * b) We will add the current header to the HTTPHeaderMap of the ResourceResponse 
     */
    if (header == String("\r\n")) {
        CURL* h = job->GetTransportHandle();
        CURLcode err;
        UrlResponseHeaderInfo* responseHeader = job->ResponseHeaders();

        double contentLength = 0;
        err = curl_easy_getinfo(h, CURLINFO_CONTENT_LENGTH_DOWNLOAD, &contentLength);
        job->ResponseHeaders()->SetContentLength( contentLength );

        const char* hdr;
        err = curl_easy_getinfo(h, CURLINFO_EFFECTIVE_URL, &hdr);
        job->ResponseHeaders()->SetResponseUrl( hdr );

        long httpCode = 0;
        err = curl_easy_getinfo(h, CURLINFO_RESPONSE_CODE, &httpCode);
        job->ResponseHeaders()->SetHttpStatus((eHttpStatus) httpCode );
        
        ((HttpTransaction*)job)->HandleGotResponseHeaders();
    
    } else {
        int splitPos = header.find(":");
        if (splitPos != -1)
            job->ResponseHeaders()->SetResponseHeaderField(header.left(splitPos), header.substring(splitPos+1).stripWhiteSpace());
    }

    return totalSize;
}

void TransactionManager::downloadTimerCallback(Timer<TransactionManager>* timer)
{
    startScheduledJobs();

    fd_set fdread;
    FD_ZERO(&fdread);
    fd_set fdwrite;
    FD_ZERO(&fdwrite);
    fd_set fdexcep;
    FD_ZERO(&fdexcep);
    int maxfd = 0;
    curl_multi_fdset(m_curlMultiHandle, &fdread, &fdwrite, &fdexcep, &maxfd);

    struct timeval timeout;
    timeout.tv_sec = 0;
    timeout.tv_usec = selectTimeoutMS * 1000;       // select waits microseconds

    int rc = ::select(maxfd + 1, &fdread, &fdwrite, &fdexcep, &timeout);

    if (-1 == rc) {
#ifndef NDEBUG
        printf("bad: select() returned -1\n");
#endif
        return;
    }

    int runningHandles = 0;
    CURLMcode curlCode = CURLM_CALL_MULTI_PERFORM;
    while (CURLM_CALL_MULTI_PERFORM == curlCode) {
        curlCode = curl_multi_perform(m_curlMultiHandle, &runningHandles);
    }

    // check the curl messages indicating completed transfers
    // and free their resources
    while (true) {
        int messagesInQueue;
        CURLMsg* msg = curl_multi_info_read(m_curlMultiHandle, &messagesInQueue);
        if (!msg)
            break;

        if (CURLMSG_DONE != msg->msg)
            continue;

        // find the node which has same d->m_handle as completed transfer
        CURL* handle = msg->easy_handle;
        ASSERT(handle);
        BaseTransaction* job = 0;
        CURLcode err = curl_easy_getinfo(handle, CURLINFO_PRIVATE, &job);
        ASSERT(CURLE_OK == err);
        ASSERT(job);
        if (!job)
            continue;
        ASSERT(job->GetTransportHandle() == handle);
        // No need to call removeFromCurl() since it automatically 
        // gets called from TransactionComplete()        
        // removeFromCurl(job);
        if (CURLE_OK == msg->data.result) {
            if (job && SEND_RESPONSE_COMPLETE != job->Status())
                job->TransactionComplete();
        } else {
#ifndef NDEBUG
            char* url = 0;
            curl_easy_getinfo(d->m_handle, CURLINFO_EFFECTIVE_URL, &url);
            printf("Curl ERROR for url='%s', error: '%s'\n", url, curl_easy_strerror(msg->data.result));
#endif
            // <TODO> : Handle error codes
            // Refer http://curl.haxx.se/libcurl/c/libcurl-errors.html for CURLcode
            if (job)
                job->HandleError((eHttpStatus)msg->data.result);
            
        }

    }

    bool started = startScheduledJobs(); // new jobs might have been added in the meantime

    if (!m_downloadTimer.isActive() && (started || (runningHandles > 0)))
        m_downloadTimer.startOneShot(pollTimeSeconds);
}

void TransactionManager::removeFromCurl(BaseTransaction* job)
{
    ASSERT(job->GetTransportHandle());
    if (!job->GetTransportHandle())
        return;
    curl_multi_remove_handle(m_curlMultiHandle, job->GetTransportHandle());
    curl_easy_cleanup(job->GetTransportHandle());
    job->SetTransportHandle(0);
}

void TransactionManager::setupPUT(BaseTransaction* job)
{
    // <TODO> : PUT method not implemented
    return;
}

void TransactionManager::setupPOST(BaseTransaction* job)
{
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_POST, TRUE);

    // <TODO> : Support Post data
    PostDataParts *postData = job->GetUrlRequestInfo()->PostData();
    if (postData->TotalSize() != 0) {
        curl_easy_setopt((CURL*)job->GetTransportHandle(), CURLOPT_POSTFIELDSIZE, postData->TotalSize());
    }

    struct curl_httppost* lastItem = 0;
    struct curl_httppost* post = 0;
    int postFieldCount = 0;
    while (postData->HasMoreParts()) {
        // Fill in the file upload field
        char* data;
        postData->GetNextDataPart(data);
        curl_formadd(&post, &lastItem, CURLFORM_COPYNAME, "sendfile", CURLFORM_FILE, data, CURLFORM_END);

        // Fill in the filename field
        curl_formadd(&post, &lastItem, CURLFORM_COPYNAME, "filename", CURLFORM_COPYCONTENTS, data, CURLFORM_END);

        // FIXME: We should not add a "submit" field for each file uploaded. Review this code.
        // Fill in the submit field too, even if this is rarely needed
        curl_formadd(&post, &lastItem, CURLFORM_COPYNAME, "submit", CURLFORM_COPYCONTENTS, "send", CURLFORM_END);

        postFieldCount++;

        // FIXME: should we support more than one file?
        break;
    }

    // Restore the post data
    job->GetUrlRequestInfo()->PostData()->Reset();

    if (post) {
        curl_easy_setopt(job->GetTransportHandle(), CURLOPT_POSTFIELDS, postFieldCount);
        curl_easy_setopt(job->GetTransportHandle(), CURLOPT_HTTPPOST, post);
    }
}

void TransactionManager::add(BaseTransaction* job)
{
    // we can be called from within curl, so to avoid re-entrancy issues
    // schedule this job to be added the next time we enter curl download loop
    m_TransactionListHead = new TransactionList(job, m_TransactionListHead);
    if (!m_downloadTimer.isActive())
        m_downloadTimer.startOneShot(pollTimeSeconds);
}

bool TransactionManager::removeScheduledJob(BaseTransaction* job)
{
    TransactionList* node = m_TransactionListHead;
    while (node) {
        TransactionList* next = node->next();
        if (job == node->job()) {
            node->setRemoved(true);
            return true;
        }
        node = next;
    }
    return false;
}

bool TransactionManager::startScheduledJobs()
{
    bool started = false;
    TransactionList* node = m_TransactionListHead;
    while (node) {
        TransactionList* next = node->next();
        if (!node->removed()) {
            startJob(node->job());
            started = true;
        }
        delete node;
        node = next;
    }
    m_TransactionListHead = 0;
    return started;
}

void TransactionManager::startJob(BaseTransaction* job)
{
    CString url( job->GetUrlRequestInfo()->Url() );
    job->SetTransportHandle(curl_easy_init());
#ifndef NDEBUG
    if (getenv("DEBUG_CURL"))
        curl_easy_setopt(job->GetTransportHandle(), CURLOPT_VERBOSE, 1);
#endif
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_PRIVATE, job);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_ERRORBUFFER, m_curlErrorBuffer);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_WRITEFUNCTION, writeCallback);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_WRITEDATA, job);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_HEADERFUNCTION, headerCallback);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_WRITEHEADER, job);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_AUTOREFERER, 1);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_FOLLOWLOCATION, 1);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_MAXREDIRS, 10);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_HTTPAUTH, CURLAUTH_ANY);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_SHARE, m_curlShareHandle);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_DNS_CACHE_TIMEOUT, 60 * 5); // 5 minutes
    // enable gzip and deflate through Accept-Encoding:
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_ENCODING, "");

    // celunite proxy
    if (!strstr(url.data(), "172.18.5"))
	curl_easy_setopt(job->GetTransportHandle(), CURLOPT_PROXY, "172.17.1.16:80");

    // url must remain valid through the request
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_URL, job->GetUrlRequestInfo()->Url());

    if (m_cookieJarFileName) {
        curl_easy_setopt(job->GetTransportHandle(), CURLOPT_COOKIEFILE, m_cookieJarFileName);
        curl_easy_setopt(job->GetTransportHandle(), CURLOPT_COOKIEJAR, m_cookieJarFileName);
    }

    if (job->GetUrlRequestInfo()->GetRequestHeaders().size() > 0) {
        struct curl_slist* headers = 0;
        HTTPHeaderMap customHeaders = job->GetUrlRequestInfo()->GetRequestHeaders();
        HTTPHeaderMap::const_iterator end = customHeaders.end();
        for (HTTPHeaderMap::const_iterator it = customHeaders.begin(); it != end; ++it) {
            String key = it->first;
            String value = it->second;
            String headerString(key);
            headerString.append(": ");
            headerString.append(value);
            CString headerLatin1( headerString.latin1() );
            headers = curl_slist_append(headers, headerLatin1.data());
        }
        curl_easy_setopt(job->GetTransportHandle(), CURLOPT_HTTPHEADER, headers);
        // job->m_customHeaders = headers;
    }
    // <TODO> : Get headers from the request instead of hardcoded headers
    struct curl_slist* tempHeaders = 0;
    TempHeaders(&tempHeaders);
    curl_easy_setopt(job->GetTransportHandle(), CURLOPT_HTTPHEADER, tempHeaders);
    
    if (URL_GET == job->GetUrlRequestInfo()->LoadMethod())
        curl_easy_setopt(job->GetTransportHandle(), CURLOPT_HTTPGET, TRUE);
    else if (URL_POST == job->GetUrlRequestInfo()->LoadMethod())
        setupPOST(job);
    // <TODO> : Support PUT method
    /*else if ("PUT" == job->UrlRequestInfo().LoadMethod())
        setupPUT(job);*/
    else if (URL_HEAD == job->GetUrlRequestInfo()->LoadMethod())
        curl_easy_setopt(job->GetTransportHandle(), CURLOPT_NOBODY, TRUE);

    CURLMcode ret = curl_multi_add_handle(m_curlMultiHandle, job->GetTransportHandle());
    // don't call perform, because events must be async
    // timeout will occur and do curl_multi_perform
    if (ret && ret != CURLM_CALL_MULTI_PERFORM) {
#ifndef NDEBUG
        printf("Error %d starting job %s\n", ret, url.data());
#endif
        job->CancelTransaction();
        return;
    }
}

void TransactionManager::cancel(BaseTransaction* job)
{
    if (removeScheduledJob(job))
        return;
    removeFromCurl(job);
    // FIXME: report an error?
}

void TransactionManager::TempHeaders(curl_slist** headers)
{
    *headers = curl_slist_append(*headers, "User-Agent: Mozilla/5.0 (X11; U; Linux i686; en-us) AppleWebKit/420+ (KHTML, like Gecko)");
    *headers = curl_slist_append(*headers, "Agent: text/xml,application/xml,application/xhtml+xml,text/html;q=0.9,text/plain;q=0.8,image/png;*/*;q=0.5");
    *headers = curl_slist_append(*headers, "Referer: http://www.google.com");
}

// End of File

