/*
 * Copyright (C) 2006 Don Gibson <dgibson77@gmail.com>
 * Copyright (C) 2006 Zack Rusin <zack@kde.org>
 * Copyright (C) 2006, 2007 Apple Computer, Inc.  All rights reserved.
 * Copyright (C) 2007 Trolltech ASA
 * Copyright (C) 2007 Alp Toker <alp.toker@collabora.co.uk>
 * Copyright (C) 2007 Holger Hans Peter Freyther
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY APPLE COMPUTER, INC. ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL APPLE COMPUTER, INC. OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
 * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "config.h"
#include "webkitshpprivate.h"
#include "FrameLoaderClientShp.h"
#include "DocumentLoader.h"
#include "FrameLoader.h"
#include "FrameView.h"
#include "FrameTree.h"
#include "HTMLFormElement.h"
#include "HTMLFrameElement.h"
#include "HTMLFrameOwnerElement.h"
#include "HTMLNames.h"
#include "MIMETypeRegistry.h"
#include "NotImplemented.h"
#include "PlatformString.h"
#include "ResourceRequest.h"
#include "CString.h"
#include "ProgressTracker.h"
#include "webkitshppage.h"
#include "webkitshpframe.h"
#include "webkitshpprivate.h"
#include <stdio.h>

#ifdef SUPPORT_VIRTUAL_CURSOR
#include "VirtualCursor.h"
#endif //SUPPORT_VIRTUAL_CURSOR

# if defined(SHP_SUPPORT_WML)
//#include "BrCoreWMLDoControl.h"//sfb_re_eng
//#include "BrCoreWMLControl.h"
//#include "BrAppWindowErrorMsgbox.h"
//#include "BrAppWindowWTAI.h"
//#include "BrAppWindowErrorMsgbox.h"
#endif


#ifdef SUPPORT_NP_PLUGIN
#include "PluginView.h"
#include "RenderPlugin.h"
#endif // SUPPORT_NP_PLUGIN

using namespace WebCore;

namespace WebKit {

FrameLoaderClient::FrameLoaderClient(WebKitShpFrame* frame)
    : m_frame(frame)
    , m_firstData(false)
{
    ASSERT(m_frame);
}

String FrameLoaderClient::userAgent(const KURL&)
{
    return "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/420+ (KHTML, like Gecko)";
}

WTF::PassRefPtr<WebCore::DocumentLoader> FrameLoaderClient::createDocumentLoader(const WebCore::ResourceRequest& request, const SubstituteData& substituteData)
{
    RefPtr<DocumentLoader> loader = new DocumentLoader(request, substituteData);
    return loader.release();
}

void FrameLoaderClient::dispatchWillSubmitForm(FramePolicyFunction policyFunction,  PassRefPtr<FormState>)
{
    // FIXME: This is surely too simple
    ASSERT(policyFunction);
    if (!policyFunction)
        return;
    (core(m_frame)->loader()->*policyFunction)(PolicyUse);
}


void FrameLoaderClient::committedLoad(DocumentLoader* loader, const char* data, int length)
{
    FrameLoader *fl = loader->frameLoader();
    fl->setEncoding(m_response.textEncodingName(), false);
    fl->addData(data, length);
}

void FrameLoaderClient::dispatchDidReceiveAuthenticationChallenge(DocumentLoader*, unsigned long  identifier, const AuthenticationChallenge&)
{
    notImplementedInGdk();
}

void FrameLoaderClient::dispatchDidCancelAuthenticationChallenge(DocumentLoader*, unsigned long  identifier, const AuthenticationChallenge&)
{
    notImplementedInGdk();
}

void FrameLoaderClient::dispatchWillSendRequest(DocumentLoader*, unsigned long , ResourceRequest&, const ResourceResponse&)
{
    notImplementedInGdk();
}

void FrameLoaderClient::assignIdentifierToInitialRequest(unsigned long identifier, DocumentLoader*, const ResourceRequest&)
{
    notImplementedInGdk();
}

void FrameLoaderClient::postProgressStartedNotification()
{		
	WebKitShpPage* page = getPageFromFrame(m_frame);
	page->privatePage->state_info.m_stateChange = STATECHANGE_LOADING;
	if(page->m_progressHandler)
		(page->m_progressHandler)(0,0,page->privatePage->hWin,page->privatePage->state_info.m_stateChange);
	core(m_frame)->view()->setPageLoadingComplete(false);

#ifdef SUPPORT_VIRTUAL_CURSOR
	CVirtualCursor::setFrameView(core(m_frame)->view());
	CVirtualCursor::resetPageProperties();
#endif

}

void FrameLoaderClient::postProgressEstimateChangedNotification()
{
	WebKitShpPage* kitPage = getPageFromFrame(m_frame);
	kitPage->privatePage->state_info.m_stateChange = STATECHANGE_PROGRESS;
	Page* corePage = core(kitPage);
	if(kitPage->m_progressHandler)
		(kitPage->m_progressHandler) (lround(corePage->progress()->estimatedProgress()*100),corePage->progress()->totalPageAndResourceBytesToLoad(),kitPage->privatePage->hWin,kitPage->privatePage->state_info.m_stateChange);
}

void FrameLoaderClient::postProgressFinishedNotification()
{
	WebKitShpPage* page = getPageFromFrame(m_frame);
	page->isLoadingDone = true;
	core(m_frame)->view()->setPageLoadingComplete(true);
	core(m_frame)->view()->update();
	core(m_frame)->loadOnUserGesture(false);
#ifdef SUPPORT_VIRTUAL_CURSOR
	//update stats as page load is complete
	//update horizontal stats
	CVirtualCursor::updateHorizontalStats();
	//update vertical stats
	CVirtualCursor::updateVerticalStats();
#endif //SUPPORT_VIRTUAL_CURSOR
}

void FrameLoaderClient::frameLoaderDestroyed()
{
    m_frame = 0;
    delete this;
}

void FrameLoaderClient::dispatchDidReceiveResponse(DocumentLoader*, unsigned long, const ResourceResponse& response)
{
    m_response = response;
    m_firstData = true;
}

void FrameLoaderClient::dispatchDecidePolicyForMIMEType(FramePolicyFunction policyFunction, const String&, const ResourceRequest&)
{
    // FIXME: we need to call directly here (comment copied from Qt version)
    ASSERT(policyFunction);
    if (!policyFunction)
        return;
    (core(m_frame)->loader()->*policyFunction)(PolicyUse);
}

void FrameLoaderClient::dispatchDecidePolicyForNewWindowAction(FramePolicyFunction policyFunction, const NavigationAction&, const ResourceRequest&, const String&)
{
    ASSERT(policyFunction);
    if (!policyFunction)
        return;
    // FIXME: I think Qt version marshals this to another thread so when we
    // have multi-threaded download, we might need to do the same
    (core(m_frame)->loader()->*policyFunction)(PolicyIgnore);
}

void FrameLoaderClient::dispatchDecidePolicyForNavigationAction(FramePolicyFunction policyFunction, const NavigationAction&, const ResourceRequest& request)
{
    ASSERT(policyFunction);
    if (!policyFunction)
        return;

#if defined(SHP_SUPPORT_WML)
	KURL url = request.url();
	DeprecatedString dProtocol = url.protocol();
	String protocol = dProtocol;
	if((equalIgnoringCase("mailto",protocol)) ||
	   (equalIgnoringCase("tel",protocol)) ||
	   (equalIgnoringCase("wtai",protocol)))
	{
                //sfb_re_eng
		//BrAppWTAIHandleScheme((const char *)url.string().ascii().data());
		(core(m_frame)->loader()->*policyFunction)(PolicyIgnore);
	}
	else
	{
		(core(m_frame)->loader()->*policyFunction)(PolicyUse);
	}
#else
    (core(m_frame)->loader()->*policyFunction)(PolicyUse);
#endif
}

Widget* FrameLoaderClient::createPlugin(const IntSize& pluginSize, Element* e, const KURL& url, const Vector<String>& paramNames, const Vector<String>& paramValues, const String& mimeType, bool loadManually)
{
#ifdef SUPPORT_NP_PLUGIN

	PluginView* pluginView = PluginView::create(core(m_frame), pluginSize, e, url, paramNames, paramValues, mimeType, loadManually);

      if (pluginView->status() == PluginStatusLoadedSuccessfully)
        return pluginView;

 #else // SUPPORT_NP_PLUGIN
    notImplementedInGdk();
    return 0;
#endif // SUPPORT_NP_PLUGIN
}

PassRefPtr<Frame> FrameLoaderClient::createFrame(const KURL& url, const String& name, HTMLFrameOwnerElement* ownerElement,
                                        const String& referrer, bool allowsScrolling, int marginWidth, int marginHeight)
{
    Frame* coreFrame = core(webFrame());

    ASSERT(core(getPageFromFrame(webFrame())) == coreFrame->page());
    WebKitShpFrame* shpFrame = webkit_shp_frame_init_with_page(getPageFromFrame(webFrame()), ownerElement);
    Frame* childFrame = core(shpFrame);

    coreFrame->tree()->appendChild(childFrame);

    // Frames are created with a refcount of 1. Release this ref, since we've assigned it to d->frame.
    childFrame->deref();
    childFrame->tree()->setName(name);
    childFrame->init();
    childFrame->loader()->load(url, referrer, FrameLoadTypeRedirectWithLockedHistory, String(), 0, 0);

    // The frame's onload handler may have removed it from the document.
    if (!childFrame->tree()->parent())
        return 0;

    // Propagate the marginwidth/height and scrolling modes to the view.
    if (ownerElement->hasTagName(HTMLNames::frameTag) || ownerElement->hasTagName(HTMLNames::iframeTag)) {
        HTMLFrameElement* frameElt = static_cast<HTMLFrameElement*>(ownerElement);
        if (frameElt->scrollingMode() == ScrollbarAlwaysOff)
            childFrame->view()->setScrollbarsMode(ScrollbarAlwaysOff);
        int marginWidth = frameElt->getMarginWidth();
        int marginHeight = frameElt->getMarginHeight();
        if (marginWidth != -1)
            childFrame->view()->setMarginWidth(marginWidth);
        if (marginHeight != -1)
            childFrame->view()->setMarginHeight(marginHeight);
    }

    return childFrame;
}

void FrameLoaderClient::redirectDataToPlugin(Widget* pluginWidget)
{
    notImplementedInGdk();
    return;
}

Widget* FrameLoaderClient::createJavaAppletWidget(const IntSize&, Element*, const KURL& baseURL,
                                                    const Vector<String>& paramNames, const Vector<String>& paramValues)
{
    notImplementedInGdk();
    return 0;
}

ObjectContentType FrameLoaderClient::objectContentType(const KURL& url, const String& mimeType)
{
	notImplemented();
    return ObjectContentType();
}

String FrameLoaderClient::overrideMediaType() const
{
    notImplementedInGdk();
    return String();
}

void FrameLoaderClient::windowObjectCleared()
{
	notImplemented();
}

void FrameLoaderClient::setMainFrameDocumentReady(bool)
{
    // this is only interesting once we provide an external API for the DOM
	notImplementedInGdk();
}

bool FrameLoaderClient::hasWebView() const
{
    notImplementedInGdk();
    return true;
}

bool FrameLoaderClient::hasFrameView() const
{
    notImplementedInGdk();
    return true;
}

void FrameLoaderClient::dispatchDidFinishLoad()
{
    notImplemented();
}

void FrameLoaderClient::frameLoadCompleted()
{
    notImplementedInGdk();
}

void FrameLoaderClient::saveViewStateToItem(HistoryItem*)
{
    notImplementedInGdk();
}

void FrameLoaderClient::restoreViewState()
{
    notImplementedInGdk();
}

bool FrameLoaderClient::shouldGoToHistoryItem(HistoryItem* item) const
{
    // FIXME: This is a very simple implementation. More sophisticated
    // implementation would delegate the decision to a PolicyDelegate.
    // See mac implementation for example.
    return item != 0;
}

bool FrameLoaderClient::privateBrowsingEnabled() const { notImplementedInGdk(); return false; }
void FrameLoaderClient::makeDocumentView() { notImplementedInGdk(); }
void FrameLoaderClient::makeRepresentation(DocumentLoader*) { notImplementedInGdk(); }
void FrameLoaderClient::forceLayout() { notImplementedInGdk(); }
void FrameLoaderClient::forceLayoutForNonHTML() { notImplementedInGdk(); }
void FrameLoaderClient::setCopiesOnScroll() { notImplementedInGdk(); }
void FrameLoaderClient::detachedFromParent1() { notImplementedInGdk(); }
void FrameLoaderClient::detachedFromParent2() { notImplementedInGdk(); }
void FrameLoaderClient::detachedFromParent3() { notImplementedInGdk(); }
void FrameLoaderClient::detachedFromParent4() { notImplementedInGdk(); }
void FrameLoaderClient::loadedFromCachedPage() { notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidHandleOnloadEvents() {notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidReceiveServerRedirectForProvisionalLoad() { notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidCancelClientRedirect() { notImplementedInGdk(); }
void FrameLoaderClient::dispatchWillPerformClientRedirect(const KURL&, double, double) { notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidChangeLocationWithinPage() { notImplementedInGdk(); }
void FrameLoaderClient::dispatchWillClose() { notImplementedInGdk(); }

void FrameLoaderClient::dispatchDidReceiveIcon()
{
	notImplemented();
}

void FrameLoaderClient::dispatchDidStartProvisionalLoad()
{
    WebKitShpPage* page = getPageFromFrame(m_frame);
	page->isLoadingDone = false;
}

void FrameLoaderClient::dispatchDidReceiveTitle(const String& title)
{
    notImplementedInGdk();
}

void FrameLoaderClient::dispatchDidCommitLoad() { notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidFinishDocumentLoad() { notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidFirstLayout() { notImplementedInGdk(); }
void FrameLoaderClient::dispatchShow() { notImplementedInGdk(); }
void FrameLoaderClient::cancelPolicyCheck() { notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidLoadMainResource(DocumentLoader*) { notImplementedInGdk(); }
void FrameLoaderClient::revertToProvisionalState(DocumentLoader*) { notImplementedInGdk(); }
void FrameLoaderClient::clearUnarchivingState(DocumentLoader*) { notImplementedInGdk(); }
void FrameLoaderClient::willChangeTitle(DocumentLoader*) { notImplementedInGdk(); }
void FrameLoaderClient::didChangeTitle(DocumentLoader *l) { setTitle(l->title(), l->url()); }
void FrameLoaderClient::finalSetupForReplace(DocumentLoader*) { notImplementedInGdk(); }
void FrameLoaderClient::setDefersLoading(bool) { notImplementedInGdk(); }
bool FrameLoaderClient::isArchiveLoadPending(ResourceLoader*) const { notImplementedInGdk(); return false; }
void FrameLoaderClient::cancelPendingArchiveLoad(ResourceLoader*) { notImplementedInGdk(); }
void FrameLoaderClient::clearArchivedResources() { notImplementedInGdk(); }
bool FrameLoaderClient::canHandleRequest(const ResourceRequest&) const { notImplementedInGdk(); return true; }
bool FrameLoaderClient::canShowMIMEType(const String&) const { notImplementedInGdk(); return true; }
bool FrameLoaderClient::representationExistsForURLScheme(const String&) const { notImplementedInGdk(); return false; }
String FrameLoaderClient::generatedMIMETypeForURLScheme(const String&) const { notImplementedInGdk(); return String(); }

void FrameLoaderClient::finishedLoading(DocumentLoader*)
{ 
}

void FrameLoaderClient::provisionalLoadStarted()
{
    notImplementedInGdk();
}

void FrameLoaderClient::didFinishLoad() {
    notImplementedInGdk();
}

void FrameLoaderClient::prepareForDataSourceReplacement() { notImplementedInGdk(); }

void FrameLoaderClient::setTitle(const String& title, const KURL& url)
{
    WebKitShpPage* page = getPageFromFrame(m_frame);

    CString titleString = title.utf8();
    DeprecatedCString urlString = url.prettyURL().utf8();
	if(page->m_titleChangedHandler)
		(page->m_titleChangedHandler)(titleString.data(), urlString.data());
}

void FrameLoaderClient::setDocumentViewFromCachedPage(WebCore::CachedPage*) { notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidReceiveContentLength(DocumentLoader*, unsigned long  identifier, int lengthReceived) { notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidFinishLoading(DocumentLoader*, unsigned long  identifier) { notImplementedInGdk(); }
void FrameLoaderClient::dispatchDidFailLoading(DocumentLoader*, unsigned long  identifier, const ResourceError&) { notImplementedInGdk(); }
bool FrameLoaderClient::dispatchDidLoadResourceFromMemoryCache(DocumentLoader*, const ResourceRequest&, const ResourceResponse&, int length) { notImplementedInGdk(); return false; }

void FrameLoaderClient::dispatchDidFailProvisionalLoad(const ResourceError&)
{
    notImplemented();
}

void FrameLoaderClient::dispatchDidFailLoad(const ResourceError&)
{
    notImplemented();
}

void FrameLoaderClient::download(ResourceHandle*, const ResourceRequest&, const ResourceRequest&, const ResourceResponse&) { notImplementedInGdk(); }
ResourceError FrameLoaderClient::cancelledError(const ResourceRequest&) { notImplementedInGdk(); return ResourceError(); }
ResourceError FrameLoaderClient::blockedError(const ResourceRequest&) { notImplementedInGdk(); return ResourceError(); }
ResourceError FrameLoaderClient::cannotShowURLError(const ResourceRequest&) { notImplementedInGdk(); return ResourceError(); }
ResourceError FrameLoaderClient::interruptForPolicyChangeError(const ResourceRequest&) { notImplementedInGdk(); return ResourceError(); }
ResourceError FrameLoaderClient::cannotShowMIMETypeError(const ResourceResponse&) { notImplementedInGdk(); return ResourceError(); }
ResourceError FrameLoaderClient::fileDoesNotExistError(const ResourceResponse&) { notImplementedInGdk(); return ResourceError(); }
bool FrameLoaderClient::shouldFallBack(const ResourceError&) { notImplementedInGdk(); return false; }
bool FrameLoaderClient::willUseArchive(ResourceLoader*, const ResourceRequest&, const KURL& originalURL) const { notImplementedInGdk(); return false; }
void FrameLoaderClient::saveDocumentViewToCachedPage(CachedPage*) { notImplementedInGdk(); }
bool FrameLoaderClient::canCachePage() const { notImplementedInGdk(); return false; }
Frame* FrameLoaderClient::dispatchCreatePage() { notImplementedInGdk(); return 0; }
void FrameLoaderClient::dispatchUnableToImplementPolicy(const ResourceError&) { notImplementedInGdk(); }
void FrameLoaderClient::setMainDocumentError(DocumentLoader*, const ResourceError&) { notImplementedInGdk(); }
void FrameLoaderClient::startDownload(const ResourceRequest&) { notImplementedInGdk(); }
void FrameLoaderClient::updateGlobalHistoryForStandardLoad(const KURL&) { notImplementedInGdk(); }
void FrameLoaderClient::updateGlobalHistoryForReload(const KURL&) { notImplementedInGdk(); }

#if defined ( SHP_SUPPORT_WML )
void FrameLoaderClient::delegateDoElementToUserMenu ( int id, WebCore::String displayString)
		{
	//sfb_re_eng
	//		BrCoreControl_WMLButtonData ( id, ( MCHAR*)displayString.charactersWithNullTermination(), displayString.length() ) ; 
		}
void FrameLoaderClient::removeDoElementinUserMenu (int id )
{
	//sfb_re_eng
	//	BrAppWmlTextData ( id ) ;
}
# endif// SHP_SUPPORT_WML


void FrameLoaderClient::showErrorMessageBox(const char *errMsg)
{
	//sfb_re_eng
	//	BrAppWindowsCreateErrorMsgbox(errMsg);
}

void FrameLoaderClient::savePlatformDataToCachedPage(WebCore::CachedPage*)
{
    notImplemented();
}

void FrameLoaderClient::transitionToCommittedFromCachedPage(WebCore::CachedPage*)
{
    notImplemented();
}
void FrameLoaderClient::transitionToCommittedForNewPage()
{
    notImplemented();
}
void FrameLoaderClient::didPerformFirstNavigation() const
{
    notImplemented();
}

void FrameLoaderClient::registerForIconNotification(bool)
{
    notImplemented();
}

void FrameLoaderClient::download( void* loader, int transID, const ResourceRequest& request, const ResourceRequest& request2, const ResourceResponse& response ) 
{
	    notImplemented();
}

} //namespace Webkit

