/*
     File:       CFNetwork/CFURLCredentialPriv.h
     Copyright:  (c) 2007 by Apple Inc., all rights reserved
*/
#ifndef __CFURLCREDENTIALPRIV__
#define __CFURLCREDENTIALPRIV__

#ifndef __CFNETWORKDEFS__
#include <CFNetwork/CFNetworkDefs.h>
#endif

#ifndef __COREFOUNDATION__
#include <CoreFoundation/CoreFoundation.h>
#endif


#if defined(__MACH__)
    #ifndef _SECURITY_SECBASE_H_
    #include <Security/SecBase.h>
    #endif
#endif /* defined(__MACH__) */


#include <AvailabilityMacros.h>

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint on
#endif


/*
 *  CFURLCredentialRef
 *  
 *  Discussion:
 *    This is the type of a reference CFURLCredential. NOTE:
 *    CFURLCredential objects may cause Keychain Services' optional
 *    user interface to be displayed. The function
 *    SecKeychainSetUserInteractionAllowed() can be used to enable or
 *    disable the Keychain Services optional user interface.
 */
typedef struct _CFURLCredential*        CFURLCredentialRef;

/*
 *  CFURLCredentialPersistence
 *  
 *  Discussion:
 *    The persistence setting for the CFURLCredential object.
 */
enum CFURLCredentialPersistence {

  /*
   * This credential won't be saved.
   */
  kCFURLCredentialPersistenceNone = 1,

  /*
   * This credential will only be stored for this session.
   */
  kCFURLCredentialPersistenceForSession = 2,

  /*
   * This credential will be stored permanently and shared with other
   * applications.
   */
  kCFURLCredentialPersistencePermanent = 3
};
typedef enum CFURLCredentialPersistence CFURLCredentialPersistence;


/*
 *  CFURLCredentialGetTypeID()
 *  
 *  Discussion:
 *    Returns the type identifier of all CFURLCredential instances.
 *  
 */
APPLE_WIN_API CFTypeID 
CFURLCredentialGetTypeID(void)                                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLCredentialCreate()
 *  
 *  Discussion:
 *    Creates a new CFURLCredential object.
 *  
 *  Parameters:
 *    
 *    allocator:
 *      The CFAllocator which should be used to allocate memory for the
 *      CFURLCredential. If this reference is not a valid CFAllocator,
 *      the behavior is undefined.
 *    
 *    username:
 *      A CFString containing the username.
 *    
 *    password:
 *      A CFString containing the password.
 *    
 *    accountDomain:
 *      A CFString containing the account domain, or NULL if there is
 *      no account domain.
 *    
 *    persistence:
 *      The persistence setting for the CFURLCredential:
 *      kCFURLCredentialPersistenceNone,
 *      kCFURLCredentialPersistenceForSession, or
 *      kCFURLCredentialPersistencePermanent.
 *  
 *  Result:
 *    A valid CFURLCredentialRef, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFURLCredentialRef 
CFURLCredentialCreate(
  CFAllocatorRef               allocator,
  CFStringRef                  username,
  CFStringRef                  password,
  CFStringRef                  accountDomain,       /* can be NULL */
  CFURLCredentialPersistence   persistence)                   AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;

/*
 *  CFURLCredentialCreateCopy()
 *  
 *  Discussion:
 *    Creates a new copy of a CFURLCredential object.
 *  
 *  Parameters:
 *    
 *    allocator:
 *      The CFAllocator which should be used to allocate memory for the
 *      CFURLCredential. If this reference is not a valid CFAllocator,
 *      the behavior is undefined.
 *    
 *    theCredential:
 *      The CFURLCredential object to be copied.
 *  
 *  Result:
 *    A valid CFURLCredentialRef, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFURLCredentialRef 
CFURLCredentialCreateCopy(
  CFAllocatorRef       allocator,
  CFURLCredentialRef   theCredential)                         AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCredentialGetUsername()
 *  
 *  Discussion:
 *    Gets the username from a CFURLCredential object.
 *  
 *  Parameters:
 *    
 *    credential:
 *      The CFURLCredential object.
 *  
 *  Result:
 *    A CFStringRef containing the username, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFStringRef 
CFURLCredentialGetUsername(CFURLCredentialRef credential)     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCredentialCopyPassword()
 *  
 *  Discussion:
 *    Copies the password from a CFURLCredential object. This is a copy
 *    operation because a new copy of the password must be created if
 *    the password is stored in a SecKeychainItem.
 *  
 *  Parameters:
 *    
 *    credential:
 *      The CFURLCredential object.
 *  
 *  Result:
 *    A CFStringRef containing the password, or NULL if unsuccessful.
 *  
 */
APPLE_WIN_API CFStringRef 
CFURLCredentialCopyPassword(CFURLCredentialRef credential)    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCredentialContainsPassword()
 *  
 *  Discussion:
 *    Determines if a CFURLCredential object contains a password.
 *  
 *  Parameters:
 *    
 *    credential:
 *      The CFURLCredential object.
 *  
 *  Result:
 *    TRUE if there is a password; otherwise, FALSE.
 *  
 */
APPLE_WIN_API Boolean 
CFURLCredentialContainsPassword(CFURLCredentialRef credential) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCredentialGetAccountDomain()
 *  
 *  Discussion:
 *    Gets the account domain from a CFURLCredential object.
 *  
 *  Parameters:
 *    
 *    credential:
 *      The CFURLCredential object.
 *  
 *  Result:
 *    A CFString containing the account domain, or NULL if there is no
 *    account domain.
 *  
 */
APPLE_WIN_API CFStringRef 
CFURLCredentialGetAccountDomain(CFURLCredentialRef credential) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLCredentialGetPersistence()
 *  
 *  Discussion:
 *    Gets the persistence setting of a CFURLCredential object.
 *  
 *  Parameters:
 *    
 *    credential:
 *      The CFURLCredential object.
 *  
 *  Result:
 *    The persistence setting of a CFURLCredential object.
 *  
 */
APPLE_WIN_API CFURLCredentialPersistence 
CFURLCredentialGetPersistence(CFURLCredentialRef credential)  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __CFURLCREDENTIALPRIV__ */

