/*
     File:       CFNetwork/CFURLDownloadPriv.h
     Copyright:  (c) 2007 by Apple Inc., all rights reserved
*/
#ifndef __CFURLDOWNLOADPRIV__
#define __CFURLDOWNLOADPRIV__

#ifndef __CFNETWORKDEFS__
#include <CFNetwork/CFNetworkDefs.h>
#endif

#ifndef __COREFOUNDATION__
#include <CoreFoundation/CoreFoundation.h>
#endif

#ifndef __CFURLCONNECTIONPRIV__
#include <CFNetwork/CFURLConnectionPriv.h>
#endif



#include <AvailabilityMacros.h>

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#pragma pack(push, 2)

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint on
#endif


/*
 *  CFURLDownload
 *  
 *  Discussion:
 *    CFURLDownload is a CFType that represents a download attempt of a
 *    CFURLRequest and saving the downloaded data to a file.  The
 *    progress of the download is reported to the client object.  Note:
 *    The word "download" is used to refer to the process of loading
 *    data off a network, decoding the data if necessary and saving the
 *    data to a file.
 */
typedef struct _CFURLDownload*          CFURLDownloadRef;

/*
 *  CFURLDownloadDidStartCallback
 *  
 *  Discussion:
 *    Callback function which is called when a download starts.  This
 *    callback is sort of redundant in the standard case where
 *    CFURLDownloadStart is called but is provided to support the
 *    CFURLDownloadCreateWithLoadingConnection case.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 */
typedef CALLBACK_API_C( void , CFURLDownloadDidStartCallback )(CFURLDownloadRef download, const void *clientInfo);

/*
 *  CFURLDownloadWillSendRequestCallback
 *  
 *  Discussion:
 *    Callback function which is called when the download object
 *    determines that is must change URLs in order to continue the
 *    download.  If the client is sataified with the proposed
 *    redirected request then the callback should return the passed in
 *    request after calling CFRetain on it (the download will take
 *    ownership of the returned request).  The client can return NULL
 *    and the redirect will be canceled (or the client can cancel the
 *    entire operation by calling CFURLDownloadCancel).
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    request:
 *      The proposed redirected request.
 *    
 *    redirectionResponse:
 *      The URL response that caused the redirect.  May be NULL.
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 *  
 *  Result:
 *    The actual URL request to use in light of the redirectionResponse.
 */
typedef CALLBACK_API_C( CFURLRequestRef , CFURLDownloadWillSendRequestCallback )(CFURLDownloadRef download, CFURLRequestRef request, CFURLResponseRef redirectionResponse, const void *clientInfo);

/*
 *  CFURLDownloadDidReceiveAuthenticationChallengeCallback
 *  
 *  Discussion:
 *    Callback function which is called when a download must
 *    authenticate a challenge in order to continue.  This callback
 *    provides the client the opportunity to determine the couuse of
 *    action to take for the challange.  The client can provide
 *    credentials, continue without credentials or cancel the download.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    challenge:
 *      The URL authentication challenge that must by authenticated in
 *      order to download the request.
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 */
typedef CALLBACK_API_C( void , CFURLDownloadDidReceiveAuthenticationChallengeCallback )(CFURLDownloadRef download, CFURLAuthChallengeRef challenge, const void *clientInfo);

/*
 *  CFURLDownloadDidReceiveResponseCallback
 *  
 *  Discussion:
 *    Callback function which is called when a download has received
 *    sufficient load data to construct the CFURLResponse object for
 *    the download. In some rare cases multiple responses may be
 *    received for a single download.  Should this occur the client
 *    should assume that each new response resets the download progress
 *    to 0 and should check the new response for the expected content
 *    length.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    response:
 *      The CFURLResponse object received as part of the download.
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 */
typedef CALLBACK_API_C( void , CFURLDownloadDidReceiveResponseCallback )(CFURLDownloadRef download, CFURLResponseRef response, const void *clientInfo);

/*
 *  CFURLDownloadWillResumeWithResponseCallback
 *  
 *  Discussion:
 *    Callback function which is called when a download has receieved a
 *    response from the server after attempting to resume a download.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    response:
 *      The CFURLResponse object received as part of the download.
 *    
 *    startingByte:
 *      the location of the start of the resumed data (in bytes)
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 */
typedef CALLBACK_API_C( void , CFURLDownloadWillResumeWithResponseCallback )(CFURLDownloadRef download, CFURLResponseRef response, UInt64 startingByte, const void *clientInfo);

/*
 *  CFURLDownloadDidReceiveDataCallback
 *  
 *  Discussion:
 *    Callback function which is called when a download receives data
 *    incrementally.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    length:
 *      The amount of data received in this increment, measured in
 *      bytes.
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 */
typedef CALLBACK_API_C( void , CFURLDownloadDidReceiveDataCallback )(CFURLDownloadRef download, CFIndex length, const void *clientInfo);

/*
 *  CFURLDownloadShouldDecodeDataOfMIMETypeCallback
 *  
 *  Discussion:
 *    Callback function which is called when a download determines that
 *    a downloaded object is encoded to inquire if the file should be
 *    automatically decoded. Return true to automatically decode the
 *    file, false to leave it encoded.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    encodingType:
 *      The type of the encoding used by the downloaded object.  The
 *      supported encoding formats are gzip ("application/gzip"),
 *      Binhex ("application/mac-binhex40") and MacBinary
 *      ("application/macbinary").
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 *  
 *  Result:
 *    true to decode, false to leave encoded
 */
typedef CALLBACK_API_C( Boolean , CFURLDownloadShouldDecodeDataOfMIMETypeCallback )(CFURLDownloadRef download, CFStringRef encodingType, const void *clientInfo);

/*
 *  CFURLDownloadDecideDestinationWithSuggestedObjectNameCallback
 *  
 *  Discussion:
 *    Callback function which is called when a download has determined
 *    a suggested name for the downloaded object.  The client can
 *    modify the path by calling CFURLDownloadSetDestination.  This
 *    callback will not be received if CFURLDownloadSetDestination has
 *    already been called on the download.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    objectName:
 *      The proposed name for the downloaded object.
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 */
typedef CALLBACK_API_C( void , CFURLDownloadDecideDestinationWithSuggestedObjectNameCallback )(CFURLDownloadRef download, CFStringRef objectName, const void *clientInfo);

/*
 *  CFURLDownloadDidCreateDestinationCallback
 *  
 *  Discussion:
 *    Callback function which is called when the download has created
 *    the destination object.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    path:
 *      CFURL file system path of created object.
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 */
typedef CALLBACK_API_C( void , CFURLDownloadDidCreateDestinationCallback )(CFURLDownloadRef download, CFURLRef path, const void *clientInfo);

/*
 *  CFURLDownloadDidFinishCallback
 *  
 *  Discussion:
 *    Callback function which is called when a download is complete.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 */
typedef CALLBACK_API_C( void , CFURLDownloadDidFinishCallback )(CFURLDownloadRef download, const void *clientInfo);

/*
 *  CFURLDownloadDidFailCallback
 *  
 *  Discussion:
 *    Callback function which is called if a download fails.
 *  
 *  Parameters:
 *    
 *    download:
 *      The download object receiving the callback.
 *    
 *    error:
 *      The reason the download failed.
 *    
 *    clientInfo:
 *      The clientInfo pointer passed in by the client when the
 *      download was created.
 */
typedef CALLBACK_API_C( void , CFURLDownloadDidFailCallback )(CFURLDownloadRef download, CFErrorRef error, const void *clientInfo);
/* This structure is passed to the various download create calls.  It specifies a pointer value that
    will be provided to all client callbacks along with the callbacks themselves.  The data will be 
    copied from the passed in structure (and the download object will retain the clientInfo if a retain
    callback is provided). */
struct CFURLDownloadClient {
  CFIndex             version;                /* set to 0 */
  const void *        clientInfo;             /* this value will be passed in to client callbacks */

  CFAllocatorRetainCallBack  retain;          /* A pointer to the function to call to retain the clientInfo object.  Pass in NULL if no retain is needed */
  CFAllocatorReleaseCallBack  release;        /* A pointer to the function to call to release the clientInfo object.  Pass in NULL if no release is needed */
  CFAllocatorCopyDescriptionCallBack  copyDescription;

  CFURLDownloadDidStartCallback  didStart;
  CFURLDownloadWillSendRequestCallback  willSendRequest; /* this callback uses the same ownership policy as the v1 CFURLConnection callback of the same name (i.e. this callback should return a reference to the returned object) */
  CFURLDownloadDidReceiveAuthenticationChallengeCallback  didReceiveAuthenticationChallenge;
  CFURLDownloadDidReceiveResponseCallback  didReceiveResponse;
  CFURLDownloadWillResumeWithResponseCallback  willResumeWithResponse;
  CFURLDownloadDidReceiveDataCallback  didReceiveData;
  CFURLDownloadShouldDecodeDataOfMIMETypeCallback  shouldDecodeDataOfMIMEType;
  CFURLDownloadDecideDestinationWithSuggestedObjectNameCallback  decideDestinationWithSuggestedObjectName;
  CFURLDownloadDidCreateDestinationCallback  didCreateDestination;
  CFURLDownloadDidFinishCallback  didFinish;
  CFURLDownloadDidFailCallback  didFail;
};
typedef struct CFURLDownloadClient      CFURLDownloadClient;
/*
 *  CFURLDownloadGetTypeID()
 *  
 *  Discussion:
 *    This routine will return the CFTypeID for the CFURLDownload type.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Result:
 *    the CFTypeID for the CFURLDownload type.
 *  
 */
APPLE_WIN_API CFTypeID 
CFURLDownloadGetTypeID(void)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCreate()
 *  
 *  Discussion:
 *    This routine will create a CFURLDownload using the provided
 *    CFURLRequest.  The download needs to scheduled using
 *    CFDownloadScheduleWithRunLoop and explicitly started using
 *    CFURLDownloadStart.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    alloc:
 *      The CFAllocator to use.  Pass NULL for the default allocator.
 *    
 *    request:
 *      A CFURLRequest specifying the source of the download.  This
 *      request will be used to create a CFURLConnection to get the
 *      data from.
 *    
 *    client:
 *      A filled in CFURLDownloadClient struct.  The data will be
 *      copied by the download.
 *  
 *  Result:
 *    A reference to the newly created object or NULL if the creation
 *    failed.
 *  
 */
APPLE_WIN_API CFURLDownloadRef 
CFURLDownloadCreate(
  CFAllocatorRef         alloc,
  CFURLRequestRef        request,
  CFURLDownloadClient *  client)                              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCreateWithResumeData()
 *  
 *  Discussion:
 *    This routine will create a CFURLDownload using the provided
 *    resumeData.  The resume data should have been created from a
 *    previously stopped download using CFURLDownloadCopyResumeData. 
 *    The download needs to scheduled using
 *    CFDownloadScheduleWithRunLoop and explicitly started using
 *    CFURLDownloadStart.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    alloc:
 *      The CFAllocator to use.  Pass NULL for the default allocator.
 *    
 *    resumeData:
 *      The resumeData returned by an earlier call to
 *      CFURLDownloadCopyResumeData
 *    
 *    path:
 *      Path to partially downloaded object to use as the basis for the
 *      resumed download.
 *    
 *    client:
 *      A filled in CFURLDownloadClient struct.  The data will be
 *      copied by the download.
 *  
 *  Result:
 *    A reference to the newly created object or NULL if the creation
 *    failed.
 *  
 */
APPLE_WIN_API CFURLDownloadRef 
CFURLDownloadCreateWithResumeData(
  CFAllocatorRef         alloc,
  CFDataRef              resumeData,
  CFURLRef               path,
  CFURLDownloadClient *  client)                              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadScheduleWithRunLoop()
 *  
 *  Discussion:
 *    Schedule the given operation on the specified runloop and mode. A
 *    CFURLDownload can be scheduled on multiple runloop/mode
 *    combinations. An opertion must be scheduled on at least one
 *    runloop.  Callbacks will be made on one of the schedule
 *    runloop/modes.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The CFURLDownload to schedule.
 *    
 *    rl:
 *      The runLoop on which to schedule the operation.  Must be
 *      non-NULL.
 *    
 *    mode:
 *      The mode on which to schedule the operation.  Must be non-NULL.
 *  
 */
APPLE_WIN_API void 
CFURLDownloadScheduleWithRunLoop(
  CFURLDownloadRef   download,
  CFRunLoopRef       rl,
  CFStringRef        mode)                                    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadUnscheduleFromRunLoop()
 *  
 *  Discussion:
 *    Unschedule the given operation from the specified runloop and
 *    mode.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The CFURLDownload to unschedule.
 *    
 *    rl:
 *      The runLoop from which to unschedule the operation.  Must be
 *      non-NULL.
 *    
 *    mode:
 *      The mode from which to unschedule the operation.  Must be
 *      non-NULL.
 *  
 */
APPLE_WIN_API void 
CFURLDownloadUnscheduleFromRunLoop(
  CFURLDownloadRef   download,
  CFRunLoopRef       rl,
  CFStringRef        mode)                                    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadStart()
 *  
 *  Discussion:
 *    Starts an already created and scheduled download operation.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download operation to start.  Must be scheduled on at least
 *      one runloop.
 *  
 */
APPLE_WIN_API void 
CFURLDownloadStart(CFURLDownloadRef download)                 AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCancel()
 *  
 *  Discussion:
 *    Cancels a running download operation.  By default the destination
 *    file will be deleted. If you want to be able to resume the
 *    operation make sure to set deletesUponFailure to false using
 *    CFURLDownloadSetDeletesUponFailure before canceling the download.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download operation to cancel.
 *  
 */
APPLE_WIN_API void 
CFURLDownloadCancel(CFURLDownloadRef download)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLDownloadSetDeletesUponFailure()
 *  
 *  Discussion:
 *    Sets a flag that determines if the partially downloaded object
 *    should be deleted if the download fails or is canceled.  The
 *    default value for a new download is true.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download to set the value for.
 *    
 *    deletesUponFailure:
 *      The value to set the flag.  If false then a partially
 *      downloaded file will be left on disk, which is necessary for
 *      the resume case.  If true the partially downloaded file will be
 *      deleted.
 *  
 */
APPLE_WIN_API void 
CFURLDownloadSetDeletesUponFailure(
  CFURLDownloadRef   download,
  Boolean            deletesUponFailure)                      AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadDeletesUponFailure()
 *  
 *  Discussion:
 *    Returns the current value of the deletesUponFailure flag for the
 *    download.  The default value for a new download is true.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download to get the value for.
 *  
 */
APPLE_WIN_API Boolean 
CFURLDownloadDeletesUponFailure(CFURLDownloadRef download)    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadUseCredential()
 *  
 *  Discussion:
 *    Used to provide credentials for an authentication challenge. 
 *    This is expected to be called from within a
 *    didReceiveAuthenticationChallenge callback.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download operation provide credentials for.
 *    
 *    creds:
 *      The credentials to use.  Pass NULL to use no credentials.
 *    
 *    challenge:
 *      The challenge to use the credentials on.
 *  
 */
APPLE_WIN_API void 
CFURLDownloadUseCredential(
  CFURLDownloadRef        download,
  CFURLCredentialRef      creds,           /* can be NULL */
  CFURLAuthChallengeRef   challenge)                          AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadSetDestination()
 *  
 *  Discussion:
 *    Sets the destination file for the download operation to the
 *    specified path.  The allowOverwrite flag determines if the
 *    operation should delete an object that exists at that path or
 *    fail if an object is already present.  Typically this call is
 *    made from withing the decideDestinationWithSuggestedObjectName
 *    callback, using the suggested object name.  Once a destination is
 *    set future calls on the same download will be ignored.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download to set the path for.
 *    
 *    path:
 *      A CFURL specifying the path for the destination object.  This
 *      path includes the filename (i.e. is not a parent directory
 *      path).
 *    
 *    allowOverwrite:
 *      If true then the download will overwrite an existing object at
 *      path, otherwise the download will fail if an existing object is
 *      encountered.
 *  
 */
APPLE_WIN_API void 
CFURLDownloadSetDestination(
  CFURLDownloadRef   download,
  CFURLRef           path,
  Boolean            allowOverwrite)                          AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCopyDestination()
 *  
 *  Discussion:
 *    Returns a CFURL containing the path of the destination object for
 *    the download.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download to copy the destination from.
 *  
 *  Result:
 *    A CFURL containing the path of the destination object for this
 *    download.  If the destination has not been determined yet then
 *    NULL will be returned.
 *  
 */
APPLE_WIN_API CFURLRef 
CFURLDownloadCopyDestination(CFURLDownloadRef download)       AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCopyRequest()
 *  
 *  Discussion:
 *    Returns the CFURLRequest associated with the download.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download to copy the request from.
 *  
 */
APPLE_WIN_API CFURLRequestRef 
CFURLDownloadCopyRequest(CFURLDownloadRef download)           AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCopyResumeData()
 *  
 *  Discussion:
 *    Returns the information necessary to resume a canceled download
 *    in a serialzed form suitable for writing out to persistent
 *    storage.  The resumeData can be used to resume the download by
 *    creating a new CFURLDownload using
 *    CFURLDownloadCreateWithResumeData.  This call should be made
 *    after canceling the download.  The download should have had
 *    deletesUponFailure set to false before the cancel occurs
 *    (otherwise the partially downloaded file will be deleted).
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download to copy the resumeData from.
 *  
 */
APPLE_WIN_API CFDataRef 
CFURLDownloadCopyResumeData(CFURLDownloadRef download)        AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCanResumeDownloadDecodedWithEncodingMIMEType()
 *  
 *  Discussion:
 *    This routine will return true if a download decoded for the
 *    passed in MIMEType can be resumed.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    MIMEType:
 *      The MIMEType to check.  If decoding the provided MIMEType is
 *      unsupported then false will be returned.
 *  
 *  Result:
 *    True if the download could be resumed if the download was decoded
 *    for this MIMEType, false otherwise.
 *  
 */
APPLE_WIN_API Boolean 
CFURLDownloadCanResumeDownloadDecodedWithEncodingMIMEType(CFStringRef MIMEType) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCreateAndStartWithLoadingConnection()
 *  
 *  Discussion:
 *    This routine will create a CFURLDownload using an existing
 *    CFURLConnection.  The download will start automatically since the
 *    connection is live at the time this call is made.  This can only
 *    be made before any data has been received on the connection.  The
 *    expected use if from within the connection's didReceiveRespone
 *    callback.  The download callbacks will arrive on the runloop/mode
 *    combinations that the underlying connection is scheduled on. 
 *    There is no need to call CFURLDownloadStart on a download created
 *    with this function.  This function will return NULL if the
 *    connection is not in the proper state to be used with a download.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    alloc:
 *      The CFAllocator to use.  Pass NULL for the default allocator.
 *    
 *    conn:
 *      The connection to "convert" into a download.
 *    
 *    request:
 *      The request associated with the connection object.
 *    
 *    response:
 *      The response object to associate with the download (such as the
 *      response provided to the connection's didReceiveRespone
 *      callback).
 *    
 *    client:
 *      A filled in CFURLDownloadClient struct.  The data will be
 *      copied by the download.
 *  
 *  Result:
 *    A reference to the newly created object or NULL if the creation
 *    failed.
 *  
 */
APPLE_WIN_API CFURLDownloadRef 
CFURLDownloadCreateAndStartWithLoadingConnection(
  CFAllocatorRef         alloc,
  CFURLConnectionRef     conn,
  CFURLRequestRef        request,
  CFURLResponseRef       response,
  CFURLDownloadClient *  client)                              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCreateWithResumeInformation()
 *  
 *  Discussion:
 *    This routine will create a CFURLDownload using the provided
 *    resumeInformation.  The resume information should have been
 *    created from a previously stopped download using
 *    CFURLDownloadCopyResumeInformation.  The download needs to
 *    scheduled using CFDownloadScheduleWithRunLoop and explicitly
 *    started using CFURLDownloadStart.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    alloc:
 *      The CFAllocator to use.  Pass NULL for the default allocator.
 *    
 *    resumeInformation:
 *      The resumeInformation returned by an earlier call to
 *      CFURLDownloadCopyResumeInformation.
 *    
 *    path:
 *      Path to partially downloaded object to use as the basis for the
 *      resumed download.
 *    
 *    client:
 *      A filled in CFURLDownloadClient struct.  The data will be
 *      copied by the download.
 *  
 *  Result:
 *    A reference to the newly created object or NULL if the creation
 *    failed.
 *  
 */
APPLE_WIN_API CFURLDownloadRef 
CFURLDownloadCreateWithResumeInformation(
  CFAllocatorRef         alloc,
  CFDictionaryRef        resumeInformation,
  CFURLRef               path,
  CFURLDownloadClient *  client)                              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCopyResumeInformation()
 *  
 *  Discussion:
 *    Returns the information necessary to resume a canceled download
 *    in a CFDictionary.  The resumeInformation is not serialized as
 *    resumeData.  The resumeInformation can be used to resume the
 *    download by creating a new CFURLDownload using
 *    CFURLDownloadCreateWithResumeInformation.  This call should be
 *    made after canceling the download.  The download should have had
 *    deletesUponFailure set to false before the cancel occurs
 *    (otherwise the partially downloaded file will be deleted).  This
 *    function is provided for cases where the data does not need to
 *    serialized.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download to copy the resumeInformation from.
 *  
 */
APPLE_WIN_API CFDictionaryRef 
CFURLDownloadCopyResumeInformation(CFURLDownloadRef download) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLDownloadSetDirectoryPath()
 *  
 *  Discussion:
 *    This sets the parent directory to use for the download
 *    destination file in the absence of an explicit destination (see
 *    CFURLDownloadSetDestination).  The download target will be this
 *    path plus the suggested object name.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download to set the directory path for.
 *    
 *    directoryPath:
 *      A CFURL representing the desired parent directory for the
 *      download.
 *  
 */
APPLE_WIN_API void 
CFURLDownloadSetDirectoryPath(
  CFURLDownloadRef   download,
  CFURLRef           directoryPath)                           AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCopyDirectoryPath()
 *  
 *  Discussion:
 *    Returns the directory path associated with the download or NULL
 *    if one has not been set.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    download:
 *      The download to copy the directory path from.
 *  
 */
APPLE_WIN_API CFURLRef 
CFURLDownloadCopyDirectoryPath(CFURLDownloadRef download)     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadCreateWithDirectoryPath()
 *  
 *  Discussion:
 *    This is equivalent to calling CFURLDownCreate followed by
 *    CFURLDownloadSetDirectoryPath.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 */
APPLE_WIN_API CFURLDownloadRef 
CFURLDownloadCreateWithDirectoryPath(
  CFAllocatorRef         alloc,
  CFURLRequestRef        request,
  CFURLRef               directoryPath,
  CFURLDownloadClient *  client)                              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
   If you want the actual download work to take on a thread other than where the download itself is scheduled, call these.  
   Otherwise, the download will take place wherever the connection itself has been scheduled.
*/
/*
 *  CFURLDownloadScheduleDownloadWithRunLoop()
 *  
 */
APPLE_WIN_API void 
CFURLDownloadScheduleDownloadWithRunLoop(
  CFURLDownloadRef   download,
  CFRunLoopRef       rl,
  CFStringRef        mode)                                    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLDownloadUnscheduleDownloadFromRunLoop()
 *  
 */
APPLE_WIN_API void 
CFURLDownloadUnscheduleDownloadFromRunLoop(
  CFURLDownloadRef   download,
  CFRunLoopRef       rl,
  CFStringRef        mode)                                    AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


#if defined(__WIN32__)
/*
 *  CFURLDownloadScheduleWithCurrentMessageQueue()
 *  
 */
APPLE_WIN_API void 
CFURLDownloadScheduleWithCurrentMessageQueue(CFURLDownloadRef download) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


#endif
/* CFURLDownloadCreateWithLoadingConnection is on its way out, since the connClient parameter is unnecessary - use CFURLDownloadCreateAndStartWithLoadingConnection instead */
/*
 *  CFURLDownloadCreateWithLoadingConnection()
 *  
 *  Discussion:
 *    This now deprecated routine (use
 *    CFURLDownloadCreateAndStartWithLoadingConnection) will create a
 *    CFURLDownload using an existing CFURLConnection. The download
 *    will start automatically since the connection is live at the time
 *    this call is made.  This can only be made before any data has
 *    been received on the connection.  The expected use if from within
 *    the connection's didReceiveRespone callback.  The download
 *    callbacks will arrive on the runloop/mode combinations that the
 *    underlying connection is scheduled on.  There is no need to call
 *    CFURLDownloadStart on a download created with this function.
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    alloc:
 *      The CFAllocator to use.  Pass NULL for the default allocator.
 *    
 *    conn:
 *      The connection to "convert" into a download.
 *    
 *    request:
 *      The request associated with the connection object.
 *    
 *    response:
 *      The response object to associate with the download (such as the
 *      response provided to the connection's didReceiveRespone
 *      callback).
 *    
 *    connClient:
 *      << THIS IS NOT NECESSARY >>
 *    
 *    client:
 *      A filled in CFURLDownloadClient struct.  The data will be
 *      copied by the download.
 *  
 *  Result:
 *    A reference to the newly created object or NULL if the creation
 *    failed.
 *  
 */
APPLE_WIN_API CFURLDownloadRef 
CFURLDownloadCreateWithLoadingConnection(
  CFAllocatorRef           alloc,
  CFURLConnectionRef       conn,
  CFURLRequestRef          request,
  CFURLResponseRef         response,
  CFURLConnectionClient *  connClient,       /* can be NULL */
  CFURLDownloadClient *    client)                            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
#endif

#pragma pack(pop)

#ifdef __cplusplus
}
#endif

#endif /* __CFURLDOWNLOADPRIV__ */

