/*
     File:       CFNetwork/CFURLResponsePriv.h
     Copyright:  (c) 2007 by Apple Inc., all rights reserved
*/
#ifndef __CFURLRESPONSEPRIV__
#define __CFURLRESPONSEPRIV__

#ifndef __CFNETWORKDEFS__
#include <CFNetwork/CFNetworkDefs.h>
#endif

#ifndef __COREFOUNDATION__
#include <CoreFoundation/CoreFoundation.h>
#endif

#ifndef __CFHTTPMESSAGE__
#include <CFNetwork/CFHTTPMessage.h>
#endif




#include <AvailabilityMacros.h>

#if PRAGMA_ONCE
#pragma once
#endif

#ifdef __cplusplus
extern "C" {
#endif

#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint on
#endif


/*
 *  CFURLCacheStoragePolicy
 *  
 *  Discussion:
 *    The CFURLCacheStoragePolicy enum defines constants that can be
 *    used to specify the type of storage that is allowable for an
 *    CFCachedURLResponse object that is to be stored in an CFURLCache.
 *     This will ultimately live in CFURLCache.h, but for now, we keep
 *    it in CFURLResponse, where it is needed.
 */
enum CFURLCacheStoragePolicy {

  /*
   * Specifies that the response may be stored in a cache without
   * restriction.
   */
  kCFURLCacheStorageAllowed     = 0,

  /*
   * Specifies that the response may be stored in a cache, but the
   * storage should be in memory only. The response may not be written
   * to disk.
   */
  kCFURLCacheStorageAllowedInMemoryOnly = 1,

  /*
   * Specifies that the response may not be stored in a cache in any
   * fashion.
   */
  kCFURLCacheStorageNotAllowed  = 2
};
typedef enum CFURLCacheStoragePolicy CFURLCacheStoragePolicy;

APPLE_WIN_API const SInt64 kCFURLResponseExpectedContentLengthUnknown       AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;
typedef struct _CFURLResponse*          CFURLResponseRef;
/*
 *  CFURLResponseGetTypeID()
 *  
 */
APPLE_WIN_API CFTypeID 
CFURLResponseGetTypeID(void)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;



/*
 *  CFURLResponseCreate()
 *  
 *  Discussion:
 *    Create a new response
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    alloc:
 *      The allocator to use to create the new object
 *    
 *    URL:
 *      The URL requested that generated this response; must not be NULL
 *    
 *    mimeType:
 *      The MIME type of the associated content; may be NULL
 *    
 *    expectedContentLength:
 *      The expected length of the associated content; use
 *      kCFURLResponseExpectedContentLengthUnknown if the the length is
 *      not known.
 *    
 *    textEncodingName:
 *      If the MIME type is a text type, the name of the expected text
 *      encoding.  May be NULL.
 *    
 *    recommendedPolicy:
 *      The cache storage policy recommended for the response
 *  
 *  Result:
 *    The newly created CFURLResponse object
 *  
 */
APPLE_WIN_API CFURLResponseRef 
CFURLResponseCreate(
  CFAllocatorRef            alloc,
  CFURLRef                  URL,
  CFStringRef               mimeType,
  SInt64                    expectedContentLength,
  CFStringRef               textEncodingName,
  CFURLCacheStoragePolicy   recommendedPolicy)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLResponseCreateWithHTTPResponse()
 *  
 *  Discussion:
 *    Create a new response based on an HTTP response; interesting
 *    values like the expected content length, MIME type, etc. will be
 *    automatically calculated from the HTTP headers
 *  
 *  Mac OS X threading:
 *    Thread safe
 *  
 *  Parameters:
 *    
 *    alloc:
 *      The allocator to use to create the new object
 *    
 *    URL:
 *      The URL requested that generated this response; must not be NULL
 *    
 *    httpResponse:
 *      The HTTP response to use
 *    
 *    recommendedPolicy:
 *      The cache storage policy recommended for the response
 *  
 *  Result:
 *    The newly created CFURLResponse object
 *  
 */
APPLE_WIN_API CFURLResponseRef 
CFURLResponseCreateWithHTTPResponse(
  CFAllocatorRef            alloc,
  CFURLRef                  URL,
  CFHTTPMessageRef          httpResponse,
  CFURLCacheStoragePolicy   recommendedPolicy)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLResponseCreateCopy()
 *  
 */
APPLE_WIN_API CFURLResponseRef 
CFURLResponseCreateCopy(
  CFAllocatorRef     alloc,
  CFURLResponseRef   origResponse)                            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/* Simple accessors */
/*
 *  CFURLResponseGetURL()
 *  
 */
APPLE_WIN_API CFURLRef 
CFURLResponseGetURL(CFURLResponseRef response)                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLResponseGetMIMEType()
 *  
 */
APPLE_WIN_API CFStringRef 
CFURLResponseGetMIMEType(CFURLResponseRef response)           AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLResponseGetExpectedContentLength()
 *  
 */
APPLE_WIN_API SInt64 
CFURLResponseGetExpectedContentLength(CFURLResponseRef response) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLResponseGetTextEncodingName()
 *  
 */
APPLE_WIN_API CFStringRef 
CFURLResponseGetTextEncodingName(CFURLResponseRef response)   AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLResponseGetRecommendedCachePolicy()
 *  
 */
APPLE_WIN_API CFURLCacheStoragePolicy 
CFURLResponseGetRecommendedCachePolicy(CFURLResponseRef response) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/* Returns the absolute time at which the response was created; important for expiry calculations*/
/*
 *  CFURLResponseGetCreationTime()
 *  
 */
APPLE_WIN_API CFAbsoluteTime 
CFURLResponseGetCreationTime(CFURLResponseRef response)       AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
   Returns the expected expiration time.  Entries that should never be cached will return an expiration
   time in the past; entries that never expire will return an expiration time one year from the present
*/
/*
 *  CFURLResponseGetExpirationTime()
 *  
 */
APPLE_WIN_API CFAbsoluteTime 
CFURLResponseGetExpirationTime(CFURLResponseRef response)     AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/* The creation time unless something indicates otherwise (like the HTTP headers)*/
/*
 *  CFURLResponseGetLastModifiedDate()
 *  
 */
APPLE_WIN_API CFAbsoluteTime 
CFURLResponseGetLastModifiedDate(CFURLResponseRef response)   AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;

/*
 *  CFURLResponseCopySuggestedFilename()
 *  
 */
APPLE_WIN_API CFStringRef 
CFURLResponseCopySuggestedFilename(CFURLResponseRef response) AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/* Caller should not mutate the returned CFHTTPMessageRef*/
/*
 *  CFURLResponseGetHTTPResponse()
 *  
 */
APPLE_WIN_API CFHTTPMessageRef 
CFURLResponseGetHTTPResponse(CFURLResponseRef response)       AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/* Mutation - hopefully these do not need to become public?*/
/*
 *  CFURLResponseSetExpectedContentLength()
 *  
 */
APPLE_WIN_API void 
CFURLResponseSetExpectedContentLength(
  CFURLResponseRef   response,
  SInt64             length)                                  AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLResponseSetMIMEType()
 *  
 */
APPLE_WIN_API void 
CFURLResponseSetMIMEType(
  CFURLResponseRef   response,
  CFStringRef        mimeType)                                AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLResponseSetExpiration()
 *  
 */
APPLE_WIN_API void 
CFURLResponseSetExpiration(
  CFURLResponseRef   response,
  CFAbsoluteTime     expiration)                              AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  CFURLResponseSetRecommendedCachePolicy()
 *  
 */
APPLE_WIN_API void 
CFURLResponseSetRecommendedCachePolicy(
  CFURLResponseRef          response,
  CFURLCacheStoragePolicy   policy)                           AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;


/*
 *  _CFURLResponseSetCreationTime()
 *  
 */
APPLE_WIN_API void 
_CFURLResponseSetCreationTime(
  CFURLResponseRef   response,
  CFAbsoluteTime     creationTime)                            AVAILABLE_MAC_OS_X_VERSION_10_5_AND_LATER;




#if PRAGMA_ENUM_ALWAYSINT
    #pragma enumsalwaysint reset
#endif

#ifdef __cplusplus
}
#endif

#endif /* __CFURLRESPONSEPRIV__ */

