/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IURI..h
 * @brief		Define interface of URILoader IURI
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.05	v1.0		Created
 */
#ifndef __IURI_Defined
#define __IURI_Defined

#include "URILoader.h"

namespace URILoader
{

/**
 * URIs are essentially structured names for things -- anything. This interface
 * provides accessors to set and query the most basic components of an URI.
 * Subclasses, including nsIURL, impose greater structure on the URI.
 *
 * This interface follows Tim Berners-Lee's URI spec (RFC2396) [1], where the
 * basic URI components are defined as such:
 * <pre> 
 *      ftp://username:password@hostname:portnumber/pathname
 *      \ /   \               / \      / \        /\       /
 *       -     ---------------   ------   --------  -------
 *       |            |             |        |         |
 *       |            |             |        |        Path
 *       |            |             |       Port         
 *       |            |            Host      /
 *       |         UserPass                 /
 *     Scheme                              /
 *       \                                /
 *        --------------------------------
 *                       |
 *                    PrePath
 * </pre>
 * The definition of the URI components has been extended to allow for
 * internationalized domain names [2] and the more generic IRI structure [3].
 *
 * [1] http://www.ietf.org/rfc/rfc2396.txt
 * [2] http://www.ietf.org/internet-drafts/draft-ietf-idn-idna-06.txt
 * [3] http://www.ietf.org/internet-drafts/draft-masinter-url-i18n-08.txt
 */

class IURI : public IUnknown
{
public:


	/************************************************************************
	 * The URI is broken down into the following principal components:
	 */

	/**
	 * Returns a string representation of the URI. Setting the spec causes
	 * the new spec to be parsed, initializing the URI.
	 *
	 * Some characters may be escaped.
	 */
	//attribute AUTF8String spec;
	virtual Result GetSpec( String *pValue ) = 0;
	virtual Result SetSpec( String *aValue ) = 0;


	/**
	 * The prePath (eg. scheme://user:password@host:port) returns the string
	 * before the path.  This is useful for authentication or managing sessions.
	 *
	 * Some characters may be escaped.
	 */
	//readonly attribute AUTF8String prePath;
	//virtual Result GetPrePath( String *pValue ) = 0;
	//virtual Result SetPrePath( String *aValue ) = 0;

	/**
	 * The Scheme is the protocol to which this URI refers.  The scheme is
	 * restricted to the US-ASCII charset per RFC2396.
	 */
	//attribute ACString scheme;
	virtual Result GetScheme( String *pValue ) = 0;
	virtual Result SetScheme( String *aValue ) = 0;

	/**
	 * The username:password (or username only if value doesn't contain a ':')
	 *
	 * Some characters may be escaped.
	 */
	//attribute AUTF8String userPass;
	

	/**
	 * The optional username and password, assuming the preHost consists of
	 * username:password.
	 *
	 * Some characters may be escaped.
	 */
	//attribute AUTF8String username;
	//attribute AUTF8String password;
	virtual Result GetUsername( String *pValue ) = 0;
	virtual Result SetUsername( String *aValue ) = 0;

	virtual Result GetPassword( String *pValue ) = 0;
	virtual Result SetPassword( String *aValue ) = 0;

	/**
	 * The host:port (or simply the host, if port == -1).
	 *
	 * Characters are NOT escaped.
	 */
	//attribute AUTF8String hostPort;
	//virtual Result GetHostPort( String *pValue ) = 0;
	//virtual Result SetHostPort( String *aValue ) = 0;


	/**
	 * The host is the internet domain name to which this URI refers.  It could
	 * be an IPv4 (or IPv6) address literal.  If supported, it could be a
	 * non-ASCII internationalized domain name.
	 *
	 * Characters are NOT escaped.
	 */
	//attribute AUTF8String host;
	virtual Result GetHost( String *pValue ) = 0;
	virtual Result SetHost( String *aValue ) = 0;

	/**
	 * A port value of -1 corresponds to the protocol's default port (eg. -1
	 * implies port 80 for http URIs).
	 */
	//attribute long port;
	virtual Result GetPort( long *pValue ) = 0;
	virtual Result SetPort( long aValue ) = 0;
	

	/**
	 * The path, typically including at least a leading '/' (but may also be
	 * empty, depending on the protocol).
	 *
	 * Some characters may be escaped.
	 */
	//attribute AUTF8String path;
	virtual Result GetPath( String *pValue ) = 0;
	virtual Result SetPath( String *aValue ) = 0;


	/************************************************************************
	 * An URI supports the following methods:
	 */

	/**
	 * URI equivalence test (not a strict string comparison).
	 *
	 * eg. http://foo.com:80/ == http://foo.com/
	 */
	virtual Result Equals( IURI *other, BOOL *pEquals) = 0;

	/**
	 * An optimization to do scheme checks without requiring the users of nsIURI
	 * to GetScheme, thereby saving extra allocating and freeing. Returns true if
	 * the schemes match (case ignored).
	 */
	virtual Result SchemeIs( String *scheme, BOOL *pEquals ) = 0;

	/**
	 * Clones the current URI.  For some protocols, this is more than just an
	 * optimization.  For example, under MacOS, the spec of a file URL does not
	 * necessarily uniquely identify a file since two volumes could share the
	 * same name.
	 */
	//nsIURI clone();

	/**
	 * This method resolves a relative string into an absolute URI string,
	 * using this URI as the base. 
	 *
	 * NOTE: some implementations may have no concept of a relative URI.
	 */
	virtual Result Resolve( String *relativePath, String *pAbsoluteURI )= 0;


	/************************************************************************
	 * Additional attributes:
	 */

	/**
	 * The URI spec with an ASCII compatible encoding.  Host portion follows
	 * the IDNA draft spec.  Other parts are URL-escaped per the rules of
	 * RFC2396.  The result is strictly ASCII.
	 */
	//readonly attribute ACString asciiSpec;

	/**
	 * The URI host with an ASCII compatible encoding.  Follows the IDNA
	 * draft spec for converting internationalized domain names (UTF-8) to
	 * ASCII for compatibility with existing internet infrasture.
	 */
	//readonly attribute ACString asciiHost;

	/**
	 * The charset of the document from which this URI originated.  An empty
	 * value implies UTF-8.
	 *
	 * If this value is something other than UTF-8 then the URI components
	 * (e.g., spec, prePath, username, etc.) will all be fully URL-escaped.
	 * Otherwise, the URI components may contain unescaped multibyte UTF-8
	 * characters.
	 */
	//readonly attribute ACString originCharset;
};


/**
* Factory funtion to create IURI
*
*/
Result CreateIURI( char *szURI, IURI **ppURI ) ;


} // namespace 

#endif //__IURI_Defined
