/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved. 
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		IContentsListener.h
 * @brief		Define interface IContentsListener of WebControl
 * @author	Chaehwan Li (chaehwan.li@samsung.com)
 * @version	1.0
 * @date		2008.03.14	v1.0		Created
 */

#ifndef __IContentsListener_Defined
#define __IContentsListener_Defined

#include "WebDefs.h"
#include "IContentsSender.h"

namespace	WebControl
{

class IControl;

class IContentsListener
{
public:
	/*
	 * @desc listner function called when notification is requested by contents from browser engine
	 * @param in_pszTitle pointer to title
	 * @param in_pszMessage pointer to message
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT notify( char* in_pszTitle, char* in_pszMessage, IContentsSender *pSender ) = 0;

	/*
	 * @desc listner function called when Confirm is requested from browser engine
	 * @param in_pszTitle : pointer to Title of Confirm
	 * @param in_pszMessage : pointer to Message of confirm
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT confirm( ConfirmType in_ConfirmType, char* in_pszTitle, char* in_pszMessage, IContentsSender *pSender, void* pData = NULL ) = 0;

	/*
	 * @desc listner function called when prompt is requested from browser engine
	 * @param in_pszTitle : pointer to Title of prompt
	 * @param in_pszMessage : pointer to Message of prompt
	 * @param in_pszDefaultValue : pointer to Deafault Value of prompt
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT prompt( char* in_pszTitle, char* in_pszMessage, char* in_pszDefaultValue, IContentsSender *pSender ) = 0;

	/*
	 * @desc listner function called when alert is requested from browser engine
	 * @param in_pszTitle : pointer to Title of alert
	 * @param in_pszMessage : pointer to Message of alert
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT alert( char* in_pszTitle, char* in_pszMessage, IContentsSender *pSender ) = 0;

	/*
	 * @desc listner function called when authentication is requested by contents from browser engine
	 * @param in_pszRealm : pointer to Title of alert
	 * @param in_pszID : pointer to Message of alert
	 * @param in_pszPass pointer to sender function
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT auth( char* in_pszRealm, char* pszUrl,char* in_pszID, char* in_pszPass, IContentsSender *pSender ) = 0;

	/*
	 * @desc listner function called when redirection is requested by contents from browser engine
	 * @param in_pszChar : pointer to Message of alert
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT redirect( char* in_pszUrl, IContentsSender *pSender) = 0;

	/*
	 * @desc listner function called when cookie operation is requested by contents from browser engine
	 * @param in_pszID : pointer to Message of alert
	 * @param in_pszPass pointer to sender function
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT cookie( char* in_pszTitle, char* in_pszMessage, IContentsSender *pSender, void* ) = 0;

	/*
	 * @desc : listner function called when url tha has unknown scheme is requested by contents from browser engine
	 * @param in_pszID : pointer to Message of alert
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT exScheme( char* in_pszScheme, IContentsSender *pSender ) = 0;

	/*
	 * @desc listner function called when recives error for contents from browser engine
	 * @param in_State : state code
	 * @param in_progress if the state is progress ( includin formatting ) this param is valid.
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT state( int in_State, int in_progress, IContentsSender *pSender ) = 0;

	/*
	* @desc called when error occurs during loding page
	*/
	virtual HRESULT error( ErrorCode in_ErrCode, char* in_pszRequestUrl, IContentsSender *pSender ) = 0;

	/*
	 * @desc listner function called when Post request is made from browser engine to initialize
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT uploadInit( IContentsSender *pSender ) = 0;
	/*
	 * @desc listner function called when recives error for contents from browser engine
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT upload( IContentsSender *pSender ) = 0;

	/*
	 * @desc listner function called when recives uploadstate for contents from browser engine
	 * @param in_nTotal : total size to upload
	 * @param in_nSentSize : sent size
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT uploadState( int in_nTotal, int in_nSentSize, IContentsSender *pSender ) = 0;

	/*
	 * @desc listner function called when needs to confirm about forward from browser engine
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT forwardHistory(IContentsSender *pSender ) = 0;

	/*
	 * @desc listner function called when needs to confirm about backward from browser engine
	 * @param in_pfSender pointer to sender function
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT backwardHistory( IContentsSender *pSender ) = 0;

	/*
	 * @desc called when WebControl want to check if the specific area can be shown to screen
	 * @return HRESULT_TRUE if the area can be shown, otherwise HRESULT_FALSE
	 */
	virtual HRESULT checkShowState( int x, int y, int w, int h ) = 0;

	/*
	 * @desc called when WenControl need to update screen
	 * so client must decide whether to uppdate screen or not
	 * If client decide to update screen, client should call Webcontrol->showScreen
	 */
	virtual HRESULT updateScreen( int x, int y, int w, int h ) = 0;

	/*
	 * @desc listner function called when needs to open file store from browser engine
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT openFileStore() = 0;
      /*
	 * @desc listner function called when file name with invalid charaters are opened.
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT DisplayUnsupportedFileError() = 0;

	/*
	 * @desc listner function called when unexpected error occurs.
	 * @return HRESULT TRUE / FALSE
	 */

	virtual HRESULT displayErrorMessage() = 0;

	/*
	 * @desc These functions below are called when Widget page has js extension
	 * @return HRESULT TRUE / FALSE
	 */
	virtual HRESULT SetPreferenceForKey( char* pValue, char* pKey ) = 0; 
	virtual HRESULT PreferenceForKey( char* pKey, char** pOut_Data) = 0; 
	virtual HRESULT MaximizeWindow( char* pUrl ) = 0; 
	virtual HRESULT ResizeWindow( int nWidth, int nHeight ) = 0; 
	virtual const char* GetLanguage() = 0;
	virtual BOOL GetNetwork() = 0; 
	virtual BOOL SetScroll( bool bScroll ) = 0;
	virtual const char* GetMccMnc() = 0;

	virtual IControl* createWebControl() = 0;
	virtual void closeWebControl() = 0;
	virtual void showWebControl() = 0;

	virtual void willSendRequest( char* pSheme, char* pUrl, unsigned long reqId ) = 0;

	virtual void didFinishLoading( unsigned long reqId ) = 0;

	virtual void OpenUrl(char* url) = 0;
#ifdef SFB_UPDATE_URL_BAR  //Pramod - Bresson Model - update Url ComboBox bar
	/*
	 * @desc : to update the Url for the combo box url bar
	 * @param szUrl : Url to be set in BrAppState
	 * @return HRESULT TRUE / FALSE
	 */
	virtual void SetLinkInBrAppState(char* szUrl) = 0;
#endif


};

} // namespace WebControl

#endif // __IContentsListener_Defined