/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebFrame.cpp
 * @brief		implement WebFrame of WebControl
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.18	v1.0		Created
 */

// WebControl headers
#include "WebFrame.h"
#include "WebControl.h"
#include "WebFrameLoaderClient.h"

// WebCore headers
#include "FrameView.h"
#include "FrameTree.h"
#include "HTMLNames.h"
#include "HTMLFrameOwnerElement.h"
#include "IFrameView.h"
#include "WebKitBridge.h"

namespace WebControl {

WebFrame::WebFrame()
{
	m_pFrame = NULL;
	m_pWebControl = NULL;
	m_pClient = NULL;
	m_bIsMainFrame = false;
}

WebFrame::~WebFrame()
{
	Destruct();
}

HRESULT
WebFrame::New(WebControl* pWebControl, WebFrame **ppWebFrame )
{
	HRESULT hResult;
	
	WebFrame *pWebFrame = new WebFrame();

	if( pWebFrame == NULL )
		return HRESULT_NO_MEMORY;

	hResult = pWebFrame->Construct( pWebControl );
	if( HRESULT_ISFAIL(hResult))
	{
		delete pWebFrame;
		return hResult;
	}
	else
	{
		*ppWebFrame = pWebFrame;
		return hResult;
	}
}


HRESULT
WebFrame::Construct( WebControl* pWebControl )
{
	HRESULT hResult;

	// create WebControl::WebFrameLoaderClient
	m_pClient = new WebFrameLoaderClient( this );
	if( m_pClient == NULL )
		return HRESULT_NO_MEMORY;

    // Create the client bridge
    WebKitBridge* clientBridge = new WebKitBridge();

    // The client bridge will be stored and deleted in WebCoreBridge
	// create WebCore::Frame and WebCore::FrameView
	m_pFrame = new WebCore::Frame( pWebControl->GetPage(), 0, m_pClient, clientBridge);
	if( m_pFrame == NULL )
		return HRESULT_NO_MEMORY;
	WebCore::FrameView* pFrameView = new WebCore::FrameView( m_pFrame );
	if( pFrameView == NULL )
		return HRESULT_NO_MEMORY;

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod - Multiple instance support
	m_pFrame->setWebControlInstance((void*)pWebControl); 
#endif
	m_pFrame->setView( pFrameView );
	m_pFrame->init();
	m_pWebControl = pWebControl;

	return HRESULT_OK;
}


HRESULT WebFrame::New( WebControl* pWebControl, WebCore::HTMLFrameOwnerElement* pElement, WebFrame **ppWebFrame )
{
	HRESULT hResult;
	
	WebFrame *pWebFrame = new WebFrame();

	if( pWebFrame == NULL )
		return HRESULT_NO_MEMORY;

	hResult = pWebFrame->Construct( pWebControl, pElement  );
	if( HRESULT_ISFAIL(hResult))
	{
		delete pWebFrame;
		return hResult;
	}
	else
	{
		*ppWebFrame = pWebFrame;
		return hResult;
	}
}


HRESULT WebFrame::Construct( WebControl* pWebControl, WebCore::HTMLFrameOwnerElement* pElement )
{
	HRESULT hResult;

	// create WebControl::WebFrameLoaderClient
	m_pClient = new WebFrameLoaderClient( this );
	if( m_pClient == NULL )
		return HRESULT_NO_MEMORY;

    // Create the client bridge
    WebKitBridge* clientBridge = new WebKitBridge();
    // The client bridge will be stored and deleted in WebCoreBridge
	// create WebCore::Frame and WebCore::FrameView
	m_pFrame = new WebCore::Frame( pWebControl->GetPage(), pElement, m_pClient, clientBridge);
	if( m_pFrame == NULL )
		return HRESULT_NO_MEMORY;

	WebCore::FrameView* pFrameView;

	if(pElement->hasTagName( WebCore::HTMLNames::iframeTag))
	{
		pFrameView = new WebCore::IFrameView( m_pFrame, pWebControl->GetHWIN() );
	}
	else
	{
		pFrameView = new WebCore::FrameView( m_pFrame );
	}

	if( pFrameView == NULL )
		return HRESULT_NO_MEMORY;

#ifdef MULTIPLE_INSTANCE_SUPPORT //Pramod - Multiple instance support (BUg Id 2283)
	m_pFrame->setWebControlInstance((void*)pWebControl);
#endif
	m_pFrame->setView( pFrameView );
	
	/* http://trac.webkit.org/changeset/35088 */
	if(pElement)
		pElement->document()->frame()->tree()->appendChild(m_pFrame);

	m_pFrame->init();
	m_pWebControl = pWebControl;

	return HRESULT_OK;

}



HRESULT
WebFrame::Destruct()
{
	// [todo]
	// need to check if m_pFrame, m_pClient must be deleted
	return HRESULT_OK;
}

} // namespace WebFrame