/*
 * Copyright (c) S/W Platform, Samsung Electronics, Inc.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Samsung Electronics, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Samsung Electronics. 
 */
/** 
 * @file		WebPreferences.cpp
 * @brief		implement interface ISetting of WebControl
 * @author	Sangheon Kim (sangheon77.kim@samsung.com)
 * @version	1.0
 * @date		2008.03.18	v1.0		Created
 */

#include "WebPreferences.h"
#ifndef STLPORT_ENABLED
#include <algorithm>
#endif

namespace WebControl {

static WebPreferences* pDefaultPreferences = NULL;


WebPreferences* WebPreferences::GetDefaultPreferences()
{
//	static WebPreferences* pDefaultPreferences = NULL;
	if( pDefaultPreferences == NULL )
	{
		HRESULT hResult;
		
		hResult = WebPreferences::New( "WebControlGlobalPreferences", &pDefaultPreferences );
		if( HRESULT_ISFAIL(hResult) )
			return NULL;
		else
			return pDefaultPreferences;
	}
	else
		return pDefaultPreferences;
}

HRESULT WebPreferences::New( String name, WebPreferences** ppWebPreferences )
{
	HRESULT hResult;
	WebPreferences* pWebPreferences = NULL;

	pWebPreferences = new WebPreferences();
	if( pWebPreferences == NULL )
		return HRESULT_NO_MEMORY;

	hResult = pWebPreferences->Construct( name );
	if( HRESULT_ISFAIL(hResult) )
	{
			*ppWebPreferences = NULL;
			delete pWebPreferences;
			return hResult;
	}
	else
	{
		*ppWebPreferences = pWebPreferences;
		return HRESULT_OK;
	}
}

HRESULT WebPreferences::Construct( String name )
{
	m_strName = name;

	// load default settings
	return LoadDefaults();
}

WebPreferences::~WebPreferences()
{
	m_IntMap.clear();
	m_StrMap.clear();
	pDefaultPreferences = NULL;
}

HRESULT WebPreferences::Destruct()
{
	return HRESULT_OK;
}

HRESULT WebPreferences::LoadDefaults()
{
	// In the future, we will load the default setting from files or ini
	IntMapPair intPair;
	StrMapPair strPair;

	intPair.first = LOADS_IMAGE_AUTOMATICALLY;
	intPair.second = 1; 
	m_IntMap.insert( intPair );

	intPair.first = MINIMUM_FONT_SIZE;
	intPair.second = 1; 
	m_IntMap.insert( intPair );

	intPair.first = MINIMUM_LOGICAL_FONT_SIZE;
	intPair.second = 1; 
	m_IntMap.insert( intPair );

	intPair.first = SHOULD_PRINT_BACKGROUND;
	intPair.second = 1; 
	m_IntMap.insert( intPair );

	intPair.first = JAVASCRIPT_ENABLED;
	intPair.second = 1; 
	m_IntMap.insert( intPair );

	intPair.first = DEFAULT_FIXED_FONT_SIZE;
	intPair.second = 16; 
	m_IntMap.insert( intPair );

	intPair.first = DEFAULT_FONT_SIZE;
	intPair.second = 16; 
	m_IntMap.insert( intPair );

	intPair.first = PLUGIN_ENABLED;
	intPair.second = 1; 
	m_IntMap.insert( intPair );

	intPair.first = ENABLE_BG_SOUND;
	intPair.second = 1; 
	m_IntMap.insert( intPair );

	strPair.first = SERIF_FONT_FAMILY;
	strPair.second = "Times New Roman"; 
	m_StrMap.insert( strPair );


	strPair.first = SANSSERIF_FONT_FAMILY;
	strPair.second = "Arial"; 
	m_StrMap.insert( strPair );

	strPair.first = FIXED_FONT_FAMILY;
	strPair.second = "Courier"; 
	m_StrMap.insert( strPair );


	strPair.first = STANDARD_FONT_FAMILY;
	strPair.second = "Arial"; 
	m_StrMap.insert( strPair );

	strPair.first = HTTP_USERAGENT;
	strPair.second = "Mozilla/5.0 (X11; U; Linux i686; en-US) AppleWebKit/420+ (KHTML, like Gecko)"; 
	m_StrMap.insert( strPair );

	intPair.first = PAGE_OVERVIEW_ENABLED;
	intPair.second = 1; 
	m_IntMap.insert( intPair );

	return HRESULT_OK;

}



/*
 * @desc sets a property of Browser Engine.
 * @param in_PropertyID : Property ID to setting
 * @param in_PropertyValue : pointer to Property Value to setting
 * @return bool TRUE / FALSE
 */
HRESULT WebPreferences::setProperty( int in_PropertyID, void* in_PropertyValue  )
{
	if( in_PropertyID < PROPERTY_STR_BASE )
	{
		int *value = (int*)in_PropertyValue;
		IntMapPair pair( in_PropertyID, *value );
	
		// update
		// bs0425.lee  to find same Property ID .
	     if (m_IntMap.find(in_PropertyID) != m_IntMap.end())
		 {
			 m_IntMap[in_PropertyID]=*value;
		 }
		 else // insert
		 {
			if( !(m_IntMap.insert( pair ).second) )
				return HRESULT_NOK;
		 }
	}
	else
	{
		String *value = (String*)in_PropertyValue;
		StrMapPair pair( in_PropertyID, *value );
	
		// bs0425.lee  to find same Property ID . 
	     if (m_StrMap.find(in_PropertyID) != m_StrMap.end())
		 {
			 m_StrMap[in_PropertyID]= *value;
		 }
		 else
		 {
			if( !(m_StrMap.insert( pair ).second) )
				return HRESULT_NOK;
		 }
	}

	//  change notification 
	//if( bNotification )
	{

		for( int i = 0; i < m_listenerVector.size(); i++ )
		{
			ISettingListener* pListener = m_listenerVector.at(i);
			if( pListener )
				pListener->OnSettingChanged( this, in_PropertyID );

		}
	}


	return HRESULT_OK;


}

/*
 * @desc gets a property of Browser Engine.
 * @param in_PropertyID : Property ID to setting
 * @param out_PropertyValue : pointer to Property Value to setting
 * @return bool TRUE / FALSE
 */
HRESULT WebPreferences::getProperty( int in_PropertyID, void* out_PropertyValue )
{
	if( in_PropertyID < PROPERTY_STR_BASE )
	{
			IntMap::iterator pos = m_IntMap.find( in_PropertyID );

			if( pos == m_IntMap.end() )
			{
				return HRESULT_NOT_FOUND;
			}
			else
			{
				int *out = (int*)out_PropertyValue;
				*out = pos->second;
				return HRESULT_OK;
			}
	}
	else
	{
			StrMap::iterator pos = m_StrMap.find( in_PropertyID );

			if( pos == m_StrMap.end() )
			{
				return HRESULT_NOT_FOUND;
			}
			else
			{
				String *out = (String*)out_PropertyValue;
				*out = pos->second;
				return HRESULT_OK;
			}

	}

		return HRESULT_NOTSUPPORT;

}


/**
* Add a listener to the listener list
* All listeners in the list will be notified whenever the setting value change
*/
HRESULT WebPreferences::addListener( ISettingListener* pListener )
{
		ListenerVector::iterator pos;
		pos = std::find( m_listenerVector.begin(), m_listenerVector.end(), pListener );
		if( pos == m_listenerVector.end() )
		{
			// insert
			m_listenerVector.push_back( pListener );

			return HRESULT_OK;
		}
		else // already exist
			return HRESULT_INVALID_PARAM;
}

/**
* Remove the listener from the listener list
*/
HRESULT WebPreferences::removeListener( ISettingListener* pListener )
{
		ListenerVector::iterator pos;
		pos = std::find( m_listenerVector.begin(), m_listenerVector.end(), pListener );
		if( pos != m_listenerVector.end() )
		{
			// remove
			m_listenerVector.erase( pos );
			return HRESULT_OK;
		}
		else // not exist
			return HRESULT_INVALID_PARAM;
}



} // namespace WebControl